@@ -1,7 +1,7 @@
 //
 //  rbbiscan.h
 //
-//  Copyright (C) 2002-2005, International Business Machines Corporation and others.
+//  Copyright (C) 2002-2007, International Business Machines Corporation and others.
 //  All Rights Reserved.
 //
 //  This file contains declarations for class RBBIRuleScanner
@@ -46,10 +46,6 @@ static const int    kStackSize = 100;               // The size of the state sta
                                                     //   to the depth of parentheses nesting
                                                     //   that is allowed in the rules.
 
-enum EParseAction {dummy01, dummy02};               // Placeholder enum for the specifier for
-                                                    //   actions that are specified in the
-                                                    //   rule parsing state table.
-
 class RBBIRuleScanner : public UMemory {
 public:
 
@@ -81,7 +77,7 @@ public:
     static UnicodeString stripRules(const UnicodeString &rules);
 private:
 
-    UBool       doParseActions(EParseAction a);
+    UBool       doParseActions(int32_t a);
     void        error(UErrorCode e);                   // error reporting convenience function.
     void        fixOpStack(RBBINode::OpPrecedence p);
                                                        //   a character.
