@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ******************************************************************************
 *   Date        Name        Description
@@ -107,7 +107,7 @@ typedef struct UHashElement UHashElement;
  * @param key A key stored in a hashtable
  * @return A NON-NEGATIVE hash code for parm.
  */
-typedef int32_t (U_EXPORT2 * U_CALLCONV UHashFunction)(const UHashTok key);
+typedef int32_t U_CALLCONV UHashFunction(const UHashTok key);
 
 /**
  * A key comparison function.
@@ -115,16 +115,23 @@ typedef int32_t (U_EXPORT2 * U_CALLCONV UHashFunction)(const UHashTok key);
  * @param key2 A key stored in a hashtable
  * @return TRUE if the two keys are equal.
  */
-typedef UBool (U_EXPORT2 * U_CALLCONV UKeyComparator)(const UHashTok key1,
-                                            const UHashTok key2);
-
+typedef UBool U_CALLCONV UKeyComparator(const UHashTok key1,
+                                        const UHashTok key2);
+/**
+ * A key comparison function.
+ * @param val1 A key stored in a hashtable
+ * @param val2 A key stored in a hashtable
+ * @return TRUE if the two keys are equal.
+ */
+typedef UBool U_CALLCONV UValueComparator(const UHashTok val1,
+                                          const UHashTok val2);
 /**
  * A function called by <TT>uhash_remove</TT>,
  * <TT>uhash_close</TT>, or <TT>uhash_put</TT> to delete
  * an existing key or value.
  * @param obj A key or value stored in a hashtable
  */
-typedef void (U_EXPORT2 * U_CALLCONV UObjectDeleter)(void* obj);
+typedef void U_CALLCONV UObjectDeleter(void* obj);
 
 /**
  * This specifies whether or not, and how, the hastable resizes itself.
@@ -146,6 +153,19 @@ struct UHashtable {
 
     UHashElement *elements;
 
+    /* Function pointers */
+
+    UHashFunction *keyHasher;      /* Computes hash from key.
+                                   * Never null. */
+    UKeyComparator *keyComparator; /* Compares keys for equality.
+                                   * Never null. */
+    UValueComparator *valueComparator; /* Compares the values for equality */
+
+    UObjectDeleter *keyDeleter;    /* Deletes keys when required.
+                                   * If NULL won't do anything */
+    UObjectDeleter *valueDeleter;  /* Deletes values when required.
+                                   * If NULL won't do anything */
+
     /* Size parameters */
   
     int32_t     count;      /* The number of key-value pairs in this table.
@@ -163,16 +183,7 @@ struct UHashtable {
     float       highWaterRatio; /* 0..1; high water as a fraction of length */
     float       lowWaterRatio;  /* 0..1; low water as a fraction of length */
     
-    /* Function pointers */
-
-    UHashFunction keyHasher;      /* Computes hash from key.
-                                   * Never null. */
-    UKeyComparator keyComparator; /* Compares keys for equality.
-                                   * Never null. */
-    UObjectDeleter keyDeleter;    /* Deletes keys when required.
-                                   * If NULL won't do anything */
-    UObjectDeleter valueDeleter;  /* Deletes values when required.
-                                   * If NULL won't do anything */
+    UBool       allocated; /* Was this UHashtable allocated? */
 };
 typedef struct UHashtable UHashtable;
 
@@ -193,8 +204,9 @@ U_CDECL_END
  * @see uhash_openSize
  */
 U_CAPI UHashtable* U_EXPORT2 
-uhash_open(UHashFunction keyHash,
-           UKeyComparator keyComp,
+uhash_open(UHashFunction *keyHash,
+           UKeyComparator *keyComp,
+           UValueComparator *valueComp,
            UErrorCode *status);
 
 /**
@@ -209,11 +221,29 @@ uhash_open(UHashFunction keyHash,
  * @see uhash_open
  */
 U_CAPI UHashtable* U_EXPORT2 
-uhash_openSize(UHashFunction keyHash,
-               UKeyComparator keyComp,
+uhash_openSize(UHashFunction *keyHash,
+               UKeyComparator *keyComp,
+               UValueComparator *valueComp,
                int32_t size,
                UErrorCode *status);
 
+/**
+ * Initialize an existing UHashtable.
+ * @param keyHash A pointer to the key hashing function.  Must not be
+ * NULL.
+ * @param keyComp A pointer to the function that compares keys.  Must
+ * not be NULL.
+ * @param status A pointer to an UErrorCode to receive any errors.
+ * @return A pointer to a UHashtable, or 0 if an error occurred.
+ * @see uhash_openSize
+ */
+U_CAPI UHashtable* U_EXPORT2 
+uhash_init(UHashtable *hash,
+           UHashFunction *keyHash,
+           UKeyComparator *keyComp,
+           UValueComparator *valueComp,
+           UErrorCode *status);
+
 /**
  * Close a UHashtable, releasing the memory used.
  * @param hash The UHashtable to close.
@@ -225,20 +255,32 @@ uhash_close(UHashtable *hash);
 
 /**
  * Set the function used to hash keys.
+ * @param hash The UHashtable to set
  * @param fn the function to be used hash keys; must not be NULL
  * @return the previous key hasher; non-NULL
  */
-U_CAPI UHashFunction U_EXPORT2 
-uhash_setKeyHasher(UHashtable *hash, UHashFunction fn);
+U_CAPI UHashFunction *U_EXPORT2 
+uhash_setKeyHasher(UHashtable *hash, UHashFunction *fn);
 
 /**
  * Set the function used to compare keys.  The default comparison is a
  * void* pointer comparison.
+ * @param hash The UHashtable to set
  * @param fn the function to be used compare keys; must not be NULL
  * @return the previous key comparator; non-NULL
  */
-U_CAPI UKeyComparator U_EXPORT2 
-uhash_setKeyComparator(UHashtable *hash, UKeyComparator fn);
+U_CAPI UKeyComparator *U_EXPORT2 
+uhash_setKeyComparator(UHashtable *hash, UKeyComparator *fn);
+
+/**
+ * Set the function used to compare values.  The default comparison is a
+ * void* pointer comparison.
+ * @param hash The UHashtable to set
+ * @param fn the function to be used compare keys; must not be NULL
+ * @return the previous key comparator; non-NULL
+ */
+U_CAPI UValueComparator *U_EXPORT2 
+uhash_setValueComparator(UHashtable *hash, UValueComparator *fn);
 
 /**
  * Set the function used to delete keys.  If this function pointer is
@@ -246,11 +288,12 @@ uhash_setKeyComparator(UHashtable *hash, UKeyComparator fn);
  * hashtable does delete keys.  This function should be set once
  * before any elements are added to the hashtable and should not be
  * changed thereafter.
+ * @param hash The UHashtable to set
  * @param fn the function to be used delete keys, or NULL
  * @return the previous key deleter; may be NULL
  */
-U_CAPI UObjectDeleter U_EXPORT2 
-uhash_setKeyDeleter(UHashtable *hash, UObjectDeleter fn);
+U_CAPI UObjectDeleter *U_EXPORT2 
+uhash_setKeyDeleter(UHashtable *hash, UObjectDeleter *fn);
 
 /**
  * Set the function used to delete values.  If this function pointer
@@ -258,16 +301,19 @@ uhash_setKeyDeleter(UHashtable *hash, UObjectDeleter fn);
  * this hashtable does delete values.  This function should be set
  * once before any elements are added to the hashtable and should not
  * be changed thereafter.
+ * @param hash The UHashtable to set
  * @param fn the function to be used delete values, or NULL
  * @return the previous value deleter; may be NULL
  */
-U_CAPI UObjectDeleter U_EXPORT2 
-uhash_setValueDeleter(UHashtable *hash, UObjectDeleter fn);
+U_CAPI UObjectDeleter *U_EXPORT2 
+uhash_setValueDeleter(UHashtable *hash, UObjectDeleter *fn);
 
 /**
  * Specify whether or not, and how, the hastable resizes itself.
  * By default, tables grow but do not shrink (policy U_GROW).
  * See enum UHashResizePolicy.
+ * @param hash The UHashtable to set
+ * @param policy The way the hashtable resizes itself, {U_GROW, U_GROW_AND_SHRINK, U_FIXED}
  */
 U_CAPI void U_EXPORT2 
 uhash_setResizePolicy(UHashtable *hash, enum UHashResizePolicy policy);
@@ -335,6 +381,24 @@ uhash_puti(UHashtable *hash,
            int32_t value,
            UErrorCode *status);
 
+/**
+ * Put a (key=integer, value=integer) item in a UHashtable.  If the
+ * keyDeleter is non-NULL, then the hashtable owns 'key' after this
+ * call.  valueDeleter must be NULL.  Storing a 0 value is the same as
+ * calling uhash_remove().
+ * @param hash The target UHashtable.
+ * @param key The key to store.
+ * @param value The integer value to store.
+ * @param status A pointer to an UErrorCode to receive any errors.
+ * @return The previous value, or 0 if none.
+ * @see uhash_get
+ */
+U_CAPI int32_t U_EXPORT2 
+uhash_iputi(UHashtable *hash,
+           int32_t key,
+           int32_t value,
+           UErrorCode *status);
+
 /**
  * Retrieve a pointer value from a UHashtable using a pointer key,
  * as previously stored by uhash_put().
@@ -367,6 +431,16 @@ uhash_iget(const UHashtable *hash,
 U_CAPI int32_t U_EXPORT2 
 uhash_geti(const UHashtable *hash,
            const void* key);
+/**
+ * Retrieve an integer value from a UHashtable using an integer key,
+ * as previously stored by uhash_iputi().
+ * @param hash The target UHashtable.
+ * @param key An integer key stored in a hashtable
+ * @return The requested item, or 0 if not found.
+ */
+U_CAPI int32_t U_EXPORT2 
+uhash_igeti(const UHashtable *hash,
+           int32_t key);
 
 /**
  * Remove an item from a UHashtable stored by uhash_put().
@@ -398,6 +472,16 @@ U_CAPI int32_t U_EXPORT2
 uhash_removei(UHashtable *hash,
               const void* key);
 
+/**
+ * Remove an item from a UHashtable stored by uhash_iputi().
+ * @param hash The target UHashtable.
+ * @param key An integer key stored in a hashtable
+ * @return The item removed, or 0 if not found.
+ */
+U_CAPI int32_t U_EXPORT2 
+uhash_iremovei(UHashtable *hash,
+               int32_t key);
+
 /**
  * Remove all items from a UHashtable.
  * @param hash The target UHashtable.
@@ -456,15 +540,19 @@ uhash_removeElement(UHashtable *hash, const UHashElement* e);
 
 /**
  * Return a UHashTok for an integer.
+ * @param i The given integer
+ * @return a UHashTok for an integer.
  */
-U_CAPI UHashTok U_EXPORT2 
-uhash_toki(int32_t i);
+/*U_CAPI UHashTok U_EXPORT2 
+uhash_toki(int32_t i);*/
 
 /**
  * Return a UHashTok for a pointer.
+ * @param p The given pointer
+ * @return a UHashTok for a pointer.
  */
-U_CAPI UHashTok U_EXPORT2 
-uhash_tokp(void* p);
+/*U_CAPI UHashTok U_EXPORT2 
+uhash_tokp(void* p);*/
 
 /********************************************************************
  * UChar* and char* Support Functions
@@ -507,6 +595,9 @@ uhash_hashIChars(const UHashTok key);
 /**
  * Comparator for null-terminated UChar* strings.  Use together with
  * uhash_hashUChars.
+ * @param key1 The string for comparison
+ * @param key2 The string for comparison
+ * @return true if key1 and key2 are equal, return false otherwise.
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareUChars(const UHashTok key1, const UHashTok key2);
@@ -514,6 +605,9 @@ uhash_compareUChars(const UHashTok key1, const UHashTok key2);
 /**
  * Comparator for null-terminated char* strings.  Use together with
  * uhash_hashChars.
+ * @param key1 The string for comparison
+ * @param key2 The string for comparison
+ * @return true if key1 and key2 are equal, return false otherwise.
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareChars(const UHashTok key1, const UHashTok key2);
@@ -521,6 +615,9 @@ uhash_compareChars(const UHashTok key1, const UHashTok key2);
 /**
  * Case-insensitive comparator for null-terminated char* strings.  Use
  * together with uhash_hashIChars.
+ * @param key1 The string for comparison
+ * @param key2 The string for comparison
+ * @return true if key1 and key2 are equal, return false otherwise.
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareIChars(const UHashTok key1, const UHashTok key2);
@@ -531,6 +628,8 @@ uhash_compareIChars(const UHashTok key1, const UHashTok key2);
 
 /**
  * Hash function for UnicodeString* keys.
+ * @param key The string (const char*) to hash.
+ * @return A hash code for the key.
  */
 U_CAPI int32_t U_EXPORT2 
 uhash_hashUnicodeString(const UHashTok key);
@@ -538,12 +637,17 @@ uhash_hashUnicodeString(const UHashTok key);
 /**
  * Hash function for UnicodeString* keys (case insensitive).
  * Make sure to use together with uhash_compareCaselessUnicodeString.
+ * @param key The string (const char*) to hash.
+ * @return A hash code for the key.
  */
 U_CAPI int32_t U_EXPORT2 
 uhash_hashCaselessUnicodeString(const UHashTok key);
 
 /**
  * Comparator function for UnicodeString* keys.
+ * @param key1 The string for comparison
+ * @param key2 The string for comparison
+ * @return true if key1 and key2 are equal, return false otherwise.
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareUnicodeString(const UHashTok key1, const UHashTok key2);
@@ -551,12 +655,16 @@ uhash_compareUnicodeString(const UHashTok key1, const UHashTok key2);
 /**
  * Comparator function for UnicodeString* keys (case insensitive).
  * Make sure to use together with uhash_hashCaselessUnicodeString.
+ * @param key1 The string for comparison
+ * @param key2 The string for comparison
+ * @return true if key1 and key2 are equal, return false otherwise.
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareCaselessUnicodeString(const UHashTok key1, const UHashTok key2);
 
 /**
  * Deleter function for UnicodeString* keys or values.
+ * @param obj The object to be deleted
  */
 U_CAPI void U_EXPORT2 
 uhash_deleteUnicodeString(void *obj);
@@ -567,12 +675,17 @@ uhash_deleteUnicodeString(void *obj);
 
 /**
  * Hash function for 32-bit integer keys.
+ * @param key The string (const char*) to hash.
+ * @return A hash code for the key.
  */
 U_CAPI int32_t U_EXPORT2 
 uhash_hashLong(const UHashTok key);
 
 /**
  * Comparator function for 32-bit integer keys.
+ * @param key1 The integer for comparison
+ * @param Key2 The integer for comparison
+ * @return true if key1 and key2 are equal, return false otherwise
  */
 U_CAPI UBool U_EXPORT2 
 uhash_compareLong(const UHashTok key1, const UHashTok key2);
@@ -583,12 +696,14 @@ uhash_compareLong(const UHashTok key1, const UHashTok key2);
 
 /**
  * Deleter for Hashtable objects.
+ * @param obj The object to be deleted
  */
 U_CAPI void U_EXPORT2 
 uhash_deleteHashtable(void *obj);
 
 /**
  * Deleter for UVector objects.
+ * @param obj The object to be deleted
  */
 U_CAPI void U_EXPORT2 
 uhash_deleteUVector(void *obj);
@@ -596,8 +711,18 @@ uhash_deleteUVector(void *obj);
 /**
  * Deleter for any key or value allocated using uprv_malloc.  Calls
  * uprv_free.
+ * @param obj The object to be deleted
  */
 U_CAPI void U_EXPORT2 
 uhash_freeBlock(void *obj);
 
+/**
+ * Checks if the given hash tables are equal or not.
+ * @param hash1
+ * @param hash2
+ * @return true if the hashtables are equal and false if not.
+ */
+U_CAPI UBool U_EXPORT2 
+uhash_equals(const UHashtable* hash1, const UHashtable* hash2);
+
 #endif
