@@ -1,6 +1,6 @@
 /*
 ***************************************************************************
-*   Copyright (C) 1999-2005 International Business Machines Corporation   *
+*   Copyright (C) 1999-2007 International Business Machines Corporation   *
 *   and others. All rights reserved.                                      *
 ***************************************************************************
 
@@ -26,6 +26,8 @@
 #include "unicode/brkiter.h"
 #include "unicode/udata.h"
 #include "unicode/parseerr.h"
+#include "unicode/schriter.h"
+#include "unicode/uchriter.h"
 
 
 struct UTrie;
@@ -37,10 +39,14 @@ struct RBBIDataHeader;
 class  RuleBasedBreakIteratorTables;
 class  BreakIterator;
 class  RBBIDataWrapper;
+class  UStack;
+class  LanguageBreakEngine;
+class  UnhandledEngine;
 struct RBBIStateTable;
 
 
 
+
 /**
  *
  * A subclass of BreakIterator whose behavior is specified using a list of rules.
@@ -60,10 +66,31 @@ class U_COMMON_API RuleBasedBreakIterator : public BreakIterator {
 
 protected:
     /**
-     * The character iterator through which this BreakIterator accesses the text
+     * The UText through which this BreakIterator accesses the text
      * @internal
      */
-    CharacterIterator*  fText;
+    UText  *fText;
+
+    /**
+     *   A character iterator that refers to the same text as the UText, above.
+     *   Only included for compatibility with old API, which was based on CharacterIterators.
+     *   Value may be adopted from outside, or one of fSCharIter or fDCharIter, below.
+     */
+    CharacterIterator  *fCharIter;
+
+    /**
+     *   When the input text is provided by a UnicodeString, this will point to
+     *    a characterIterator that wraps that data.  Needed only for the
+     *    implementation of getText(), a backwards compatibility issue.
+     */
+    StringCharacterIterator *fSCharIter;
+
+    /**
+     *  When the input text is provided by a UText, this
+     *    dummy CharacterIterator over an empty string will
+     *    be returned from getText()
+     */
+    UCharCharacterIterator *fDCharIter;
 
     /**
      * The rule data for this BreakIterator instance
@@ -86,20 +113,58 @@ protected:
 
     /**
      * Counter for the number of characters encountered with the "dictionary"
-     *   flag set.  Normal RBBI iterators don't use it, although the code
-     *   for updating it is live.  Dictionary Based break iterators (a subclass
-     *   of us) access this field directly.
+     *   flag set.
      * @internal
      */
-    uint32_t           fDictionaryCharCount;
+    uint32_t            fDictionaryCharCount;
 
     /**
-     * Debugging flag.  Trace operation of state machine when true.
+     * When a range of characters is divided up using the dictionary, the break
+     * positions that are discovered are stored here, preventing us from having
+     * to use either the dictionary or the state table again until the iterator
+     * leaves this range of text. Has the most impact for line breaking.
      * @internal
      */
-    static UBool        fTrace;
+    int32_t*            fCachedBreakPositions;
 
+    /**
+     * The number of elements in fCachedBreakPositions
+     * @internal
+     */
+    int32_t             fNumCachedBreakPositions;
 
+    /**
+     * if fCachedBreakPositions is not null, this indicates which item in the
+     * cache the current iteration position refers to
+     * @internal
+     */
+    int32_t             fPositionInCache;
+    
+    /**
+     *
+     * If present, UStack of LanguageBreakEngine objects that might handle
+     * dictionary characters. Searched from top to bottom to find an object to
+     * handle a given character.
+     * @internal
+     */
+    UStack              *fLanguageBreakEngines;
+    
+    /**
+     *
+     * If present, the special LanguageBreakEngine used for handling
+     * characters that are in the dictionary set, but not handled by any
+     * LangugageBreakEngine.
+     * @internal
+     */
+    UnhandledEngine     *fUnhandledBreakEngine;
+    
+    /**
+     *
+     * The type of the break iterator, or -1 if it has not been set.
+     * @internal
+     */
+    int32_t             fBreakType;
+    
 protected:
     //=======================================================================
     // constructors
@@ -117,7 +182,7 @@ protected:
      */
     RuleBasedBreakIterator(RBBIDataHeader* data, UErrorCode &status);
 
-    /** @internal */
+
     friend class RBBIRuleBuilder;
     /** @internal */
     friend class BreakIterator;
@@ -232,14 +297,31 @@ public:
     //=======================================================================
 
     /**
-     * Return a CharacterIterator over the text being analyzed.  This version
-     * of this method returns the actual CharacterIterator we're using internally.
-     * Changing the state of this iterator can have undefined consequences.  If
-     * you need to change it, clone it first.
+     * <p>
+     * Return a CharacterIterator over the text being analyzed.
+     * The returned character iterator is owned by the break iterator, and must
+     * not be deleted by the caller.  Repeated calls to this function may
+     * return the same CharacterIterator.
+     * </p>
+     * <p>
+     * The returned character iterator must not be used concurrently with
+     * the break iterator.  If concurrent operation is needed, clone the
+     * returned character iterator first and operate on the clone.
+     * </p>
+     * <p>
+     * When the break iterator is operating on text supplied via a UText,
+     * this function will fail.  Lacking any way to signal failures, it
+     * returns an CharacterIterator containing no text.
+     * The function getUText() provides similar functionality,
+     * is reliable, and is more efficient.
+     * </p>
+     *
+     * TODO:  deprecate this function?
+     *
      * @return An iterator over the text being analyzed.
-     *  @stable ICU 2.0
+     * @stable ICU 2.0
      */
-    virtual const CharacterIterator& getText(void) const;
+    virtual  CharacterIterator& getText(void) const;
 
 
     /**
@@ -254,7 +336,7 @@ public:
       * @param status receives any error codes.
       * @return   The current UText for this break iterator.  If an input
       *           UText was provided, it will always be returned.
-      * @draft ICU 3.4
+      * @stable ICU 3.4
       */
      virtual UText *getUText(UText *fillIn, UErrorCode &status) const;
 
@@ -286,13 +368,12 @@ public:
      *
      * @param text    The UText used to change the text.
      * @param status  Receives any error codes.
-     * @draft ICU 3.4
+     * @stable ICU 3.4
      */
     virtual void  setText(UText *text, UErrorCode &status);
 
     /**
      * Sets the current iteration position to the beginning of the text.
-     * (i.e., the CharacterIterator's starting offset).
      * @return The offset of the beginning of the text.
      *  @stable ICU 2.0
      */
@@ -300,7 +381,6 @@ public:
 
     /**
      * Sets the current iteration position to the end of the text.
-     * (i.e., the CharacterIterator's ending offset).
      * @return The text's past-the-end offset.
      *  @stable ICU 2.0
      */
@@ -423,7 +503,7 @@ public:
     *                  is the total number of status values that were available,
     *                  not the reduced number that were actually returned.
     * @see getRuleStatus
-    * @draft ICU 3.0
+    * @stable ICU 3.0
     */
     virtual int32_t getRuleStatusVec(int32_t *fillInVec, int32_t capacity, UErrorCode &status);
 
@@ -506,34 +586,14 @@ protected:
     //=======================================================================
     // implementation
     //=======================================================================
-    /**
-     * This method is the actual implementation of the next() method.  All iteration
-     * vectors through here.  This method initializes the state machine to state 1
-     * and advances through the text character by character until we reach the end
-     * of the text or the state machine transitions to state 0.  We update our return
-     * value every time the state machine passes through a possible end state.
-     * @internal
-     */
-    virtual int32_t handleNext(void);
-
-    /**
-     * This method backs the iterator back up to a "safe position" in the text.
-     * This is a position that we know, without any context, must be a break position.
-     * The various calling methods then iterate forward from this safe position to
-     * the appropriate position to return.  (For more information, see the description
-     * of buildBackwardsStateTable() in RuleBasedBreakIterator.Builder.)
-     * @internal
-     */
-    virtual int32_t handlePrevious(void);
-
     /**
      * Dumps caches and performs other actions associated with a complete change
-     * in text or iteration position.  This function is a no-op in RuleBasedBreakIterator,
-     * but subclasses can and do override it.
+     * in text or iteration position.
      * @internal
      */
     virtual void reset(void);
 
+#if 0
     /**
       * Return true if the category lookup for this char
       * indicates that it is in the set of dictionary lookup chars.
@@ -544,6 +604,19 @@ protected:
       */
     virtual UBool isDictionaryChar(UChar32);
 
+    /**
+      * Get the type of the break iterator.
+      * @internal
+      */
+    virtual int32_t getBreakType() const;
+#endif
+
+    /**
+      * Set the type of the break iterator.
+      * @internal
+      */
+    virtual void setBreakType(int32_t type);
+
     /**
       * Common initialization function, used by constructors and bufferClone.
       *   (Also used by DictionaryBasedBreakIterator::createBufferClone().)
@@ -575,6 +648,34 @@ private:
      */
     int32_t handleNext(const RBBIStateTable *statetable);
 
+protected:
+
+    /**
+     * This is the function that actually implements dictionary-based
+     * breaking.  Covering at least the range from startPos to endPos,
+     * it checks for dictionary characters, and if it finds them determines
+     * the appropriate object to deal with them. It may cache found breaks in
+     * fCachedBreakPositions as it goes. It may well also look at text outside
+     * the range startPos to endPos.
+     * If going forward, endPos is the normal Unicode break result, and
+     * if goind in reverse, startPos is the normal Unicode break result
+     * @param startPos  The start position of a range of text
+     * @param endPos    The end position of a range of text
+     * @param reverse   The call is for the reverse direction
+     * @internal
+     */
+    int32_t checkDictionary(int32_t startPos, int32_t endPos, UBool reverse);
+
+private:
+
+    /**
+     * This function returns the appropriate LanguageBreakEngine for a
+     * given character c.
+     * @param c         A character in the dictionary set
+     * @internal
+     */
+    const LanguageBreakEngine *getLanguageBreakEngine(UChar32 c);
+
     /**
      *  @internal
      */
