@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-* Copyright (C) 1999-2001, International Business Machines Corporation and   *
+* Copyright (C) 1999-2007, International Business Machines Corporation and   *
 * others. All Rights Reserved.                                               *
 ******************************************************************************
 *
@@ -20,14 +20,10 @@
 
 #include "unicode/utypes.h"
 #include "unicode/putil.h"
-#include "unicode/locid.h"
 #include "cstring.h"
 #include "cmemory.h"
 #include "unicode/ustring.h"
 #include "unicode/unistr.h"
-#include "unicode/uchar.h"
-#include "unicode/ucnv.h"
-#include "unicode/ubrk.h"
 #include "uhash.h"
 #include "ustr_imp.h"
 #include "umutex.h"
@@ -90,8 +86,30 @@ us_arrayCopy(const UChar *src, int32_t srcStart,
   }
 }
 
+// u_unescapeAt() callback to get a UChar from a UnicodeString
+U_CDECL_BEGIN
+static UChar U_CALLCONV
+UnicodeString_charAt(int32_t offset, void *context) {
+    return ((U_NAMESPACE_QUALIFIER UnicodeString*) context)->charAt(offset);
+}
+U_CDECL_END
+
 U_NAMESPACE_BEGIN
 
+/* The Replaceable virtual destructor can't be defined in the header
+   due to how AIX works with multiple definitions of virtual functions.
+*/
+Replaceable::~Replaceable() {}
+Replaceable::Replaceable() {}
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(UnicodeString)
+
+UnicodeString U_EXPORT2
+operator+ (const UnicodeString &s1, const UnicodeString &s2) {
+    return
+        UnicodeString(s1.length()+s2.length()+1, (UChar32)0, 0).
+            append(s1).
+                append(s2);
+}
 
 //========================================
 // Reference Counting functions, put at top of file so that optimizing compilers
@@ -108,11 +126,14 @@ UnicodeString::removeRef()
 
 int32_t
 UnicodeString::refCount() const 
-{ return *((int32_t *)fArray - 1); }
-
-int32_t
-UnicodeString::setRefCount(int32_t count)
-{ return (*((int32_t *)fArray - 1) = count); }
+{ 
+    umtx_lock(NULL);
+    // Note: without the lock to force a memory barrier, we might see a very
+    //       stale value on some multi-processor systems.
+    int32_t  count = *((int32_t *)fArray - 1);
+    umtx_unlock(NULL);
+    return count;
+ }
 
 void
 UnicodeString::releaseArray() {
@@ -139,7 +160,7 @@ UnicodeString::UnicodeString(int32_t capacity, UChar32 c, int32_t count)
     fArray(0),
     fFlags(0)
 {
-  if(count <= 0) {
+  if(count <= 0 || (uint32_t)c > 0x10ffff) {
     // just allocate and do not do anything else
     allocate(capacity);
   } else {
@@ -195,7 +216,8 @@ UnicodeString::UnicodeString(UChar32 ch)
     fFlags(kShortString)
 {
   int32_t i = 0;
-  UTF_APPEND_CHAR(fStackBuffer, i, US_STACKBUF_SIZE, ch);
+  UBool isError = FALSE;
+  U16_APPEND(fStackBuffer, i, US_STACKBUF_SIZE, ch, isError);
   fLength = i;
 }
 
@@ -205,7 +227,7 @@ UnicodeString::UnicodeString(const UChar *text)
     fArray(fStackBuffer),
     fFlags(kShortString)
 {
-  doReplace(0, 0, text, 0, u_strlen(text));
+  doReplace(0, 0, text, 0, -1);
 }
 
 UnicodeString::UnicodeString(const UChar *text,
@@ -226,7 +248,16 @@ UnicodeString::UnicodeString(UBool isTerminated,
     fArray((UChar *)text),
     fFlags(kReadonlyAlias)
 {
-  if(text == 0 || textLength < -1 || textLength == -1 && !isTerminated) {
+  if(text == NULL) {
+    // treat as an empty string, do not alias
+    fLength = 0;
+    fCapacity = US_STACKBUF_SIZE;
+    fArray = fStackBuffer;
+    fFlags = kShortString;
+  } else if(textLength < -1 ||
+            (textLength == -1 && !isTerminated) ||
+            (textLength >= 0 && isTerminated && text[textLength] != 0)
+  ) {
     setToBogus();
   } else if(textLength == -1) {
     // text is terminated, or else it would have failed the above test
@@ -236,76 +267,47 @@ UnicodeString::UnicodeString(UBool isTerminated,
 }
 
 UnicodeString::UnicodeString(UChar *buff,
-                             int32_t bufLength,
+                             int32_t buffLength,
                              int32_t buffCapacity)
-  : fLength(bufLength),
+  : fLength(buffLength),
     fCapacity(buffCapacity),
     fArray(buff),
     fFlags(kWritableAlias)
 {
-  if(buff == 0 || bufLength < 0 || bufLength > buffCapacity) {
+  if(buff == NULL) {
+    // treat as an empty string, do not alias
+    fLength = 0;
+    fCapacity = US_STACKBUF_SIZE;
+    fArray = fStackBuffer;
+    fFlags = kShortString;
+  } else if(buffLength < -1 || buffCapacity < 0 || buffLength > buffCapacity) {
     setToBogus();
-  }
-}
-
-UnicodeString::UnicodeString(const char *codepageData,
-                             const char *codepage)
-  : fLength(0),
-    fCapacity(US_STACKBUF_SIZE),
-    fArray(fStackBuffer),
-    fFlags(kShortString)
-{
-  if(codepageData != 0) {
-    doCodepageCreate(codepageData, (int32_t)uprv_strlen(codepageData), codepage);
-  }
-}
-
-
-UnicodeString::UnicodeString(const char *codepageData,
-                             int32_t dataLength,
-                             const char *codepage)
-  : fLength(0),
-    fCapacity(US_STACKBUF_SIZE),
-    fArray(fStackBuffer),
-    fFlags(kShortString)
-{
-  if(codepageData != 0) {
-    doCodepageCreate(codepageData, dataLength, codepage);
-  }
-}
-
-UnicodeString::UnicodeString(const char *src, int32_t srcLength,
-                             UConverter *cnv,
-                             UErrorCode &errorCode)
-  : fLength(0),
-    fCapacity(US_STACKBUF_SIZE),
-    fArray(fStackBuffer),
-    fFlags(kShortString)
-{
-  if(U_SUCCESS(errorCode)) {
-    // check arguments
-    if(srcLength<-1 || (srcLength!=0 && src==0)) {
-      errorCode=U_ILLEGAL_ARGUMENT_ERROR;
-    } else {
-      // get input length
-      if(srcLength==-1) {
-        srcLength=(int32_t)uprv_strlen(src);
-      }
-      if(srcLength>0) {
-        if(cnv!=0) {
-          // use the provided converter
-          ucnv_resetToUnicode(cnv);
-          doCodepageCreate(src, srcLength, cnv, errorCode);
-        } else {
-          // use the default converter
-          cnv=u_getDefaultConverter(&errorCode);
-          doCodepageCreate(src, srcLength, cnv, errorCode);
-          u_releaseDefaultConverter(cnv);
-        }
-      }
+  } else if(buffLength == -1) {
+    // fLength = u_strlen(buff); but do not look beyond buffCapacity
+    const UChar *p = buff, *limit = buff + buffCapacity;
+    while(p != limit && *p != 0) {
+      ++p;
     }
+    fLength = (int32_t)(p - buff);
+  }
+}
 
-    if(U_FAILURE(errorCode)) {
+UnicodeString::UnicodeString(const char *src, int32_t length, EInvariant)
+  : fLength(0),
+    fCapacity(US_STACKBUF_SIZE),
+    fArray(fStackBuffer),
+    fFlags(kShortString)
+{
+  if(src==NULL) {
+    // treat as an empty string
+  } else {
+    if(length<0) {
+      length=(int32_t)uprv_strlen(src);
+    }
+    if(cloneArrayIfNeeded(length, length, FALSE)) {
+      u_charsToUChars(src, getArrayStart(), length);
+      fLength = length;
+    } else {
       setToBogus();
     }
   }
@@ -318,7 +320,42 @@ UnicodeString::UnicodeString(const UnicodeString& that)
     fArray(fStackBuffer),
     fFlags(kShortString)
 {
-  *this = that;
+  copyFrom(that);
+}
+
+UnicodeString::UnicodeString(const UnicodeString& that,
+                             int32_t srcStart)
+  : Replaceable(),
+    fLength(0),
+    fCapacity(US_STACKBUF_SIZE),
+    fArray(fStackBuffer),
+    fFlags(kShortString)
+{
+  setTo(that, srcStart);
+}
+
+UnicodeString::UnicodeString(const UnicodeString& that,
+                             int32_t srcStart,
+                             int32_t srcLength)
+  : Replaceable(),
+    fLength(0),
+    fCapacity(US_STACKBUF_SIZE),
+    fArray(fStackBuffer),
+    fFlags(kShortString)
+{
+  setTo(that, srcStart, srcLength);
+}
+
+// Replaceable base class clone() default implementation, does not clone
+Replaceable *
+Replaceable::clone() const {
+  return NULL;
+}
+
+// UnicodeString overrides clone() with a real implementation
+Replaceable *
+UnicodeString::clone() const {
+  return new UnicodeString(*this);
 }
 
 //========================================
@@ -367,9 +404,19 @@ UnicodeString::~UnicodeString()
 //========================================
 // Assignment
 //========================================
-UnicodeString&
-UnicodeString::operator= (const UnicodeString& src)
-{
+
+UnicodeString &
+UnicodeString::operator=(const UnicodeString &src) {
+  return copyFrom(src);
+}
+
+UnicodeString &
+UnicodeString::fastCopyFrom(const UnicodeString &src) {
+  return copyFrom(src, TRUE);
+}
+
+UnicodeString &
+UnicodeString::copyFrom(const UnicodeString &src, UBool fastCopy) {
   // if assigning to ourselves, do nothing
   if(this == 0 || this == &src) {
     return *this;
@@ -407,13 +454,22 @@ UnicodeString::operator= (const UnicodeString& src)
     // src uses a refCounted string buffer, use that buffer with refCount
     // src is const, use a cast - we don't really change it
     ((UnicodeString &)src).addRef();
-    // fall through to readonly alias copying: copy all fields
-  case kReadonlyAlias:
-    // src is a readonly alias, do the same
+    // copy all fields, share the reference-counted buffer
     fArray = src.fArray;
     fCapacity = src.fCapacity;
     fFlags = src.fFlags;
     break;
+  case kReadonlyAlias:
+    if(fastCopy) {
+      // src is a readonly alias, do the same
+      // -> maintain the readonly alias as such
+      fArray = src.fArray;
+      fCapacity = src.fCapacity;
+      fFlags = src.fFlags;
+      break;
+    }
+    // else if(!fastCopy) fall through to case kWritableAlias
+    // -> allocate a new buffer and copy the contents
   case kWritableAlias:
     // src is a writable alias; we make a copy of that instead
     if(allocate(fLength)) {
@@ -437,46 +493,6 @@ UnicodeString::operator= (const UnicodeString& src)
 //========================================
 // Miscellaneous operations
 //========================================
-int32_t
-UnicodeString::numDisplayCells( int32_t start,
-                int32_t length,
-                UBool asian) const
-{
-  // pin indices to legal values
-  pinIndices(start, length);
-
-  UChar32 c;
-  int32_t result = 0;
-  int32_t limit = start + length;
-
-  while(start < limit) {
-    UTF_NEXT_CHAR(fArray, start, limit, c);
-    switch(u_charCellWidth(c)) {
-    case U_ZERO_WIDTH:
-      break;
-
-    case U_HALF_WIDTH:
-      result += 1;
-      break;
-
-    case U_FULL_WIDTH:
-      result += 2;
-      break;
-
-    case U_NEUTRAL_WIDTH:
-      result += (asian ? 2 : 1);
-      break;
-    }
-  }
-
-  return result;
-}
-
-UCharReference
-UnicodeString::operator[] (int32_t pos)
-{
-  return UCharReference(this, pos);
-}
 
 UnicodeString UnicodeString::unescape() const {
     UnicodeString result;
@@ -494,14 +510,6 @@ UnicodeString UnicodeString::unescape() const {
     return result;
 }
 
-// u_unescapeAt() callback to get a UChar from a UnicodeString
-U_CDECL_BEGIN
-static UChar U_CALLCONV
-UnicodeString_charAt(int32_t offset, void *context) {
-    return ((UnicodeString*) context)->charAt(offset);
-}
-U_CDECL_END
-
 UChar32 UnicodeString::unescapeAt(int32_t &offset) const {
     return u_unescapeAt(UnicodeString_charAt, &offset, length(), (void*)this);
 }
@@ -517,19 +525,18 @@ UnicodeString::doCompare( int32_t start,
               int32_t srcLength) const
 {
   // compare illegal string values
+  // treat const UChar *srcChars==NULL as an empty string
   if(isBogus()) {
-    if(srcChars==0) {
-      return 0;
-    } else {
-      return -1;
-    }
-  } else if(srcChars==0) {
-    return 1;
+    return -1;
   }
-
+  
   // pin indices to legal values
   pinIndices(start, length);
 
+  if(srcChars == NULL) {
+    srcStart = srcLength = 0;
+  }
+
   // get the correct pointer
   const UChar *chars = getArrayStart();
 
@@ -597,98 +604,25 @@ UnicodeString::doCompareCodePointOrder(int32_t start,
                                        int32_t srcLength) const
 {
   // compare illegal string values
+  // treat const UChar *srcChars==NULL as an empty string
   if(isBogus()) {
-    if(srcChars==0) {
-      return 0;
-    } else {
-      return -1;
-    }
-  } else if(srcChars==0) {
-    return 1;
+    return -1;
   }
 
   // pin indices to legal values
   pinIndices(start, length);
 
-  // get the correct pointer
-  const UChar *chars = getArrayStart();
-
-  chars += start;
-  srcChars += srcStart;
-
-  int32_t minLength;
-  int8_t lengthResult;
-
-  // get the srcLength if necessary
-  if(srcLength < 0) {
-    srcLength = u_strlen(srcChars + srcStart);
+  if(srcChars == NULL) {
+    srcStart = srcLength = 0;
   }
 
-  // are we comparing different lengths?
-  if(length != srcLength) {
-    if(length < srcLength) {
-      minLength = length;
-      lengthResult = -1;
-    } else {
-      minLength = srcLength;
-      lengthResult = 1;
-    }
+  int32_t diff = uprv_strCompare(fArray + start, length, srcChars + srcStart, srcLength, FALSE, TRUE);
+  /* translate the 32-bit result into an 8-bit one */
+  if(diff!=0) {
+    return (int8_t)(diff >> 15 | 1);
   } else {
-    minLength = length;
-    lengthResult = 0;
+    return 0;
   }
-
-  if(minLength > 0 && chars != srcChars) {
-    int32_t diff = u_memcmpCodePointOrder(chars, srcChars, minLength);
-    if(diff!=0) {
-      return (int8_t)(diff >> 15 | 1);
-    }
-  }
-  return lengthResult;
-}
-
-int8_t
-UnicodeString::doCaseCompare(int32_t start,
-                             int32_t length,
-                             const UChar *srcChars,
-                             int32_t srcStart,
-                             int32_t srcLength,
-                             uint32_t options) const
-{
-  // compare illegal string values
-  if(isBogus()) {
-    if(srcChars==0) {
-      return 0;
-    } else {
-      return -1;
-    }
-  } else if(srcChars==0) {
-    return 1;
-  }
-
-  // pin indices to legal values
-  pinIndices(start, length);
-
-  // get the srcLength if necessary
-  if(srcLength < 0) {
-    srcLength = u_strlen(srcChars + srcStart);
-  }
-
-  // get the correct pointer
-  const UChar *chars = getArrayStart();
-
-  chars += start;
-  srcChars += srcStart;
-
-  if(chars != srcChars) {
-    int32_t result=u_internalStrcasecmp(chars, length, srcChars, srcLength, options);
-    if(result!=0) {
-      return (int8_t)(result >> 24 | 1);
-    }
-  } else if(length != srcLength) {
-    return (int8_t)((length - srcLength) >> 24 | 1);
-  }
-  return 0;
 }
 
 int32_t
@@ -713,6 +647,13 @@ UnicodeString::countChar32(int32_t start, int32_t length) const {
   return u_countChar32(fArray+start, length);
 }
 
+UBool
+UnicodeString::hasMoreChar32Than(int32_t start, int32_t length, int32_t number) const {
+  pinIndices(start, length);
+  // if(isBogus()) then fArray==0 and start==0 - u_strHasMoreChar32Than() checks for NULL
+  return u_strHasMoreChar32Than(fArray+start, length, number);
+}
+
 int32_t
 UnicodeString::moveIndex32(int32_t index, int32_t delta) const {
   // pin index
@@ -763,6 +704,37 @@ UnicodeString::extract(UChar *dest, int32_t destCapacity,
   return fLength;
 }
 
+int32_t
+UnicodeString::extract(int32_t start,
+                       int32_t length,
+                       char *target,
+                       int32_t targetCapacity,
+                       enum EInvariant) const
+{
+  // if the arguments are illegal, then do nothing
+  if(targetCapacity < 0 || (targetCapacity > 0 && target == NULL)) {
+    return 0;
+  }
+
+  // pin the indices to legal values
+  pinIndices(start, length);
+
+  if(length <= targetCapacity) {
+    u_UCharsToChars(getArrayStart() + start, target, length);
+  }
+  UErrorCode status = U_ZERO_ERROR;
+  return u_terminateChars(target, targetCapacity, length, &status);
+}
+
+void 
+UnicodeString::extractBetween(int32_t start,
+                  int32_t limit,
+                  UnicodeString& target) const {
+  pinIndex(start);
+  pinIndex(limit);
+  doExtract(start, limit - start, target);
+}
+
 int32_t 
 UnicodeString::indexOf(const UChar *srcChars,
                int32_t srcStart,
@@ -774,42 +746,21 @@ UnicodeString::indexOf(const UChar *srcChars,
     return -1;
   }
 
-  // get the srcLength if necessary
-  if(srcLength < 0) {
-    srcLength = u_strlen(srcChars + srcStart);
-    if(srcLength == 0) {
-      return -1;
-    }
+  // UnicodeString does not find empty substrings
+  if(srcLength < 0 && srcChars[srcStart] == 0) {
+    return -1;
   }
 
-  // now we will only work with srcLength-1
-  --srcLength;
-
   // get the indices within bounds
   pinIndices(start, length);
 
-  // set length for the last possible match start position
-  // note the --srcLength above
-  length -= srcLength;
-
-  if(length <= 0) {
+  // find the first occurrence of the substring
+  const UChar *match = u_strFindFirst(fArray + start, length, srcChars + srcStart, srcLength);
+  if(match == NULL) {
     return -1;
+  } else {
+    return (int32_t)(match - fArray);
   }
-
-  const UChar *array = getArrayStart();
-  int32_t limit = start + length;
-
-  // search for the first char, then compare the rest of the string
-  // increment srcStart here for that, matching the --srcLength above
-  UChar ch = srcChars[srcStart++];
-
-  do {
-    if(array[start] == ch && (srcLength == 0 || compare(start + 1, srcLength, srcChars, srcStart, srcLength) == 0)) {
-      return start;
-    }
-  } while(++start < limit);
-
-  return -1;
 }
 
 int32_t
@@ -819,21 +770,14 @@ UnicodeString::doIndexOf(UChar c,
 {
   // pin indices
   pinIndices(start, length);
-  if(length == 0) {
-    return -1;
-  }
 
   // find the first occurrence of c
-  const UChar *begin = getArrayStart() + start;
-  const UChar *limit = begin + length;
-
-  do {
-    if(*begin == c) {
-      return begin - getArrayStart();
-    }
-  } while(++begin < limit);
-
-  return -1;
+  const UChar *match = u_memchr(fArray + start, c, length);
+  if(match == NULL) {
+    return -1;
+  } else {
+    return (int32_t)(match - fArray);
+  }
 }
 
 int32_t
@@ -842,26 +786,13 @@ UnicodeString::doIndexOf(UChar32 c,
                          int32_t length) const {
   // pin indices
   pinIndices(start, length);
-  if(length == 0) {
-    return -1;
-  }
 
-  // c<0xd800 handled by inline function indexOf(UChar32 c, start, length)
-  if(c<=0xdfff) {
-    // surrogate code point
-    const UChar *t = uprv_strFindSurrogate(fArray + start, length, (UChar)c);
-    if(t != 0) {
-      return t - fArray;
-    } else {
-      return -1;
-    }
-  } else if(c<=0xffff) {
-    // non-surrogate BMP code point
-    return doIndexOf((UChar)c, start, length);
+  // find the first occurrence of c
+  const UChar *match = u_memchr32(fArray + start, c, length);
+  if(match == NULL) {
+    return -1;
   } else {
-    // supplementary code point, search for string
-    UChar buffer[2] = { UTF16_LEAD(c), UTF16_TRAIL(c) };
-    return indexOf(buffer, 2, start, length);
+    return (int32_t)(match - fArray);
   }
 }
 
@@ -876,43 +807,21 @@ UnicodeString::lastIndexOf(const UChar *srcChars,
     return -1;
   }
 
-  // get the srcLength if necessary
-  if(srcLength < 0) {
-    srcLength = u_strlen(srcChars + srcStart);
-    if(srcLength == 0) {
-      return -1;
-    }
+  // UnicodeString does not find empty substrings
+  if(srcLength < 0 && srcChars[srcStart] == 0) {
+    return -1;
   }
 
-  // now we will only work with srcLength-1
-  --srcLength;
-
   // get the indices within bounds
   pinIndices(start, length);
 
-  // set length for the last possible match start position
-  // note the --srcLength above
-  length -= srcLength;
-
-  if(length <= 0) {
+  // find the last occurrence of the substring
+  const UChar *match = u_strFindLast(fArray + start, length, srcChars + srcStart, srcLength);
+  if(match == NULL) {
     return -1;
+  } else {
+    return (int32_t)(match - fArray);
   }
-
-  const UChar *array = getArrayStart();
-  int32_t pos;
-
-  // search for the first char, then compare the rest of the string
-  // increment srcStart here for that, matching the --srcLength above
-  UChar ch = srcChars[srcStart++];
-
-  pos = start + length;
-  do {
-    if(array[--pos] == ch && (srcLength == 0 || compare(pos + 1, srcLength, srcChars, srcStart, srcLength) == 0)) {
-      return pos;
-    }
-  } while(pos > start);
-
-  return -1;
 }
 
 int32_t
@@ -926,20 +835,14 @@ UnicodeString::doLastIndexOf(UChar c,
 
   // pin indices
   pinIndices(start, length);
-  if(length == 0) {
+
+  // find the last occurrence of c
+  const UChar *match = u_memrchr(fArray + start, c, length);
+  if(match == NULL) {
     return -1;
+  } else {
+    return (int32_t)(match - fArray);
   }
-
-  const UChar *begin = getArrayStart() + start;
-  const UChar *limit = begin + length;
-
-  do {
-    if(*--limit == c) {
-      return limit - getArrayStart();
-    }
-  } while(limit > begin);
-
-  return -1;
 }
 
 int32_t
@@ -948,29 +851,20 @@ UnicodeString::doLastIndexOf(UChar32 c,
                              int32_t length) const {
   // pin indices
   pinIndices(start, length);
-  if(length == 0) {
-    return -1;
-  }
 
-  // c<0xd800 handled by inline function lastIndexOf(UChar32 c, start, length)
-  if(c<=0xdfff) {
-    // surrogate code point
-    const UChar *t = uprv_strFindLastSurrogate(fArray + start, length, (UChar)c);
-    if(t != 0) {
-      return t - fArray;
-    } else {
-      return -1;
-    }
-  } else if(c<=0xffff) {
-    // non-surrogate BMP code point
-    return doLastIndexOf((UChar)c, start, length);
+  // find the last occurrence of c
+  const UChar *match = u_memrchr32(fArray + start, c, length);
+  if(match == NULL) {
+    return -1;
   } else {
-    // supplementary code point, search for string
-    UChar buffer[2] = { UTF16_LEAD(c), UTF16_TRAIL(c) };
-    return lastIndexOf(buffer, 2, start, length);
+    return (int32_t)(match - fArray);
   }
 }
 
+//========================================
+// Write implementation
+//========================================
+
 UnicodeString& 
 UnicodeString::findAndReplace(int32_t start,
                   int32_t length,
@@ -1010,10 +904,6 @@ UnicodeString::findAndReplace(int32_t start,
 }
 
 
-//========================================
-// Write implementation
-//========================================
-
 void
 UnicodeString::setToBogus()
 {
@@ -1024,6 +914,17 @@ UnicodeString::setToBogus()
   fFlags = kIsBogus;
 }
 
+// turn a bogus string into an empty one
+void
+UnicodeString::unBogus() {
+  if(fFlags & kIsBogus) {
+    fArray = fStackBuffer;
+    fLength = 0;
+    fCapacity = US_STACKBUF_SIZE;
+    fFlags = kShortString;
+  }
+}
+
 // setTo() analogous to the readonly-aliasing constructor with the same signature
 UnicodeString &
 UnicodeString::setTo(UBool isTerminated,
@@ -1035,7 +936,20 @@ UnicodeString::setTo(UBool isTerminated,
     return *this;
   }
 
-  if(text == 0 || textLength < -1 || textLength == -1 && !isTerminated) {
+  if(text == NULL) {
+    // treat as an empty string, do not alias
+    releaseArray();
+    fLength = 0;
+    fCapacity = US_STACKBUF_SIZE;
+    fArray = fStackBuffer;
+    fFlags = kShortString;
+    return *this;
+  }
+
+  if( textLength < -1 ||
+      (textLength == -1 && !isTerminated) ||
+      (textLength >= 0 && isTerminated && text[textLength] != 0)
+  ) {
     setToBogus();
     return *this;
   }
@@ -1045,13 +959,13 @@ UnicodeString::setTo(UBool isTerminated,
   fArray = (UChar *)text;
   if(textLength != -1) {
     fLength = textLength;
+    fCapacity = isTerminated ? fLength + 1 : fLength;
   } else {
     // text is terminated, or else it would have failed the above test
     fLength = u_strlen(text);
     fCapacity = fLength + 1;
   }
 
-  fCapacity = isTerminated ? fLength + 1 : fLength;
   fFlags = kReadonlyAlias;
   return *this;
 }
@@ -1066,9 +980,26 @@ UnicodeString::setTo(UChar *buffer,
     return *this;
   }
 
-  if(buffer == 0 || buffLength < 0 || buffLength > buffCapacity) {
+  if(buffer == NULL) {
+    // treat as an empty string, do not alias
+    releaseArray();
+    fLength = 0;
+    fCapacity = US_STACKBUF_SIZE;
+    fArray = fStackBuffer;
+    fFlags = kShortString;
+    return *this;
+  }
+
+  if(buffLength < -1 || buffCapacity < 0 || buffLength > buffCapacity) {
     setToBogus();
     return *this;
+  } else if(buffLength == -1) {
+    // buffLength = u_strlen(buff); but do not look beyond buffCapacity
+    const UChar *p = buffer, *limit = buffer + buffCapacity;
+    while(p != limit && *p != 0) {
+      ++p;
+    }
+    buffLength = (int32_t)(p - buffer);
   }
 
   releaseArray();
@@ -1096,143 +1027,6 @@ UnicodeString::setCharAt(int32_t offset,
   return *this;
 }
 
-/*
- * Implement argument checking and buffer handling
- * for string case mapping as a common function.
- */
-enum {
-    TO_LOWER,
-    TO_UPPER,
-    TO_TITLE,
-    FOLD_CASE
-};
-
-UnicodeString &
-UnicodeString::toLower() {
-  return caseMap(0, Locale::getDefault(), 0, TO_LOWER);
-}
-
-UnicodeString &
-UnicodeString::toLower(const Locale &locale) {
-  return caseMap(0, locale, 0, TO_LOWER);
-}
-
-UnicodeString &
-UnicodeString::toUpper() {
-  return caseMap(0, Locale::getDefault(), 0, TO_UPPER);
-}
-
-UnicodeString &
-UnicodeString::toUpper(const Locale &locale) {
-  return caseMap(0, locale, 0, TO_UPPER);
-}
-
-UnicodeString &
-UnicodeString::toTitle(BreakIterator *titleIter) {
-  return caseMap(titleIter, Locale::getDefault(), 0, TO_TITLE);
-}
-
-UnicodeString &
-UnicodeString::toTitle(BreakIterator *titleIter, const Locale &locale) {
-  return caseMap(titleIter, locale, 0, TO_TITLE);
-}
-
-UnicodeString &
-UnicodeString::foldCase(uint32_t options) {
-    return caseMap(0, Locale::getDefault(), options, FOLD_CASE);
-}
-
-UnicodeString &
-UnicodeString::caseMap(BreakIterator *titleIter,
-                       const Locale& locale,
-                       uint32_t options,
-                       int32_t toWhichCase) {
-  if(fLength <= 0) {
-    // nothing to do
-    return *this;
-  }
-
-  // We need to allocate a new buffer for the internal string case mapping function.
-  // This is very similar to how doReplace() below keeps the old array pointer
-  // and deletes the old array itself after it is done.
-  // In addition, we are forcing cloneArrayIfNeeded() to always allocate a new array.
-  UChar *oldArray = fArray;
-  int32_t oldLength = fLength;
-  int32_t *bufferToDelete = 0;
-
-  // Make sure that if the string is in fStackBuffer we do not overwrite it!
-  int32_t capacity;
-  if(fLength <= US_STACKBUF_SIZE) {
-    if(fArray == fStackBuffer) {
-      capacity = 2 * US_STACKBUF_SIZE; // make sure that cloneArrayIfNeeded() allocates a new buffer
-    } else {
-      capacity = US_STACKBUF_SIZE;
-    }
-  } else {
-    capacity = fLength + 20;
-  }
-  if(!cloneArrayIfNeeded(capacity, capacity, FALSE, &bufferToDelete, TRUE)) {
-    return *this;
-  }
-
-  // set up the titlecasing break iterator
-  UBreakIterator *cTitleIter = 0;
-  UErrorCode errorCode;
-
-  if(toWhichCase == TO_TITLE) {
-    if(titleIter != 0) {
-      cTitleIter = (UBreakIterator *)titleIter;
-    } else {
-      errorCode = U_ZERO_ERROR;
-      cTitleIter = ubrk_open(UBRK_TITLE, locale.getName(),
-                             oldArray, oldLength,
-                             &errorCode);
-      if(U_FAILURE(errorCode)) {
-        delete [] bufferToDelete;
-        setToBogus();
-        return *this;
-      }
-    }
-  }
-
-  // Case-map, and if the result is too long, then reallocate and repeat.
-  do {
-    errorCode = U_ZERO_ERROR;
-    if(toWhichCase==TO_LOWER) {
-      fLength = u_internalStrToLower(fArray, fCapacity,
-                                     oldArray, oldLength,
-                                     0, oldLength,
-                                     locale.getName(),
-                                     &errorCode);
-    } else if(toWhichCase==TO_UPPER) {
-      fLength = u_internalStrToUpper(fArray, fCapacity,
-                                     oldArray, oldLength,
-                                     locale.getName(),
-                                     &errorCode);
-    } else if(toWhichCase==TO_TITLE) {
-      fLength = u_internalStrToTitle(fArray, fCapacity,
-                                     oldArray, oldLength,
-                                     cTitleIter, locale.getName(),
-                                     &errorCode);
-    } else {
-      fLength = u_internalStrFoldCase(fArray, fCapacity,
-                                      oldArray, oldLength,
-                                      options,
-                                      &errorCode);
-    }
-  } while(errorCode==U_BUFFER_OVERFLOW_ERROR && cloneArrayIfNeeded(fLength, fLength, FALSE));
-
-  if(cTitleIter != 0 && titleIter == 0) {
-    ubrk_close(cTitleIter);
-  }
-
-  uprv_free(bufferToDelete);
-  if(U_FAILURE(errorCode)) {
-    setToBogus();
-  }
-  return *this;
-}
-
 UnicodeString&
 UnicodeString::doReplace( int32_t start,
               int32_t length,
@@ -1260,12 +1054,8 @@ UnicodeString::doReplace(int32_t start,
              int32_t srcStart,
              int32_t srcLength)
 {
-  // if we're bogus, set us to empty first
   if(isBogus()) {
-    fArray = fStackBuffer;
-    fLength = 0;
-    fCapacity = US_STACKBUF_SIZE;
-    fFlags = kShortString;
+    return *this;
   }
 
   if(srcChars == 0) {
@@ -1317,7 +1107,9 @@ UnicodeString::doReplace(int32_t start,
 
   // delayed delete in case srcChars == fArray when we started, and
   // to keep oldArray alive for the above operations
-  uprv_free(bufferToDelete);
+  if (bufferToDelete) {
+    uprv_free(bufferToDelete);
+  }
 
   return *this;
 }
@@ -1346,6 +1138,23 @@ UnicodeString::copy(int32_t start, int32_t limit, int32_t dest) {
     uprv_free(text);
 }
 
+/**
+ * Replaceable API
+ *
+ * NOTE: This is for the Replaceable class.  There is no rep.cpp,
+ * so we implement this function here.
+ */
+UBool Replaceable::hasMetaData() const {
+    return TRUE;
+}
+
+/**
+ * Replaceable API
+ */
+UBool UnicodeString::hasMetaData() const {
+    return FALSE;
+}
+
 UnicodeString&
 UnicodeString::doReverse(int32_t start,
              int32_t length)
@@ -1424,53 +1233,6 @@ UnicodeString::padTrailing(int32_t targetLength,
   }
 }
 
-UnicodeString& 
-UnicodeString::trim()
-{
-  if(isBogus()) {
-    return *this;
-  }
-
-  UChar32 c;
-  int32_t i = fLength, length;
-
-  // first cut off trailing white space
-  for(;;) {
-    length = i;
-    if(i <= 0) {
-      break;
-    }
-    UTF_PREV_CHAR(fArray, 0, i, c);
-    if(!(c == 0x20 || u_isWhitespace(c))) {
-      break;
-    }
-  }
-  if(length < fLength) {
-    fLength = length;
-  }
-
-  // find leading white space
-  int32_t start;
-  i = 0;
-  for(;;) {
-    start = i;
-    if(i >= length) {
-      break;
-    }
-    UTF_NEXT_CHAR(fArray, i, length, c);
-    if(!(c == 0x20 || u_isWhitespace(c))) {
-      break;
-    }
-  }
-
-  // move string forward over leading white space
-  if(start > 0) {
-    doReplace(0, start, 0, 0, 0);
-  }
-
-  return *this;
-}
-
 //========================================
 // Hashing
 //========================================
@@ -1486,267 +1248,6 @@ UnicodeString::doHashCode() const
     return hashCode;
 }
 
-//========================================
-// Codeset conversion
-//========================================
-int32_t
-UnicodeString::extract(int32_t start,
-                       int32_t length,
-                       char *target,
-                       uint32_t dstSize,
-                       const char *codepage) const
-{
-  // if the arguments are illegal, then do nothing
-  if(/*dstSize < 0 || */(dstSize > 0 && target == 0)) {
-    return 0;
-  }
-
-  // pin the indices to legal values
-  pinIndices(start, length);
-
-  // create the converter
-  UConverter *converter;
-  UErrorCode status = U_ZERO_ERROR;
-
-  // just write the NUL if the string length is 0
-  if(length == 0) {
-    return u_terminateChars(target, dstSize, 0, &status);
-  }
-
-  // if the codepage is the default, use our cache
-  // if it is an empty string, then use the "invariant character" conversion
-  if (codepage == 0) {
-    converter = u_getDefaultConverter(&status);
-  } else if (*codepage == 0) {
-    // use the "invariant characters" conversion
-    int32_t destLength;
-    // careful: dstSize is unsigned! (0xffffffff means "unlimited")
-    if(dstSize >= 0x80000000) {
-      destLength = length;
-      // make sure that the NUL-termination works (takes int32_t)
-      dstSize=0x7fffffff;
-    } else if(length <= (int32_t)dstSize) {
-      destLength = length;
-    } else {
-      destLength = (int32_t)dstSize;
-    }
-    u_UCharsToChars(getArrayStart() + start, target, destLength);
-    return u_terminateChars(target, (int32_t)dstSize, length, &status);
-  } else {
-    converter = ucnv_open(codepage, &status);
-  }
-
-  length = doExtract(start, length, target, (int32_t)dstSize, converter, status);
-
-  // close the converter
-  if (codepage == 0) {
-    u_releaseDefaultConverter(converter);
-  } else {
-    ucnv_close(converter);
-  }
-
-  return length;
-}
-
-int32_t
-UnicodeString::extract(char *dest, int32_t destCapacity,
-                       UConverter *cnv,
-                       UErrorCode &errorCode) const {
-  if(U_FAILURE(errorCode)) {
-    return 0;
-  }
-
-  if(isBogus() || destCapacity<0 || (destCapacity>0 && dest==0)) {
-    errorCode=U_ILLEGAL_ARGUMENT_ERROR;
-    return 0;
-  }
-
-  // nothing to do?
-  if(fLength<=0) {
-    return u_terminateChars(dest, destCapacity, 0, &errorCode);
-  }
-
-  // get the converter
-  UBool isDefaultConverter;
-  if(cnv==0) {
-    isDefaultConverter=TRUE;
-    cnv=u_getDefaultConverter(&errorCode);
-    if(U_FAILURE(errorCode)) {
-      return 0;
-    }
-  } else {
-    isDefaultConverter=FALSE;
-    ucnv_resetFromUnicode(cnv);
-  }
-
-  // convert
-  int32_t length=doExtract(0, fLength, dest, destCapacity, cnv, errorCode);
-
-  // release the converter
-  if(isDefaultConverter) {
-    u_releaseDefaultConverter(cnv);
-  }
-
-  return length;
-}
-
-void 
-UnicodeString::extractBetween(int32_t start,
-                  int32_t limit,
-                  UnicodeString& target) const
-{ doExtract(start, limit - start, target); }
-
-int32_t
-UnicodeString::doExtract(int32_t start, int32_t length,
-                         char *dest, int32_t destCapacity,
-                         UConverter *cnv,
-                         UErrorCode &errorCode) const {
-  if(U_FAILURE(errorCode)) {
-    if(destCapacity!=0) {
-      *dest=0;
-    }
-    return 0;
-  }
-
-  const UChar *src=fArray+start, *srcLimit=src+length;
-  char *originalDest=dest;
-  const char *destLimit;
-
-  if(destCapacity==0) {
-    destLimit=dest=0;
-  } else if(destCapacity==-1) {
-    // Pin the limit to U_MAX_PTR if the "magic" destCapacity is used.
-    destLimit=(char*)U_MAX_PTR(dest);
-    // for NUL-termination, translate into highest int32_t
-    destCapacity=0x7fffffff;
-  } else {
-    destLimit=dest+destCapacity;
-  }
-
-  // perform the conversion
-  ucnv_fromUnicode(cnv, &dest, destLimit, &src, srcLimit, 0, TRUE, &errorCode);
-  length=(int32_t)(dest-originalDest);
-
-  // if an overflow occurs, then get the preflighting length
-  if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
-    char buffer[1024];
-
-    destLimit=buffer+sizeof(buffer);
-    do {
-      dest=buffer;
-      errorCode=U_ZERO_ERROR;
-      ucnv_fromUnicode(cnv, &dest, destLimit, &src, srcLimit, 0, TRUE, &errorCode);
-      length+=(int32_t)(dest-buffer);
-    } while(errorCode==U_BUFFER_OVERFLOW_ERROR);
-  }
-
-  return u_terminateChars(originalDest, destCapacity, length, &errorCode);
-}
-
-void
-UnicodeString::doCodepageCreate(const char *codepageData,
-                int32_t dataLength,
-                const char *codepage)
-{
-  // if there's nothing to convert, do nothing
-  if(codepageData == 0 || dataLength <= 0) {
-    return;
-  }
-
-  UErrorCode status = U_ZERO_ERROR;
-
-  // create the converter
-  // if the codepage is the default, use our cache
-  // if it is an empty string, then use the "invariant character" conversion
-  UConverter *converter = (codepage == 0 ?
-                             u_getDefaultConverter(&status) :
-                             *codepage == 0 ?
-                               0 :
-                               ucnv_open(codepage, &status));
-
-  // if we failed, set the appropriate flags and return
-  if(U_FAILURE(status)) {
-    setToBogus();
-    return;
-  }
-
-  // perform the conversion
-  if(converter == 0) {
-    // use the "invariant characters" conversion
-    if(cloneArrayIfNeeded(dataLength, dataLength, FALSE)) {
-      u_charsToUChars(codepageData, getArrayStart(), dataLength);
-      fLength = dataLength;
-    } else {
-      setToBogus();
-    }
-    return;
-  }
-
-  // convert using the real converter
-  doCodepageCreate(codepageData, dataLength, converter, status);
-  if(U_FAILURE(status)) {
-    setToBogus();
-  }
-
-  // close the converter
-  if(codepage == 0) {
-    u_releaseDefaultConverter(converter);
-  } else {
-    ucnv_close(converter);
-  }
-}
-
-void
-UnicodeString::doCodepageCreate(const char *codepageData,
-                                int32_t dataLength,
-                                UConverter *converter,
-                                UErrorCode &status) {
-  if(U_FAILURE(status)) {
-    return;
-  }
-
-  // set up the conversion parameters
-  const char *mySource     = codepageData;
-  const char *mySourceEnd  = mySource + dataLength;
-  UChar *myTarget;
-
-  // estimate the size needed:
-  // 1.25 UChar's per source byte should cover most cases
-  int32_t arraySize = dataLength + (dataLength >> 2);
-
-  // we do not care about the current contents
-  UBool doCopyArray = FALSE;
-  for(;;) {
-    if(!cloneArrayIfNeeded(arraySize, arraySize, doCopyArray)) {
-      setToBogus();
-      break;
-    }
-
-    // perform the conversion
-    myTarget = fArray + fLength;
-    ucnv_toUnicode(converter, &myTarget,  fArray + fCapacity,
-           &mySource, mySourceEnd, 0, TRUE, &status);
-
-    // update the conversion parameters
-    fLength = myTarget - fArray;
-
-    // allocate more space and copy data, if needed
-    if(status == U_BUFFER_OVERFLOW_ERROR) {
-      // reset the error code
-      status = U_ZERO_ERROR;
-
-      // keep the previous conversion results
-      doCopyArray = TRUE;
-
-      // estimate the new size needed, larger than before
-      // try 2 UChar's per remaining source byte
-      arraySize = fLength + 2 * (mySourceEnd - mySource);
-    } else {
-      break;
-    }
-  }
-}
-
 //========================================
 // External Buffer
 //========================================
@@ -1799,18 +1300,11 @@ UnicodeString::cloneArrayIfNeeded(int32_t newCapacity,
 
   // while a getBuffer(minCapacity) is "open",
   // prevent any modifications of the string by returning FALSE here
-  if(fFlags & kOpenGetBuffer) {
+  // if the string is bogus, then only an assignment or similar can revive it
+  if((fFlags&(kOpenGetBuffer|kIsBogus))!=0) {
     return FALSE;
   }
 
-  // if we're bogus, set us to empty first
-  if(fFlags & kIsBogus) {
-    fArray = fStackBuffer;
-    fLength = 0;
-    fCapacity = US_STACKBUF_SIZE;
-    fFlags = kShortString;
-  }
-
   /*
    * We need to make a copy of the array if
    * the buffer is read-only, or
@@ -1874,3 +1368,18 @@ UnicodeString::cloneArrayIfNeeded(int32_t newCapacity,
   return TRUE;
 }
 U_NAMESPACE_END
+
+#ifdef U_STATIC_IMPLEMENTATION
+/*
+This should never be called. It is defined here to make sure that the
+virtual vector deleting destructor is defined within unistr.cpp.
+The vector deleting destructor is already a part of UObject,
+but defining it here makes sure that it is included with this object file.
+This makes sure that static library dependencies are kept to a minimum.
+*/
+static void uprv_UnicodeStringDummy(void) {
+    U_NAMESPACE_USE
+    delete [] (new UnicodeString[2]);
+}
+#endif
+
