@@ -1,5 +1,5 @@
 /************************************************************************
- * Copyright (C) 1996-2006, International Business Machines Corporation *
+ * Copyright (C) 1996-2007, International Business Machines Corporation *
  * and others. All Rights Reserved.                                     *
  ************************************************************************
  *  2003-nov-07   srl       Port from Java
@@ -10,7 +10,7 @@
 #if !UCONFIG_NO_FORMATTING
 
 #include "unicode/calendar.h"
-#include "math.h"
+#include <math.h>
 #include <float.h>
 #include "unicode/putil.h"
 #include "uhash.h"
