@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2006, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ******************************************************************************
 *   file name:  nfrule.cpp
@@ -25,10 +25,9 @@
 #include "nfrlist.h"
 #include "nfsubs.h"
 
-U_NAMESPACE_BEGIN
+#include "util.h"
 
-extern const UChar* CSleftBracket;
-extern const UChar* CSrightBracket;
+U_NAMESPACE_BEGIN
 
 NFRule::NFRule(const RuleBasedNumberFormat* _rbnf)
   : baseValue((int32_t)0)
@@ -55,10 +54,11 @@ static const UChar gNine = 0x0039;
 static const UChar gSpace = 0x0020;
 static const UChar gSlash = 0x002f;
 static const UChar gGreaterThan = 0x003e;
+static const UChar gLessThan = 0x003c;
 static const UChar gComma = 0x002c;
 static const UChar gDot = 0x002e;
 static const UChar gTick = 0x0027;
-static const UChar gMinus = 0x002d;
+//static const UChar gMinus = 0x002d;
 static const UChar gSemicolon = 0x003b;
 
 static const UChar gMinusX[] =                  {0x2D, 0x78, 0};    /* "-x" */
@@ -99,6 +99,11 @@ NFRule::makeRules(UnicodeString& description,
     // (this also strips the rule descriptor, if any, off the
     // descripton string)
     NFRule* rule1 = new NFRule(rbnf);
+    /* test for NULL */
+    if (rule1 == 0) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return;
+    }
     rule1->parseRuleDescriptor(description, status);
 
     // check the description to see whether there's text enclosed
@@ -136,6 +141,11 @@ NFRule::makeRules(UnicodeString& description,
             // increment the original rule's base value ("rule1" actually
             // goes SECOND in the rule set's rule list)
             rule2 = new NFRule(rbnf);
+            /* test for NULL */
+            if (rule2 == 0) {
+                status = U_MEMORY_ALLOCATION_ERROR;
+                return;
+            }
             if (rule1->baseValue >= 0) {
                 rule2->baseValue = rule1->baseValue;
                 if (!ruleSet->isFractionRuleSet()) {
@@ -216,7 +226,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
     // it's omitted, just set the base value to 0.
     int32_t p = description.indexOf(gColon);
     if (p == -1) {
-        setBaseValue((int32_t)0);
+        setBaseValue((int32_t)0, status);
     } else {
         // copy the descriptor out into its own string and strip it,
         // along with any trailing whitespace, out of the original
@@ -225,7 +235,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
         descriptor.setTo(description, 0, p);
 
         ++p;
-        while (p < description.length() && u_isWhitespace(description.charAt(p))) {
+        while (p < description.length() && uprv_isRuleWhiteSpace(description.charAt(p))) {
             ++p;
         }
         description.removeBetween(0, p);
@@ -268,7 +278,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
                 else if (c == gSlash || c == gGreaterThan) {
                     break;
                 }
-                else if (u_isWhitespace(c) || c == gComma || c == gDot) {
+                else if (uprv_isRuleWhiteSpace(c) || c == gComma || c == gDot) {
                 }
                 else {
                     // throw new IllegalArgumentException("Illegal character in rule descriptor");
@@ -279,7 +289,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
             }
 
             // we have the base value, so set it
-            setBaseValue(val);
+            setBaseValue(val, status);
 
             // if we stopped the previous loop on a slash, we're
             // now parsing the rule's radix.  Again, accumulate digits
@@ -297,7 +307,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
                     else if (c == gGreaterThan) {
                         break;
                     }
-                    else if (u_isWhitespace(c) || c == gComma || c == gDot) {
+                    else if (uprv_isRuleWhiteSpace(c) || c == gComma || c == gDot) {
                     }
                     else {
                         // throw new IllegalArgumentException("Illegal character is rule descriptor");
@@ -309,7 +319,7 @@ NFRule::parseRuleDescriptor(UnicodeString& description, UErrorCode& status)
 
                 // tempValue now contain's the rule's radix.  Set it
                 // accordingly, and recalculate the rule's exponent
-                radix = (int16_t)val;
+                radix = (int32_t)val;
                 if (radix == 0) {
                     // throw new IllegalArgumentException("Rule can't have radix of 0");
                     status = U_PARSE_ERROR;
@@ -411,8 +421,17 @@ NFRule::extractSubstitution(const NFRuleSet* ruleSet,
         // otherwise the substitution token ends with the same character
         // it began with
     } else {
-        subEnd = ruleText.indexOf(ruleText.charAt(subStart), subStart + 1);
-    }
+        UChar c = ruleText.charAt(subStart);
+        subEnd = ruleText.indexOf(c, subStart + 1);
+        // special case for '<%foo<<'
+        if (c == gLessThan && subEnd != -1 && subEnd < ruleText.length() - 1 && ruleText.charAt(subEnd+1) == c) {
+            // ordinals use "=#,##0==%abbrev=" as their rule.  Notice that the '==' in the middle
+            // occurs because of the juxtaposition of two different rules.  The check for '<' is a hack
+            // to get around this.  Having the duplicate at the front would cause problems with
+            // rules like "<<%" to format, say, percents...
+            ++subEnd;
+        }
+   }
 
     // if we don't find the end of the token (i.e., if we're on a single,
     // unmatched token character), create a null substitution positioned
@@ -444,7 +463,7 @@ NFRule::extractSubstitution(const NFRuleSet* ruleSet,
  * @param The new base value for the rule.
  */
 void
-NFRule::setBaseValue(int64_t newBaseValue)
+NFRule::setBaseValue(int64_t newBaseValue, UErrorCode& status)
 {
     // set the base value
     baseValue = newBaseValue;
@@ -463,10 +482,10 @@ NFRule::setBaseValue(int64_t newBaseValue)
         // has substitutions, and some substitutions hold on to copies
         // of the rule's divisor.  Fix their copies of the divisor.
         if (sub1 != NULL) {
-            sub1->setDivisor(radix, exponent);
+            sub1->setDivisor(radix, exponent, status);
         }
         if (sub2 != NULL) {
-            sub2->setDivisor(radix, exponent);
+            sub2->setDivisor(radix, exponent, status);
         }
 
         // if this is a special rule, its radix and exponent are basically
@@ -560,7 +579,7 @@ static void util_append64(UnicodeString& result, int64_t n)
 }
 
 void
-NFRule::appendRuleText(UnicodeString& result) const
+NFRule::_appendRuleText(UnicodeString& result) const
 {
     switch (getType()) {
     case kNegativeNumberRule: result.append(gMinusX); break;
@@ -723,11 +742,11 @@ NFRule::shouldRollBack(double number) const
 
 static void dumpUS(FILE* f, const UnicodeString& us) {
   int len = us.length();
-  char* buf = new char[len+1];
+  char* buf = (char *)uprv_malloc((len+1)*sizeof(char)); //new char[len+1];
   us.extract(0, len, buf);
   buf[len] = 0;
   fprintf(f, "%s", buf);
-  delete[] buf;
+  uprv_free(buf); //delete[] buf;
 }
 #endif
 
@@ -754,7 +773,7 @@ NFRule::doParse(const UnicodeString& text,
     fprintf(stderr, "doParse %x ", this);
     {
         UnicodeString rt;
-        appendRuleText(rt);
+        _appendRuleText(rt);
         dumpUS(stderr, rt);
     }
 
@@ -1001,7 +1020,12 @@ NFRule::matchToDelimiter(const UnicodeString& text,
             subText.setTo(text, 0, dPos);
             if (subText.length() > 0) {
                 UBool success = sub->doParse(subText, tempPP, _baseValue, upperBound,
-                    formatter->isLenient(), result);
+#if UCONFIG_NO_COLLATION
+                    FALSE,
+#else
+                    formatter->isLenient(),
+#endif
+                    result);
 
                 // if the substitution could match all the text up to
                 // where we found "delimiter", then this function has
@@ -1045,7 +1069,12 @@ NFRule::matchToDelimiter(const UnicodeString& text,
 
         // try to match the whole string against the substitution
         UBool success = sub->doParse(text, tempPP, _baseValue, upperBound,
-            formatter->isLenient(), result);
+#if UCONFIG_NO_COLLATION
+            FALSE,
+#else
+            formatter->isLenient(),
+#endif
+            result);
         if (success && (tempPP.getIndex() != 0 || sub->isNullSubstitution())) {
             // if there's a successful match (or it's a null
             // substitution), update pp to point to the first
@@ -1089,6 +1118,7 @@ NFRule::prefixLength(const UnicodeString& str, const UnicodeString& prefix) cons
         return 0;
     }
 
+#if !UCONFIG_NO_COLLATION
     // go through all this grief if we're in lenient-parse mode
     if (formatter->isLenient()) {
         // get the formatter's collator and use it to create two
@@ -1236,7 +1266,9 @@ NFRule::prefixLength(const UnicodeString& str, const UnicodeString& prefix) cons
 
         // If lenient parsing is turned off, forget all that crap above.
         // Just use String.startsWith() and be done with it.
-  } else {
+  } else
+#endif
+  {
       if (str.startsWith(prefix)) {
           return prefix.length();
       } else {
@@ -1266,6 +1298,7 @@ NFRule::findText(const UnicodeString& str,
                  int32_t startingAt,
                  int32_t* length) const
 {
+#if !UCONFIG_NO_COLLATION
     // if lenient parsing is turned off, this is easy: just call
     // String.indexOf() and we're done
     if (!formatter->isLenient()) {
@@ -1274,7 +1307,9 @@ NFRule::findText(const UnicodeString& str,
 
         // but if lenient parsing is turned ON, we've got some work
         // ahead of us
-    } else {
+    } else
+#endif
+    {
         //----------------------------------------------------------------
         // JDK 1.1 HACK (take out of 1.2-specific code)
 
@@ -1378,6 +1413,7 @@ NFRule::allIgnorable(const UnicodeString& str) const
         return TRUE;
     }
 
+#if !UCONFIG_NO_COLLATION
     // if lenient parsing is turned on, walk through the string with
     // a collation element iterator and make sure each collation
     // element is 0 (ignorable) at the primary level
@@ -1395,6 +1431,8 @@ NFRule::allIgnorable(const UnicodeString& str) const
         delete iter;
         return o == CollationElementIterator::NULLORDER;
     }
+#endif
+
     // if lenient parsing is turned off, there is no such thing as
     // an ignorable character: return true only if the string is empty
     return FALSE;
