@@ -1,6 +1,7 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 1997-2005, International Business Machines
+* Corporation and others. All Rights Reserved.
 *******************************************************************************
 */
 
@@ -12,7 +13,9 @@
 #if U_HAVE_RBNF
 
 #include "unicode/utypes.h"
+#include "unicode/uobject.h"
 #include "unicode/unistr.h"
+#include "putilimp.h"
 
 U_NAMESPACE_BEGIN
 
@@ -25,7 +28,7 @@ class ParsePosition;
 class RuleBasedNumberFormat;
 class UnicodeString;
 
-class NFRule {
+class NFRule : public UMemory {
 public:
 
     enum ERuleType {
@@ -54,7 +57,7 @@ public:
     void setType(ERuleType ruleType) { baseValue = (int32_t)ruleType; }
 
     int64_t getBaseValue() const { return baseValue; }
-    void setBaseValue(int64_t value);
+    void setBaseValue(int64_t value, UErrorCode& status);
 
     double getDivisor() const { return uprv_pow(radix, exponent); }
 
@@ -69,7 +72,7 @@ public:
 
     UBool shouldRollBack(double number) const;
 
-    void appendRuleText(UnicodeString& result) const;
+    void _appendRuleText(UnicodeString& result) const;
 
 private:
     void parseRuleDescriptor(UnicodeString& descriptor, UErrorCode& status);
@@ -90,12 +93,15 @@ private:
 
 private:
     int64_t baseValue;
-    int16_t radix;
+    int32_t radix;
     int16_t exponent;
     UnicodeString ruleText;
     NFSubstitution* sub1;
     NFSubstitution* sub2;
     const RuleBasedNumberFormat* formatter;
+
+    NFRule(const NFRule &other); // forbid copying of this class
+    NFRule &operator=(const NFRule &other); // forbid copying of this class
 };
 
 U_NAMESPACE_END
