@@ -1,28 +1,32 @@
 /*
-**********************************************************************
-*   Copyright (C) 1999-2001, International Business Machines
-*   Corporation and others.  All Rights Reserved.
-**********************************************************************
-*   Date        Name        Description
-*   11/17/99    aliu        Creation.
-**********************************************************************
-*/
-#include "rbt_rule.h"
+ **********************************************************************
+ *   Copyright (C) 1999-2007, International Business Machines
+ *   Corporation and others.  All Rights Reserved.
+ **********************************************************************
+ *   Date        Name        Description
+ *   11/17/99    aliu        Creation.
+ **********************************************************************
+ */
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/rep.h"
-#include "rbt_data.h"
 #include "unicode/unifilt.h"
 #include "unicode/uniset.h"
+#include "rbt_rule.h"
+#include "rbt_data.h"
 #include "cmemory.h"
 #include "strmatch.h"
 #include "strrepl.h"
 #include "util.h"
+#include "putilimp.h"
 
 static const UChar FORWARD_OP[] = {32,62,32,0}; // " > "
 
 U_NAMESPACE_BEGIN
 
-const UChar TransliterationRule::ETHER = 0xFFFF;
-
 /**
  * Construct a new rule with the given input, output text, and other
  * attributes.  A cursor position may be specified for the output text.
@@ -57,6 +61,7 @@ TransliterationRule::TransliterationRule(const UnicodeString& input,
                                          UBool anchorStart, UBool anchorEnd,
                                          const TransliterationRuleData* theData,
                                          UErrorCode& status) :
+    UMemory(),
     segments(0),
     data(theData) {
 
@@ -112,12 +117,22 @@ TransliterationRule::TransliterationRule(const UnicodeString& input,
     if (anteContextLength > 0) {
         anteContext = new StringMatcher(pattern, 0, anteContextLength,
                                         FALSE, *data);
+        /* test for NULL */
+        if (anteContext == 0) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
     }
     
     key = NULL;
     if (keyLength > 0) {
         key = new StringMatcher(pattern, anteContextLength, anteContextLength + keyLength,
                                 FALSE, *data);
+        /* test for NULL */
+        if (key == 0) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
     }
     
     int32_t postContextLength = pattern.length() - keyLength - anteContextLength;
@@ -125,15 +140,26 @@ TransliterationRule::TransliterationRule(const UnicodeString& input,
     if (postContextLength > 0) {
         postContext = new StringMatcher(pattern, anteContextLength + keyLength, pattern.length(),
                                         FALSE, *data);
+        /* test for NULL */
+        if (postContext == 0) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
     }
 
     this->output = new StringReplacer(outputStr, cursorPosition + cursorOffset, data);
+    /* test for NULL */
+    if (this->output == 0) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return;
+    }
 }
 
 /**
  * Copy constructor.
  */
 TransliterationRule::TransliterationRule(TransliterationRule& other) :
+    UMemory(other),
     anteContext(NULL),
     key(NULL),
     postContext(NULL),
@@ -146,7 +172,7 @@ TransliterationRule::TransliterationRule(TransliterationRule& other) :
     segments = NULL;
     segmentsCount = 0;
     if (other.segmentsCount > 0) {
-        segments = new UnicodeFunctor*[other.segmentsCount];
+        segments = (UnicodeFunctor **)uprv_malloc(other.segmentsCount * sizeof(UnicodeFunctor *));
         uprv_memcpy(segments, other.segments, other.segmentsCount*sizeof(segments[0]));
     }
 
@@ -163,7 +189,7 @@ TransliterationRule::TransliterationRule(TransliterationRule& other) :
 }
 
 TransliterationRule::~TransliterationRule() {
-    delete[] segments;
+    uprv_free(segments);
     delete anteContext;
     delete key;
     delete postContext;
@@ -266,6 +292,7 @@ UBool TransliterationRule::masks(const TransliterationRule& r2) const {
     int32_t left2 = r2.anteContextLength;
     int32_t right = len - left;
     int32_t right2 = r2.pattern.length() - left2;
+    int32_t cachedCompare = r2.pattern.compare(left2 - left, len, pattern);
 
     // TODO Clean this up -- some logic might be combinable with the
     // next statement.
@@ -273,7 +300,7 @@ UBool TransliterationRule::masks(const TransliterationRule& r2) const {
     // Test for anchor masking
     if (left == left2 && right == right2 &&
         keyLength <= r2.keyLength &&
-        0 == r2.pattern.compare(0, len, pattern)) {
+        0 == cachedCompare) {
         // The following boolean logic implements the table above
         return (flags == r2.flags) ||
             (!(flags & ANCHOR_START) && !(flags & ANCHOR_END)) ||
@@ -283,7 +310,7 @@ UBool TransliterationRule::masks(const TransliterationRule& r2) const {
     return left <= left2 &&
         (right < right2 ||
          (right == right2 && keyLength <= r2.keyLength)) &&
-        0 == r2.pattern.compare(left2 - left, len, pattern);
+         (0 == cachedCompare);
 }
 
 static inline int32_t posBefore(const Replaceable& str, int32_t pos) {
@@ -496,6 +523,34 @@ void TransliterationRule::setData(const TransliterationRuleData* d) {
     // Don't have to do segments since they are in the context or key
 }
 
+/**
+ * Union the set of all characters that may be modified by this rule
+ * into the given set.
+ */
+void TransliterationRule::addSourceSetTo(UnicodeSet& toUnionTo) const {
+    int32_t limit = anteContextLength + keyLength;
+    for (int32_t i=anteContextLength; i<limit; ) {
+        UChar32 ch = pattern.char32At(i);
+        i += UTF_CHAR_LENGTH(ch);
+        const UnicodeMatcher* matcher = data->lookupMatcher(ch);
+        if (matcher == NULL) {
+            toUnionTo.add(ch);
+        } else {
+            matcher->addMatchSetTo(toUnionTo);
+        }
+    }
+}
+
+/**
+ * Union the set of all characters that may be emitted by this rule
+ * into the given set.
+ */
+void TransliterationRule::addTargetSetTo(UnicodeSet& toUnionTo) const {
+    output->toReplacer()->addReplacementSetTo(toUnionTo);
+}
+
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 //eof
