@@ -1,12 +1,16 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001 IBM and others. All rights reserved.
+*   Copyright (C) 2001-2005 IBM and others. All rights reserved.
 **********************************************************************
 *   Date        Name        Description
 *  03/22/2000   helena      Creation.
 **********************************************************************
 */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/brkiter.h"
 #include "unicode/schriter.h"
 #include "unicode/search.h"
@@ -17,6 +21,7 @@
 U_NAMESPACE_BEGIN
 
 SearchIterator::SearchIterator(const SearchIterator &other)
+    : UObject(other)
 {   
     m_breakiterator_            = other.m_breakiterator_;
     m_text_                     = other.m_text_;
@@ -122,6 +127,7 @@ void SearchIterator::setText(const UnicodeString &text, UErrorCode &status)
         else {
             m_text_        = text;
             m_search_->text = m_text_.getBuffer();
+            m_search_->textLength = m_text_.length();
         }
     }
 }
@@ -200,10 +206,10 @@ int32_t SearchIterator::preceding(int32_t position,
 int32_t SearchIterator::next(UErrorCode &status)
 {
     if (U_SUCCESS(status)) {
-        int32_t offset      = getOffset();
+        int32_t offset = getOffset();
         int32_t matchindex  = m_search_->matchedIndex;
         int32_t     matchlength = m_search_->matchedLength;
-        m_search_->reset        = FALSE;
+        m_search_->reset = FALSE;
         if (m_search_->isForwardSearching == TRUE) {
             int32_t textlength = m_search_->textLength;
             if (offset == textlength || matchindex == textlength || 
@@ -228,8 +234,14 @@ int32_t SearchIterator::next(UErrorCode &status)
             }
         }
 
-        if (matchindex != USEARCH_DONE) {
-            return handleNext(matchindex + matchlength, status); 
+        if (matchlength > 0) {
+            // if matchlength is 0 we are at the start of the iteration
+            if (m_search_->isOverlap) {
+                offset ++;
+            }
+            else {
+                offset += matchlength;
+            }
         }
         return handleNext(offset, status);
     }
@@ -280,7 +292,9 @@ int32_t SearchIterator::previous(UErrorCode &status)
 
 void SearchIterator::reset()
 {
+    UErrorCode status = U_ZERO_ERROR;
     setMatchNotFound();
+    setOffset(0, status);
     m_search_->isOverlap          = FALSE;
     m_search_->isCanonicalMatch   = FALSE;
     m_search_->isForwardSearching = TRUE;
@@ -289,7 +303,7 @@ void SearchIterator::reset()
 
 // protected constructors and destructors -----------------------------
 
-SearchIterator::SearchIterator() : m_breakiterator_(NULL)
+SearchIterator::SearchIterator()
 {
     m_search_                     = (USearch *)uprv_malloc(sizeof(USearch));
     m_search_->breakIter          = NULL;
@@ -301,12 +315,13 @@ SearchIterator::SearchIterator() : m_breakiterator_(NULL)
     m_search_->matchedLength      = 0;
     m_search_->text               = NULL;
     m_search_->textLength         = 0;
+    m_breakiterator_              = NULL;
 }
 
 SearchIterator::SearchIterator(const UnicodeString &text, 
                                      BreakIterator *breakiter) :
-                               m_breakiterator_(breakiter),
-                               m_text_(text)
+                                     m_breakiterator_(breakiter),
+                                     m_text_(text)
 {
     m_search_                     = (USearch *)uprv_malloc(sizeof(USearch));
     m_search_->breakIter          = NULL;
@@ -383,3 +398,5 @@ void SearchIterator::setMatchNotFound()
 
 
 U_NAMESPACE_END
+
+#endif /* #if !UCONFIG_NO_COLLATION */
