@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (c) 2002, International Business Machines Corporation
+*   Copyright (c) 2002-2004, International Business Machines Corporation
 *   and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -11,6 +11,10 @@
 #ifndef STRREPL_H
 #define STRREPL_H
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/unifunct.h"
 #include "unicode/unirepl.h"
 #include "unicode/unistr.h"
@@ -127,14 +131,35 @@ class StringReplacer : public UnicodeFunctor, public UnicodeReplacer {
     virtual UnicodeString& toReplacerPattern(UnicodeString& result,
                                              UBool escapeUnprintable) const;
 
+    /**
+     * Implement UnicodeReplacer
+     */
+    virtual void addReplacementSetTo(UnicodeSet& toUnionTo) const;
+
     /**
      * UnicodeFunctor API
      */
     virtual void setData(const TransliterationRuleData*);
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @draft ICU 2.2
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @draft ICU 2.2
+     */
+    virtual UClassID getDynamicClassID() const;
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 #endif
 
 //eof
