@@ -1,12 +1,17 @@
 /*
 **********************************************************************
-*   Copyright (c) 2002, International Business Machines Corporation
+*   Copyright (c) 2002-2006, International Business Machines Corporation
 *   and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   01/14/2002  aliu        Creation.
 **********************************************************************
 */
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "tridpars.h"
 #include "hash.h"
 #include "mutex.h"
@@ -28,7 +33,7 @@ static const UChar VARIANT_SEP = 0x002F; // /
 static const UChar OPEN_REV    = 0x0028; // (
 static const UChar CLOSE_REV   = 0x0029; // )
 
-static const UChar EMPTY[]     = {0}; // ""
+//static const UChar EMPTY[]     = {0}; // ""
 static const UChar ANY[]       = {65,110,121,0}; // "Any"
 static const UChar ANY_NULL[]  = {65,110,121,45,78,117,108,108,0}; // "Any-Null"
 
@@ -85,37 +90,6 @@ Transliterator* TransliteratorIDParser::SingleID::createInstance() {
     return t;
 }
 
-/**
- * Parse a basic ID from the given string.  A basic ID contains
- * only a single source, target, and variant.  It does not contain
- * a filter or an explicit inverse.
- * @param id the id to be parsed
- * @param pos INPUT-OUTPUT parameter.  On input, the position of
- * the first character to parse.  On output, the position after
- * the last character parsed.  If the parse fails pos will be
- * unchanged.
- * @return the parsed ID in canonical format, or NULL on parse
- * failure.  If the parsed ID did not contain a source, the return
- * ID will not.
- */
-UnicodeString TransliteratorIDParser::parseBasicID(const UnicodeString& id, int32_t& pos) {
-    Specs* specs = parseFilterID(id, pos, FALSE);
-    if (specs != NULL) {
-        UnicodeString buf;
-        if (specs->sawSource) {
-            buf.append(specs->source);
-            buf.append(TARGET_SEP);
-        }
-        buf.append(specs->target);
-        if (specs->variant.length() != 0) {
-            buf.append(VARIANT_SEP);
-            buf.append(specs->variant);
-        }
-        delete specs;
-        return buf;
-    }
-    return EMPTY;
-}
 
 /**
  * Parse a single ID, that is, an ID of the general form
@@ -131,7 +105,7 @@ UnicodeString TransliteratorIDParser::parseBasicID(const UnicodeString& id, int3
  */
 TransliteratorIDParser::SingleID*
 TransliteratorIDParser::parseSingleID(const UnicodeString& id, int32_t& pos,
-                                      int32_t dir) {
+                                      int32_t dir, UErrorCode& status) {
 
     int32_t start = pos;
 
@@ -193,7 +167,7 @@ TransliteratorIDParser::parseSingleID(const UnicodeString& id, int32_t& pos,
         if (dir == FORWARD) {
             single = specsToID(specsA, FORWARD);
         } else {
-            single = specsToSpecialInverse(*specsA);
+            single = specsToSpecialInverse(*specsA, status);
             if (single == NULL) {
                 single = specsToID(specsA, REVERSE);
             }
@@ -207,6 +181,33 @@ TransliteratorIDParser::parseSingleID(const UnicodeString& id, int32_t& pos,
     return single;
 }
 
+/**
+ * Parse a filter ID, that is, an ID of the general form
+ * "[f1] s1-t1/v1", with the filters optional, and the variants optional.
+ * @param id the id to be parsed
+ * @param pos INPUT-OUTPUT parameter.  On input, the position of
+ * the first character to parse.  On output, the position after
+ * the last character parsed.
+ * @return a SingleID object or null if the parse fails
+ */
+TransliteratorIDParser::SingleID*
+TransliteratorIDParser::parseFilterID(const UnicodeString& id, int32_t& pos) {
+
+    int32_t start = pos;
+
+    Specs* specs = parseFilterID(id, pos, TRUE);
+    if (specs == NULL) {
+        pos = start;
+        return NULL;
+    }
+
+    // Assemble return results
+    SingleID* single = specsToID(specs, FORWARD);
+    single->filter = specs->filter;
+    delete specs;
+    return single;
+}
+
 /**
  * Parse a global filter of the form "[f]" or "([f])", depending
  * on 'withParens'.
@@ -250,7 +251,12 @@ UnicodeSet* TransliteratorIDParser::parseGlobalFilter(const UnicodeString& id, i
     if (UnicodeSet::resemblesPattern(id, pos)) {
         ParsePosition ppos(pos);
         UErrorCode ec = U_ZERO_ERROR;
-        filter = new UnicodeSet(id, ppos, ec);
+        filter = new UnicodeSet(id, ppos, USET_IGNORE_SPACE, NULL, ec);
+        /* test for NULL */
+        if (filter == 0) {
+            pos = start;
+            return 0;
+        }
         if (U_FAILURE(ec)) {
             delete filter;
             pos = start;
@@ -295,7 +301,7 @@ static void U_CALLCONV _deleteSingleID(void* obj) {
     delete (TransliteratorIDParser::SingleID*) obj;
 }
 
-static void U_CALLCONV _deleteTransliterator(void* obj) {
+static void U_CALLCONV _deleteTransliteratorTrIDPars(void* obj) {
     delete (Transliterator*) obj;
 }
 U_CDECL_END
@@ -354,7 +360,7 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
 
     UBool sawDelimiter = TRUE;
     for (;;) {
-        SingleID* single = parseSingleID(id, pos, dir);
+        SingleID* single = parseSingleID(id, pos, dir, ec);
         if (single == NULL) {
             break;
         }
@@ -412,7 +418,7 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
     return TRUE;
 
  FAIL:
-    UObjectDeleter save = list.setDeleter(_deleteSingleID);
+    UObjectDeleter *save = list.setDeleter(_deleteSingleID);
     list.removeAllElements();
     list.setDeleter(save);
     delete globalFilter;
@@ -431,41 +437,23 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
  * the reverse.  THIS MAY RESULT IN AN EMPTY VECTOR.  Convert
  * SingleID entries to actual transliterators.
  *
- * Also, optionally, insert the given transliterator at the given
- * position.  This effectively happens before anything else.
- *
  * @param list vector of SingleID objects.  On exit, vector
  * of one or more Transliterators.
- * @param insert Transliterator to insert, or NULL if none.
- * Adopted.
- * @param insertIndex index from 0..list.size()-1, at which
- * to place 'insert', or -1 if none.
  * @return new value of insertIndex.  The index will shift if
  * there are empty items, like "(Lower)", with indices less than
  * insertIndex.
  */
-int32_t TransliteratorIDParser::instantiateList(UVector& list,
-                                                Transliterator* insert,
-                                                int32_t insertIndex,
+void TransliteratorIDParser::instantiateList(UVector& list,
                                                 UErrorCode& ec) {
     UVector tlist(ec);
     if (U_FAILURE(ec)) {
         goto RETURN;
     }
-    tlist.setDeleter(_deleteTransliterator);
+    tlist.setDeleter(_deleteTransliteratorTrIDPars);
 
     Transliterator* t;
     int32_t i;
     for (i=0; i<=list.size(); ++i) { // [sic]: i<=list.size()
-        if (insertIndex == i) {
-            insertIndex = tlist.size();
-            tlist.addElement(insert, ec);
-            if (U_FAILURE(ec)) {
-                goto RETURN;
-            }
-            insert = NULL;
-        }
-
         // We run the loop too long by one, so we can
         // do an insert after the last element
         if (i==list.size()) {
@@ -481,6 +469,7 @@ int32_t TransliteratorIDParser::instantiateList(UVector& list,
             }
             tlist.addElement(t, ec);
             if (U_FAILURE(ec)) {
+                delete t;
                 goto RETURN;
             }
         }
@@ -494,27 +483,31 @@ int32_t TransliteratorIDParser::instantiateList(UVector& list,
             ec = U_INTERNAL_TRANSLITERATOR_ERROR;
         }
         tlist.addElement(t, ec);
+        if (U_FAILURE(ec)) {
+            delete t;
+        }
     }
 
  RETURN:
 
-    UObjectDeleter save = list.setDeleter(_deleteSingleID);
+    UObjectDeleter *save = list.setDeleter(_deleteSingleID);
     list.removeAllElements();
-    list.setDeleter(_deleteTransliterator);
 
-    while (tlist.size() > 0) {
-        t = (Transliterator*) tlist.orphanElementAt(0);
-        list.addElement(t, ec);
-        if (U_FAILURE(ec)) {
-            delete t;
-            list.removeAllElements();
-            break;
+    if (U_SUCCESS(ec)) {
+        list.setDeleter(_deleteTransliteratorTrIDPars);
+
+        while (tlist.size() > 0) {
+            t = (Transliterator*) tlist.orphanElementAt(0);
+            list.addElement(t, ec);
+            if (U_FAILURE(ec)) {
+                delete t;
+                list.removeAllElements();
+                break;
+            }
         }
     }
 
-    delete insert; // Clean up in case of failure
     list.setDeleter(save);
-    return insertIndex;
 }
 
 /**
@@ -571,6 +564,29 @@ void TransliteratorIDParser::IDtoSTV(const UnicodeString& id,
     }
 }
 
+/**
+ * Given source, target, and variant strings, concatenate them into a
+ * full ID.  If the source is empty, then "Any" will be used for the
+ * source, so the ID will always be of the form s-t/v or s-t.
+ */
+void TransliteratorIDParser::STVtoID(const UnicodeString& source,
+                                     const UnicodeString& target,
+                                     const UnicodeString& variant,
+                                     UnicodeString& id) {
+    id = source;
+    if (id.length() == 0) {
+        id = ANY;
+    }
+    id.append(TARGET_SEP).append(target);
+    if (variant.length() != 0) {
+        id.append(VARIANT_SEP).append(variant);
+    }
+    // NUL-terminate the ID string for getTerminatedBuffer.
+    // This prevents valgrind and Purify warnings.
+    id.append((UChar)0);
+    id.truncate(id.length()-1);
+}
+
 /**
  * Register two targets as being inverses of one another.  For
  * example, calling registerSpecialInverse("NFC", "NFD", TRUE) causes
@@ -605,14 +621,24 @@ void TransliteratorIDParser::IDtoSTV(const UnicodeString& id,
  */
 void TransliteratorIDParser::registerSpecialInverse(const UnicodeString& target,
                                                     const UnicodeString& inverseTarget,
-                                                    UBool bidirectional) {
-    init();
+                                                    UBool bidirectional,
+                                                    UErrorCode &status) {
+    init(status);
+    if (U_FAILURE(status)) {
+        return;
+    }
+
+    // If target == inverseTarget then force bidirectional => FALSE
+    if (bidirectional && 0==target.caseCompare(inverseTarget, U_FOLD_CASE_DEFAULT)) {
+        bidirectional = FALSE;
+    }
+
+    umtx_init(&LOCK);
     Mutex lock(&LOCK);
 
-    UErrorCode ec = U_ZERO_ERROR;
-    SPECIAL_INVERSES->put(target, new UnicodeString(inverseTarget), ec);
-    if (bidirectional && 0!=target.caseCompare(inverseTarget, U_FOLD_CASE_DEFAULT)) {
-        SPECIAL_INVERSES->put(inverseTarget, new UnicodeString(target), ec);
+    SPECIAL_INVERSES->put(target, new UnicodeString(inverseTarget), status);
+    if (bidirectional) {
+        SPECIAL_INVERSES->put(inverseTarget, new UnicodeString(target), status);
     }
 }
 
@@ -666,7 +692,7 @@ TransliteratorIDParser::parseFilterID(const UnicodeString& id, int32_t& pos,
 
             ParsePosition ppos(pos);
             UErrorCode ec = U_ZERO_ERROR;
-            UnicodeSet set(id, ppos, ec);
+            UnicodeSet set(id, ppos, USET_IGNORE_SPACE, NULL, ec);
             if (U_FAILURE(ec)) {
                 pos = start;
                 return NULL;
@@ -791,14 +817,19 @@ TransliteratorIDParser::specsToID(const Specs* specs, int32_t dir) {
  * 'filter' field of NULL.
  */
 TransliteratorIDParser::SingleID*
-TransliteratorIDParser::specsToSpecialInverse(const Specs& specs) {
+TransliteratorIDParser::specsToSpecialInverse(const Specs& specs, UErrorCode &status) {
     if (0!=specs.source.caseCompare(ANY, U_FOLD_CASE_DEFAULT)) {
         return NULL;
     }
-    init();
-    Mutex lock(&LOCK);
-    UnicodeString* inverseTarget = (UnicodeString*) SPECIAL_INVERSES->get(
-        specs.target);
+    init(status);
+
+    UnicodeString* inverseTarget;
+
+    umtx_init(&LOCK);
+    umtx_lock(&LOCK);
+    inverseTarget = (UnicodeString*) SPECIAL_INVERSES->get(specs.target);
+    umtx_unlock(&LOCK);
+
     if (inverseTarget != NULL) {
         // If the original ID contained "Any-" then make the
         // special inverse "Any-Foo"; otherwise make it "Foo".
@@ -836,18 +867,24 @@ Transliterator* TransliteratorIDParser::createBasicInstance(const UnicodeString&
 /**
  * Initialize static memory.
  */
-void TransliteratorIDParser::init() {
-    // Lock first, check static pointer second
-    Mutex lock(&LOCK);
-    if (SPECIAL_INVERSES != 0) {
-        // We were blocked by another thread in this method
+void TransliteratorIDParser::init(UErrorCode &status) {
+    if (SPECIAL_INVERSES != NULL) {
         return;
     }
 
-    SPECIAL_INVERSES = new Hashtable(TRUE);
-    SPECIAL_INVERSES->setValueDeleter(uhash_deleteUnicodeString);
+    Hashtable* special_inverses = new Hashtable(TRUE, status);
+    special_inverses->setValueDeleter(uhash_deleteUnicodeString);
 
-    ucln_i18n_registerCleanup();
+    umtx_init(&LOCK);
+    umtx_lock(&LOCK);
+    if (SPECIAL_INVERSES == NULL) {
+        SPECIAL_INVERSES = special_inverses;
+        special_inverses = NULL;
+    }
+    umtx_unlock(&LOCK);
+    delete special_inverses; /*null instance*/
+
+    ucln_i18n_registerCleanup(UCLN_I18N_TRANSLITERATOR, transliterator_cleanup);
 }
 
 /**
@@ -863,4 +900,6 @@ void TransliteratorIDParser::cleanup() {
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 //eof
