@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 2001-2007, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -21,6 +21,11 @@
 #ifndef UCOL_TOKENS_H
 #define UCOL_TOKENS_H
 
+#include "unicode/utypes.h"
+#include "unicode/uset.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "ucol_imp.h"
 #include "uhash.h"
 #include "unicode/parseerr.h"
@@ -38,7 +43,7 @@
 
 /* this is space for the extra strings that need to be unquoted */
 /* during the parsing of the rules */
-#define UCOL_TOK_EXTRA_RULE_SPACE_SIZE 2048
+#define UCOL_TOK_EXTRA_RULE_SPACE_SIZE 4096
 typedef struct UColToken UColToken;
 
 typedef struct  {
@@ -78,6 +83,7 @@ struct UColToken {
   UColToken* previous;
   UColToken* next;
   UChar *rulesToParse;
+  uint16_t flags;
 };
 
 /* 
@@ -102,17 +108,21 @@ typedef struct {
   UColParsedToken parsedToken;
   UChar *source;
   UChar *end;
-  UChar *current;
+  const UChar *current;
   UChar *sourceCurrent;
   UChar *extraCurrent;
   UChar *extraEnd;
-  const InverseTableHeader *invUCA;
+  const InverseUCATableHeader *invUCA;
   const UCollator *UCA;
   UHashtable *tailored;
   UColOptionSet *opts;
   uint32_t resultLen;
+  uint32_t listCapacity;
   UColTokListHeader *lh;
   UColToken *varTop;
+  USet *copySet;
+  USet *removeSet;
+  UBool buildCCTabFlag;  /* Tailoring rule requirs building combining class table. */
 } UColTokenParser;
 
 typedef struct {
@@ -143,7 +153,7 @@ uint32_t ucol_tok_assembleTokenList(UColTokenParser *src,
                                     UErrorCode *status);
 
 U_CFUNC
-void ucol_tok_initTokenList(UColTokenParser *src, const UChar *rules, const uint32_t rulesLength, UCollator *UCA, UErrorCode *status);
+void ucol_tok_initTokenList(UColTokenParser *src, const UChar *rules, const uint32_t rulesLength, const UCollator *UCA, UErrorCode *status);
 
 U_CFUNC void ucol_tok_closeTokenList(UColTokenParser *src);
 
@@ -152,10 +162,23 @@ U_CAPI const UChar* U_EXPORT2 ucol_tok_parseNextToken(UColTokenParser *src,
                         UParseError *parseError,
                         UErrorCode *status);
 
+U_CAPI const UChar * U_EXPORT2
+ucol_tok_getNextArgument(const UChar *start, const UChar *end, 
+                               UColAttribute *attrib, UColAttributeValue *value, 
+                               UErrorCode *status);
+U_CAPI int32_t U_EXPORT2 ucol_inv_getNextCE(const UColTokenParser *src,
+                                            uint32_t CE, uint32_t contCE,
+                                            uint32_t *nextCE, uint32_t *nextContCE,
+                                            uint32_t strength);
+U_CAPI int32_t U_EXPORT2 ucol_inv_getPrevCE(const UColTokenParser *src,
+                                            uint32_t CE, uint32_t contCE,
+                                            uint32_t *prevCE, uint32_t *prevContCE,
+                                            uint32_t strength);
+
+U_CAPI uint32_t U_EXPORT2 ucol_getCEStrengthDifference(uint32_t CE, uint32_t contCE, 
+                                            uint32_t prevCE, uint32_t prevContCE);
+
+
+#endif /* #if !UCONFIG_NO_COLLATION */
 
 #endif
-
-
-
-
-
