@@ -1,30 +1,39 @@
 /*
-********************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
-*   Corporation and others.  All Rights Reserved.
-********************************************************************************
-*
-* File DATEFMT.H
-*
-* Modification History:
-*
-*   Date        Name        Description
-*   02/19/97    aliu        Converted from java.
-*   04/01/97    aliu        Added support for centuries.
-*    07/23/98    stephen        JDK 1.2 sync
-*   11/15/99    weiv        Added support for week of year/day of week formatting
-********************************************************************************
-*/
+ ********************************************************************************
+ *   Copyright (C) 1997-2007, International Business Machines
+ *   Corporation and others.  All Rights Reserved.
+ ********************************************************************************
+ *
+ * File DATEFMT.H
+ *
+ * Modification History:
+ *
+ *   Date        Name        Description
+ *   02/19/97    aliu        Converted from java.
+ *   04/01/97    aliu        Added support for centuries.
+ *   07/23/98    stephen     JDK 1.2 sync
+ *   11/15/99    weiv        Added support for week of year/day of week formatting
+ ********************************************************************************
+ */
 
 #ifndef DATEFMT_H
 #define DATEFMT_H
- 
+
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/udat.h"
 #include "unicode/calendar.h"
 #include "unicode/numfmt.h"
 #include "unicode/format.h"
 #include "unicode/locid.h"
 
+/**
+ * \file 
+ * \brief C++ API: Abstract class for converting dates.
+ */
+
 U_NAMESPACE_BEGIN
 
 class TimeZone;
@@ -120,82 +129,20 @@ class TimeZone;
  *   <li>   Align any particular field, or find out where it is for selection
  *          on the screen.
  * </ul>
+ *
+ * <p><em>User subclasses are not supported.</em> While clients may write
+ * subclasses, such code will not necessarily work and will not be
+ * guaranteed to work stably from release to release.
  */
 class U_I18N_API DateFormat : public Format {
 public:
-    /**
-     * The following enum values are used in FieldPosition with date/time formatting.
-     * They are also used to index into DateFormatSymbols::fgPatternChars, which
-     * is the list of standard internal-representation pattern characters, and
-     * the resource bundle localPatternChars data. For this reason, this enum
-     * should be treated with care; don't change the order or contents of it
-     * unless you really know what you are doing. You'll probably have to change
-     * the code in DateFormatSymbols, SimpleDateFormat, and all the locale
-     * resource bundle data files.
-     */
-    enum EField
-    {
-        kEraField = 0,      // ERA field alignment.
-        kYearField,         // YEAR field alignment.
-        kMonthField,        // MONTH field alignment.
-        kDateField,         // DATE field alignment.
-        kHourOfDay1Field,     // One-based HOUR_OF_DAY field alignment.
-                            // kHourOfDay1Field is used for the one-based 24-hour clock.
-                            // For example, 23:59 + 01:00 results in 24:59.
-        kHourOfDay0Field,     // Zero-based HOUR_OF_DAY field alignment.
-                            // HOUR_OF_DAY0_FIELD is used for the zero-based 24-hour clock.
-                            // For example, 23:59 + 01:00 results in 00:59.
-        kMinuteField,       // MINUTE field alignment.
-        kSecondField,       // SECOND field alignment.
-        kMillisecondField,  // MILLISECOND field alignment.
-        kDayOfWeekField,      // DAY_OF_WEEK field alignment.
-        kDayOfYearField,      // DAY_OF_YEAR field alignment.
-        kDayOfWeekInMonthField,// DAY_OF_WEEK_IN_MONTH field alignment.
-        kWeekOfYearField,     // WEEK_OF_YEAR field alignment.
-        kWeekOfMonthField,    // WEEK_OF_MONTH field alignment.
-        kAmPmField,            // AM_PM field alignment.
-        kHour1Field,        // One-based HOUR field alignment.
-                            // HOUR1_FIELD is used for the one-based 12-hour clock.
-                            // For example, 11:30 PM + 1 hour results in 12:30 AM.
-        kHour0Field,        // Zero-based HOUR field alignment.
-                            // HOUR0_FIELD is used for the zero-based 12-hour clock.
-                            // For example, 11:30 PM + 1 hour results in 00:30 AM.
-        kTimezoneField,      // TIMEZONE field alignment.
-        kYearWOYField,   // Corrected year for week representation
-        kDOWLocalField, // localized day of week
-
-        
-        
-    /**
-     * These constants are provided for backwards compatibility only,
-     * and are deprecated.  Please use the C++ style constants defined above.
-     */
-        ERA_FIELD                     = kEraField,
-        YEAR_FIELD                     = kYearField,
-        MONTH_FIELD                 = kMonthField,
-        DATE_FIELD                     = kDateField,
-        HOUR_OF_DAY1_FIELD             = kHourOfDay1Field,
-        HOUR_OF_DAY0_FIELD             = kHourOfDay0Field,
-        MINUTE_FIELD                 = kMinuteField,
-        SECOND_FIELD                 = kSecondField,
-        MILLISECOND_FIELD             = kMillisecondField,
-        DAY_OF_WEEK_FIELD             = kDayOfWeekField,
-        DAY_OF_YEAR_FIELD             = kDayOfYearField,
-        DAY_OF_WEEK_IN_MONTH_FIELD     = kDayOfWeekInMonthField,
-        WEEK_OF_YEAR_FIELD             = kWeekOfYearField,
-        WEEK_OF_MONTH_FIELD         = kWeekOfMonthField,
-        AM_PM_FIELD                 = kAmPmField,
-        HOUR1_FIELD                 = kHour1Field,
-        HOUR0_FIELD                 = kHour0Field,
-        TIMEZONE_FIELD                 = kTimezoneField
-
-    };
 
     /**
      * Constants for various style patterns. These reflect the order of items in
      * the DateTimePatterns resource. There are 4 time patterns, 4 date patterns,
      * and then the date-time pattern. Each block of 4 values in the resource occurs
      * in the order full, long, medium, short.
+     * @stable ICU 2.4
      */
     enum EStyle
     {
@@ -213,15 +160,28 @@ public:
      // kShort  + kDateOffset = 7
 
         kDateTime             = 8,
+        
+
+        // relative dates
+        kRelative = (1 << 7),
+        
+        kFullRelative = (kFull | kRelative),
+            
+        kLongRelative = kLong | kRelative,
+        
+        kMediumRelative = kMedium | kRelative,
+        
+        kShortRelative = kShort | kRelative,
+        
 
         kDefault      = kMedium,
 
-        
-        
+
+
     /**
-     * These constants are provided for backwards compatibility only,
-     * and are deprecated.  Please use the C++ style constants defined above.
-     */       
+     * These constants are provided for backwards compatibility only.
+     * Please use the C++ style constants defined above.
+     */
         FULL        = kFull,
         LONG        = kLong,
         MEDIUM        = kMedium,
@@ -234,13 +194,13 @@ public:
 
     /**
      * Destructor.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual ~DateFormat();
 
     /**
      * Equality operator.  Returns true if the two formats have the same behavior.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const Format&) const;
 
@@ -249,18 +209,17 @@ public:
      * objects with a UDate type. If a the Formattable object type is not a Date,
      * then it returns a failing UErrorCode.
      *
-     * @param obj           The object to format. Must be a Date.
-     * @param toAppendTo    The result of the formatting operation is appended to
-     *                      this string.
-     * @param pos           On input: an alignment field, if desired.
-     *                      On output: the offsets of the alignment field.
-     * @param status        Output param filled with success/failure status.
-     * @return              The value passed in as toAppendTo (this allows chaining,
-     *                      as with UnicodeString::append())
-     * @stable
+     * @param obj       The object to format. Must be a Date.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @param status    Output param filled with success/failure status.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     virtual UnicodeString& format(const Formattable& obj,
-                                  UnicodeString& toAppendTo,
+                                  UnicodeString& appendTo,
                                   FieldPosition& pos,
                                   UErrorCode& status) const;
 
@@ -270,11 +229,11 @@ public:
      * <P>
      * On input, the FieldPosition parameter may have its "field" member filled with
      * an enum value specifying a field.  On output, the FieldPosition will be filled
-     * in with the text offsets for that field.  
+     * in with the text offsets for that field.
      * <P> For example, given a time text
      * "1996.07.10 AD at 15:08:56 PDT", if the given fieldPosition.field is
-     * DateFormat::kYearField, the offsets fieldPosition.beginIndex and
-     * statfieldPositionus.getEndIndex will be set to 0 and 4, respectively. 
+     * UDAT_YEAR_FIELD, the offsets fieldPosition.beginIndex and
+     * statfieldPositionus.getEndIndex will be set to 0 and 4, respectively.
      * <P> Notice
      * that if the same time field appears more than once in a pattern, the status will
      * be set for the first occurence of that time field. For instance,
@@ -284,17 +243,17 @@ public:
      * fieldPosition.getEndIndex will be set to 5 and 8, respectively, for the first
      * occurence of the timezone pattern character 'z'.
      *
-     * @param cal           a Calendar set to the date and time to be formatted
+     * @param cal           Calendar set to the date and time to be formatted
      *                      into a date/time string.
-     * @param toAppendTo    the result of the formatting operation is appended to
-     *                      the end of this string.
+     * @param appendTo      Output parameter to receive result.
+     *                      Result is appended to existing contents.
      * @param fieldPosition On input: an alignment field, if desired (see examples above)
      *                      On output: the offsets of the alignment field (see examples above)
-     * @return              A reference to 'toAppendTo'.
-     * @draft ICU 2.1
+     * @return              Reference to 'appendTo' parameter.
+     * @stable ICU 2.1
      */
     virtual UnicodeString& format(  Calendar& cal,
-                                    UnicodeString& toAppendTo,
+                                    UnicodeString& appendTo,
                                     FieldPosition& fieldPosition) const = 0;
 
     /**
@@ -302,11 +261,11 @@ public:
      * <P>
      * On input, the FieldPosition parameter may have its "field" member filled with
      * an enum value specifying a field.  On output, the FieldPosition will be filled
-     * in with the text offsets for that field.  
+     * in with the text offsets for that field.
      * <P> For example, given a time text
      * "1996.07.10 AD at 15:08:56 PDT", if the given fieldPosition.field is
-     * DateFormat::kYearField, the offsets fieldPosition.beginIndex and
-     * statfieldPositionus.getEndIndex will be set to 0 and 4, respectively. 
+     * UDAT_YEAR_FIELD, the offsets fieldPosition.beginIndex and
+     * statfieldPositionus.getEndIndex will be set to 0 and 4, respectively.
      * <P> Notice
      * that if the same time field appears more than once in a pattern, the status will
      * be set for the first occurence of that time field. For instance,
@@ -316,16 +275,16 @@ public:
      * fieldPosition.getEndIndex will be set to 5 and 8, respectively, for the first
      * occurence of the timezone pattern character 'z'.
      *
-     * @param date          a UDate to be formatted into a date/time string.
-     * @param toAppendTo    the result of the formatting operation is appended to
-     *                      the end of this string.
+     * @param date          UDate to be formatted into a date/time string.
+     * @param appendTo      Output parameter to receive result.
+     *                      Result is appended to existing contents.
      * @param fieldPosition On input: an alignment field, if desired (see examples above)
      *                      On output: the offsets of the alignment field (see examples above)
-     * @return              A reference to 'toAppendTo'.
-     * @stable
+     * @return              Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(  UDate date,
-                            UnicodeString& toAppendTo,
+                            UnicodeString& appendTo,
                             FieldPosition& fieldPosition) const;
 
     /**
@@ -334,18 +293,25 @@ public:
      * FieldPosition& to detect formatting problems.
      *
      * @param date      The UDate value to be formatted into a string.
-     * @param result    Output param which will receive the formatted date.
-     * @return          A reference to 'result'.
-     * @stable
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
-    UnicodeString& format(UDate date, UnicodeString& result) const;
+    UnicodeString& format(UDate date, UnicodeString& appendTo) const;
 
     /**
      * Redeclared Format method.
-     * @stable
+     *
+     * @param obj       The object to be formatted into a string.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param status    Output param filled with success/failure status.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(const Formattable& obj,
-                          UnicodeString& result,
+                          UnicodeString& appendTo,
                           UErrorCode& status) const;
 
     /**
@@ -356,7 +322,7 @@ public:
      *                  'text' cannot be parsed, it will be set to a failure
      *                  code.
      * @result          The parsed UDate value, if successful.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UDate parse( const UnicodeString& text,
                         UErrorCode& status) const;
@@ -374,11 +340,13 @@ public:
      * @see DateFormat::setLenient(boolean)
      *
      * @param text  The date/time string to be parsed
+     * @param cal   a Calendar set to the date and time to be formatted
+     *              into a date/time string.
      * @param pos   On input, the position at which to start parsing; on
      *              output, the position at which parsing terminated, or the
      *              start position if the parse failed.
      * @return      A valid UDate if the input could be parsed.
-     * @draft ICU 2.1
+     * @stable ICU 2.1
      */
     virtual void parse( const UnicodeString& text,
                         Calendar& cal,
@@ -401,7 +369,7 @@ public:
      *              output, the position at which parsing terminated, or the
      *              start position if the parse failed.
      * @return      A valid UDate if the input could be parsed.
-     * @stable
+     * @stable ICU 2.0
      */
     UDate parse( const UnicodeString& text,
                  ParsePosition& pos) const;
@@ -430,7 +398,7 @@ public:
      * @return          A newly created Formattable* object, or NULL
      *                  on failure.  The caller owns this and should
      *                  delete it when done.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void parseObject(const UnicodeString& source,
                              Formattable& result,
@@ -441,49 +409,49 @@ public:
      * the date and the time.
      *
      * @return A date/time formatter which the caller owns.
-     * @stable
+     * @stable ICU 2.0
      */
-    static DateFormat* createInstance(void);
+    static DateFormat* U_EXPORT2 createInstance(void);
 
     /**
      * Creates a time formatter with the given formatting style for the given
      * locale.
-     * 
+     *
      * @param style     The given formatting style. For example,
      *                  SHORT for "h:mm a" in the US locale.
      * @param aLocale   The given locale.
      * @return          A time formatter which the caller owns.
-     * @stable
+     * @stable ICU 2.0
      */
-    static DateFormat* createTimeInstance(EStyle style = kDefault,
+    static DateFormat* U_EXPORT2 createTimeInstance(EStyle style = kDefault,
                                           const Locale& aLocale = Locale::getDefault());
 
     /**
      * Creates a date formatter with the given formatting style for the given
      * const locale.
-     * 
+     *
      * @param style     The given formatting style. For example,
      *                  SHORT for "M/d/yy" in the US locale.
      * @param aLocale   The given locale.
      * @return          A date formatter which the caller owns.
-     * @stable
+     * @stable ICU 2.0
      */
-    static DateFormat* createDateInstance(EStyle style = kDefault,
+    static DateFormat* U_EXPORT2 createDateInstance(EStyle style = kDefault,
                                           const Locale& aLocale = Locale::getDefault());
 
     /**
      * Creates a date/time formatter with the given formatting styles for the
      * given locale.
-     * 
+     *
      * @param dateStyle The given formatting style for the date portion of the result.
      *                  For example, SHORT for "M/d/yy" in the US locale.
      * @param timeStyle The given formatting style for the time portion of the result.
      *                  For example, SHORT for "h:mm a" in the US locale.
      * @param aLocale   The given locale.
      * @return          A date/time formatter which the caller owns.
-     * @stable
+     * @stable ICU 2.0
      */
-    static DateFormat* createDateTimeInstance(EStyle dateStyle = kDefault,
+    static DateFormat* U_EXPORT2 createDateTimeInstance(EStyle dateStyle = kDefault,
                                               EStyle timeStyle = kDefault,
                                               const Locale& aLocale = Locale::getDefault());
 
@@ -492,13 +460,13 @@ public:
      * @param count Filled in with the number of locales in the list that is returned.
      * @return the set of locales for which DateFormats are installed.  The caller
      *  does NOT own this list and must not delete it.
-     * @stable
+     * @stable ICU 2.0
      */
-    static const Locale* getAvailableLocales(int32_t& count);
-  
+    static const Locale* U_EXPORT2 getAvailableLocales(int32_t& count);
+
     /**
      * Returns true if the formatter is set for lenient parsing.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool isLenient(void) const;
 
@@ -507,98 +475,104 @@ public:
      * parsing, the parser may use heuristics to interpret inputs that do not
      * precisely match this object's format. With strict parsing, inputs must
      * match this object's format.
+     *
+     * @param lenient  True specifies date/time interpretation to be lenient.
      * @see Calendar::setLenient
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void setLenient(UBool lenient);
-    
+
     /**
      * Gets the calendar associated with this date/time formatter.
      * @return the calendar associated with this date/time formatter.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const Calendar* getCalendar(void) const;
-    
+
     /**
      * Set the calendar to be used by this date format. Initially, the default
      * calendar for the specified or default locale is used.  The caller should
      * not delete the Calendar object after it is adopted by this call.
-     * @stable
+     * Adopting a new calendar will change to the default symbols.
+     *
+     * @param calendarToAdopt    Calendar object to be adopted.
+     * @stable ICU 2.0
      */
     virtual void adoptCalendar(Calendar* calendarToAdopt);
 
     /**
      * Set the calendar to be used by this date format. Initially, the default
      * calendar for the specified or default locale is used.
-     * @stable
+     *
+     * @param newCalendar Calendar object to be set.
+     * @stable ICU 2.0
      */
     virtual void setCalendar(const Calendar& newCalendar);
 
-   
+
     /**
      * Gets the number formatter which this date/time formatter uses to format
      * and parse the numeric portions of the pattern.
      * @return the number formatter which this date/time formatter uses.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const NumberFormat* getNumberFormat(void) const;
-    
+
     /**
      * Allows you to set the number formatter.  The caller should
      * not delete the NumberFormat object after it is adopted by this call.
      * @param formatToAdopt     NumberFormat object to be adopted.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void adoptNumberFormat(NumberFormat* formatToAdopt);
 
     /**
      * Allows you to set the number formatter.
-     * @param formatToAdopt     NumberFormat object to be adopted.
-     * @stable
+     * @param newNumberFormat  NumberFormat object to be set.
+     * @stable ICU 2.0
      */
     virtual void setNumberFormat(const NumberFormat& newNumberFormat);
 
     /**
      * Returns a reference to the TimeZone used by this DateFormat's calendar.
      * @return the time zone associated with the calendar of DateFormat.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const TimeZone& getTimeZone(void) const;
-    
+
     /**
      * Sets the time zone for the calendar of this DateFormat object. The caller
      * no longer owns the TimeZone object and should not delete it after this call.
-     * @param zone the new time zone.
-     * @stable
+     * @param zoneToAdopt the TimeZone to be adopted.
+     * @stable ICU 2.0
      */
     virtual void adoptTimeZone(TimeZone* zoneToAdopt);
 
     /**
      * Sets the time zone for the calendar of this DateFormat object.
      * @param zone the new time zone.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void setTimeZone(const TimeZone& zone);
 
-    
 protected:
     /**
      * Default constructor.  Creates a DateFormat with no Calendar or NumberFormat
      * associated with it.  This constructor depends on the subclasses to fill in
      * the calendar and numberFormat fields.
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormat();
 
     /**
      * Copy constructor.
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormat(const DateFormat&);
 
     /**
      * Default assignment operator.
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormat& operator=(const DateFormat&);
 
@@ -606,6 +580,7 @@ protected:
      * The calendar that DateFormat uses to produce the time field values needed
      * to implement date/time formatting. Subclasses should generally initialize
      * this to the default calendar for the locale associated with this DateFormat.
+     * @stable ICU 2.4
      */
     Calendar* fCalendar;
 
@@ -613,6 +588,7 @@ protected:
      * The number formatter that DateFormat uses to format numbers in dates and
      * times. Subclasses should generally initialize this to the default number
      * format for the locale associated with this DateFormat.
+     * @stable ICU 2.4
      */
     NumberFormat* fNumberFormat;
 
@@ -625,16 +601,72 @@ private:
      * @param inLocale the given locale.
      * @return a date/time formatter, or 0 on failure.
      */
-    static DateFormat* create(EStyle timeStyle, EStyle dateStyle, const Locale&);
+    static DateFormat* U_EXPORT2 create(EStyle timeStyle, EStyle dateStyle, const Locale&);
+
+public:
+    /**
+     * Field selector for FieldPosition for DateFormat fields.
+     * @obsolete ICU 3.4 use UDateFormatField instead, since this API will be
+     * removed in that release
+     */
+    enum EField
+    {
+        // Obsolete; use UDateFormatField instead
+        kEraField = UDAT_ERA_FIELD,
+        kYearField = UDAT_YEAR_FIELD,
+        kMonthField = UDAT_MONTH_FIELD,
+        kDateField = UDAT_DATE_FIELD,
+        kHourOfDay1Field = UDAT_HOUR_OF_DAY1_FIELD,
+        kHourOfDay0Field = UDAT_HOUR_OF_DAY0_FIELD,
+        kMinuteField = UDAT_MINUTE_FIELD,
+        kSecondField = UDAT_SECOND_FIELD,
+        kMillisecondField = UDAT_FRACTIONAL_SECOND_FIELD,
+        kDayOfWeekField = UDAT_DAY_OF_WEEK_FIELD,
+        kDayOfYearField = UDAT_DAY_OF_YEAR_FIELD,
+        kDayOfWeekInMonthField = UDAT_DAY_OF_WEEK_IN_MONTH_FIELD,
+        kWeekOfYearField = UDAT_WEEK_OF_YEAR_FIELD,
+        kWeekOfMonthField = UDAT_WEEK_OF_MONTH_FIELD,
+        kAmPmField = UDAT_AM_PM_FIELD,
+        kHour1Field = UDAT_HOUR1_FIELD,
+        kHour0Field = UDAT_HOUR0_FIELD,
+        kTimezoneField = UDAT_TIMEZONE_FIELD,
+        kYearWOYField = UDAT_YEAR_WOY_FIELD,
+        kDOWLocalField = UDAT_DOW_LOCAL_FIELD,
+        kExtendedYearField = UDAT_EXTENDED_YEAR_FIELD,
+        kJulianDayField = UDAT_JULIAN_DAY_FIELD,
+        kMillisecondsInDayField = UDAT_MILLISECONDS_IN_DAY_FIELD,
+
+        // Obsolete; use UDateFormatField instead
+        ERA_FIELD = UDAT_ERA_FIELD,
+        YEAR_FIELD = UDAT_YEAR_FIELD,
+        MONTH_FIELD = UDAT_MONTH_FIELD,
+        DATE_FIELD = UDAT_DATE_FIELD,
+        HOUR_OF_DAY1_FIELD = UDAT_HOUR_OF_DAY1_FIELD,
+        HOUR_OF_DAY0_FIELD = UDAT_HOUR_OF_DAY0_FIELD,
+        MINUTE_FIELD = UDAT_MINUTE_FIELD,
+        SECOND_FIELD = UDAT_SECOND_FIELD,
+        MILLISECOND_FIELD = UDAT_FRACTIONAL_SECOND_FIELD,
+        DAY_OF_WEEK_FIELD = UDAT_DAY_OF_WEEK_FIELD,
+        DAY_OF_YEAR_FIELD = UDAT_DAY_OF_YEAR_FIELD,
+        DAY_OF_WEEK_IN_MONTH_FIELD = UDAT_DAY_OF_WEEK_IN_MONTH_FIELD,
+        WEEK_OF_YEAR_FIELD = UDAT_WEEK_OF_YEAR_FIELD,
+        WEEK_OF_MONTH_FIELD = UDAT_WEEK_OF_MONTH_FIELD,
+        AM_PM_FIELD = UDAT_AM_PM_FIELD,
+        HOUR1_FIELD = UDAT_HOUR1_FIELD,
+        HOUR0_FIELD = UDAT_HOUR0_FIELD,
+        TIMEZONE_FIELD = UDAT_TIMEZONE_FIELD
+    };
 };
 
 inline UnicodeString&
 DateFormat::format(const Formattable& obj,
-                   UnicodeString& result,
+                   UnicodeString& appendTo,
                    UErrorCode& status) const {
-    return Format::format(obj, result, status);
+    return Format::format(obj, appendTo, status);
 }
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif // _DATEFMT
 //eof
