@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001 IBM and others. All rights reserved.
+*   Copyright (C) 2001-2007 IBM and others. All rights reserved.
 **********************************************************************
 *   Date        Name        Description
 *  06/28/2001   synwee      Creation.
@@ -10,22 +10,28 @@
 #define USEARCH_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/ucoleitr.h"
 #include "unicode/ubrk.h"
 
 /**
+ * \file
+ * \brief C API: StringSearch
+ *
  * C Apis for an engine that provides language-sensitive text searching based 
  * on the comparison rules defined in a <tt>UCollator</tt> data struct,
  * see <tt>ucol.h</tt>. This ensures that language eccentricity can be 
- * handled, e.g. for the German collator, characters  and SS will be matched 
+ * handled, e.g. for the German collator, characters &szlig; and SS will be matched 
  * if case is chosen to be ignored. 
- * See the <a href=http://oss.software.ibm.com/icu/develop/collation/ICU_collation_design.htm>
+ * See the <a href="http://source.icu-project.org/repos/icu/icuhtml/trunk/design/collation/ICU_collation_design.htm">
  * "ICU Collation Design Document"</a> for more information.
  * <p> 
  * The algorithm implemented is a modified form of the Boyer Moore's search.
  * For more information  see 
- * <a href=http://oss.software.ibm.com/icu/docs/papers/text-search.html>
+ * <a href="http://icu-project.org/docs/papers/efficient_text_searching_in_java.html">
  * "Efficient Text Searching in Java"</a>, published in <i>Java Report</i> 
  * in February, 1999, for further information on the algorithm.
  * <p>
@@ -39,10 +45,10 @@
  * option 1. Some canonical equivalent of P matches some canonical equivalent 
  *           of S'
  * option 2. P matches S' and if P starts or ends with a combining mark, 
- *           there exists no non-ignorable combining mark before or after S 
+ *           there exists no non-ignorable combining mark before or after S' 
  *           in S respectively. 
  * </pre>
- * Option 2. will be the default
+ * Option 2. will be the default.
  * <p>
  * This search has APIs similar to that of other text iteration mechanisms 
  * such as the break iterators in <tt>ubrk.h</tt>. Using these 
@@ -72,7 +78,7 @@
  *      contracting sequence except the first will fail. Vice versa if a 
  *      preceding pattern match is to be found, a invalid starting point 
  *      would be any character within a contracting sequence except the last.
- * <\ul>
+ * </ul>
  * <p>
  * A breakiterator can be used if only matches at logical breaks are desired.
  * Using a breakiterator will only give you results that exactly matches the
@@ -109,31 +115,43 @@
  * u_uastrcpy(pattern, patstr);
  *
  * UStringSearch *search = usearch_open(pattern, -1, target, -1, "en_US", 
- *                                  &status);
+ *                                  NULL, &status);
  * if (U_SUCCESS(status)) {
- *     for (int pos = usearch_first(search); 
- *                                      pos != USEARCH_DONE; 
- *                                      pos = usearch_next(search)) {
+ *     for (int pos = usearch_first(search, &status); 
+ *          pos != USEARCH_DONE; 
+ *          pos = usearch_next(search, &status))
+ *     {
  *         printf("Found match at %d pos, length is %d\n", pos, 
  *                                        usearch_getMatchLength(search));
  *     }
  * }
+ *
+ * usearch_close(search);
  * </code></pre>
- * @since ICU 2.0
+ * @stable ICU 2.4
  */
 
 /**
 * DONE is returned by previous() and next() after all valid matches have 
 * been returned, and by first() and last() if there are no matches at all.
+* @stable ICU 2.4
 */
 #define USEARCH_DONE -1
 
 /**
 * Data structure for searching
+* @stable ICU 2.4
 */
 struct UStringSearch;
+/**
+* Data structure for searching
+* @stable ICU 2.4
+*/
 typedef struct UStringSearch UStringSearch;
 
+/**
+* @stable ICU 2.4
+*/
 typedef enum {
     /** Option for overlapping matches */
     USEARCH_OVERLAP,
@@ -145,6 +163,9 @@ typedef enum {
     USEARCH_ATTRIBUTE_COUNT
 } USearchAttribute;
 
+/**
+* @stable ICU 2.4
+*/
 typedef enum {
     /** default value for any USearchAttribute */
     USEARCH_DEFAULT = -1,
@@ -177,9 +198,9 @@ typedef enum {
 *               patternlength or textlength is 0 then an 
 *               U_ILLEGAL_ARGUMENT_ERROR is returned.
 * @return search iterator data structure, or NULL if there is an error.
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI UStringSearch * U_EXPORT2 usearch_open(const UChar          *pattern, 
+U_STABLE UStringSearch * U_EXPORT2 usearch_open(const UChar          *pattern, 
                                               int32_t         patternlength, 
                                         const UChar          *text, 
                                               int32_t         textlength,
@@ -191,6 +212,8 @@ U_CAPI UStringSearch * U_EXPORT2 usearch_open(const UChar          *pattern,
 * Creating a search iterator data struct using the argument collator language
 * rule set. Note, user retains the ownership of this collator, thus the 
 * responsibility of deletion lies with the user.
+* NOTE: string search cannot be instantiated from a collator that has 
+* collate digits as numbers (CODAN) turned on.
 * @param pattern for matching
 * @param patternlength length of the pattern, -1 for null-termination
 * @param text text string
@@ -207,9 +230,9 @@ U_CAPI UStringSearch * U_EXPORT2 usearch_open(const UChar          *pattern,
 *               or if patternlength or textlength is 0 then an 
 *               U_ILLEGAL_ARGUMENT_ERROR is returned.
 * @return search iterator data structure, or NULL if there is an error.
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI UStringSearch * U_EXPORT2 usearch_openFromCollator(
+U_STABLE UStringSearch * U_EXPORT2 usearch_openFromCollator(
                                          const UChar *pattern, 
                                                int32_t         patternlength,
                                          const UChar          *text, 
@@ -220,11 +243,11 @@ U_CAPI UStringSearch * U_EXPORT2 usearch_openFromCollator(
 
 /**
 * Destroying and cleaning up the search iterator data struct.
-* If a collator is created in usearch_open, it will be destroyed here.
+* If a collator is created in <tt>usearch_open</tt>, it will be destroyed here.
 * @param searchiter data struct to clean up
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_close(UStringSearch *strsrch);
+U_STABLE void U_EXPORT2 usearch_close(UStringSearch *searchiter);
 
 /* get and set methods -------------------------------------------------- */
 
@@ -237,35 +260,38 @@ U_CAPI void U_EXPORT2 usearch_close(UStringSearch *strsrch);
 * Search positions that may render incorrect results are highlighted in the
 * header comments
 * @param strsrch search iterator data struct
-* @param position position to start next search from.
+* @param position position to start next search from. If position is less
+*          than or greater than the text range for searching, 
+*          an U_INDEX_OUTOFBOUNDS_ERROR will be returned
 * @param status error status if any.
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setOffset(UStringSearch *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setOffset(UStringSearch *strsrch, 
                                         int32_t    position,
                                         UErrorCode    *status);
 
 /**
 * Return the current index in the string text being searched.
 * If the iteration has gone past the end of the text (or past the beginning 
-* for a backwards search), {@link #USEARCH_DONE} is returned.
+* for a backwards search), <tt>USEARCH_DONE</tt> is returned.
 * @param strsrch search iterator data struct
-* @draft ICU 2.0
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_getOffset(const UStringSearch *strsrch);
+U_STABLE int32_t U_EXPORT2 usearch_getOffset(const UStringSearch *strsrch);
     
 /**
 * Sets the text searching attributes located in the enum USearchAttribute
 * with values from the enum USearchAttributeValue.
-* USEARCH_DEFAULT can be used for all attributes for resetting.
+* <tt>USEARCH_DEFAULT</tt> can be used for all attributes for resetting.
 * @param strsrch search iterator data struct
 * @param attribute text attribute to be set
 * @param value text attribute value
 * @param status for errors if it occurs
 * @see #usearch_getAttribute
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setAttribute(UStringSearch         *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setAttribute(UStringSearch         *strsrch, 
                                            USearchAttribute       attribute,
                                            USearchAttributeValue  value,
                                            UErrorCode            *status);
@@ -276,48 +302,58 @@ U_CAPI void U_EXPORT2 usearch_setAttribute(UStringSearch         *strsrch,
 * @param attribute text attribute to be retrieve
 * @return text attribute value
 * @see #usearch_setAttribute
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI USearchAttributeValue U_EXPORT2 usearch_getAttribute(
+U_STABLE USearchAttributeValue U_EXPORT2 usearch_getAttribute(
                                          const UStringSearch    *strsrch,
                                                USearchAttribute  attribute);
 
 /**
 * Returns the index to the match in the text string that was searched.
 * This call returns a valid result only after a successful call to 
-* {@link #usearch_first}, {@link #usearch_next}, 
-* {@link #usearch_previous}, or {@link #usearch_last}.
+* <tt>usearch_first</tt>, <tt>usearch_next</tt>, <tt>usearch_previous</tt>, 
+* or <tt>usearch_last</tt>.
 * Just after construction, or after a searching method returns 
 * <tt>USEARCH_DONE</tt>, this method will return <tt>USEARCH_DONE</tt>.
 * <p>
-* Use usearch_getMatchedLength to get the matched string length.
+* Use <tt>usearch_getMatchedLength</tt> to get the matched string length.
 * @param strsrch search iterator data struct
 * @return index to a substring within the text string that is being 
 *         searched.
-* @draft ICU 2.0
+* @see #usearch_first
+* @see #usearch_next
+* @see #usearch_previous
+* @see #usearch_last
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_getMatchedStart(
+U_STABLE int32_t U_EXPORT2 usearch_getMatchedStart(
                                                const UStringSearch *strsrch);
     
 /**
 * Returns the length of text in the string which matches the search pattern. 
 * This call returns a valid result only after a successful call to 
-* {@link #usearch_first}, {@link #usearch_next}, 
-* {@link #usearch_previous}, or {@link #usearch_last}.
+* <tt>usearch_first</tt>, <tt>usearch_next</tt>, <tt>usearch_previous</tt>, 
+* or <tt>usearch_last</tt>.
 * Just after construction, or after a searching method returns 
 * <tt>USEARCH_DONE</tt>, this method will return 0.
 * @param strsrch search iterator data struct
 * @return The length of the match in the string text, or 0 if there is no 
 *         match currently.
-* @draft ICU 2.0
+* @see #usearch_first
+* @see #usearch_next
+* @see #usearch_previous
+* @see #usearch_last
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_getMatchedLength(
+U_STABLE int32_t U_EXPORT2 usearch_getMatchedLength(
                                                const UStringSearch *strsrch);
 
 /**
 * Returns the text that was matched by the most recent call to 
-* {@link #usearch_first}, {@link #usearch_next}, 
-* {@link #usearch_previous}, or {@link #usearch_last}.
+* <tt>usearch_first</tt>, <tt>usearch_next</tt>, <tt>usearch_previous</tt>, 
+* or <tt>usearch_last</tt>.
 * If the iterator is not pointing at a valid match (e.g. just after 
 * construction or after <tt>USEARCH_DONE</tt> has been returned, returns
 * an empty string. If result is not large enough to store the matched text,
@@ -326,19 +362,26 @@ U_CAPI int32_t U_EXPORT2 usearch_getMatchedLength(
 * possible. If the buffer fits the matched text exactly, a null-termination 
 * is not possible, then a U_STRING_NOT_TERMINATED_ERROR set in status.
 * Pre-flighting can be either done with length = 0 or the API 
-* usearch_getMatchLength().
+* <tt>usearch_getMatchLength</tt>.
 * @param strsrch search iterator data struct
 * @param result UChar buffer to store the matched string
 * @param resultCapacity length of the result buffer
 * @param status error returned if result is not large enough
 * @return exact length of the matched text, not counting the null-termination
-* @draft ICU 2.0
+* @see #usearch_first
+* @see #usearch_next
+* @see #usearch_previous
+* @see #usearch_last
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_getMatchedText(const UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_getMatchedText(const UStringSearch *strsrch, 
                                             UChar         *result, 
                                             int32_t        resultCapacity, 
                                             UErrorCode    *status);
 
+#if !UCONFIG_NO_BREAK_ITERATION
+
 /**
 * Set the BreakIterator that will be used to restrict the points at which 
 * matches are detected.
@@ -352,12 +395,12 @@ U_CAPI int32_t U_EXPORT2 usearch_getMatchedText(const UStringSearch *strsrch,
 *                  attempted.
 * @param status for errors if it occurs
 * @see #usearch_getBreakIterator
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setBreakIterator(UStringSearch  *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setBreakIterator(UStringSearch  *strsrch, 
                                                UBreakIterator *breakiter,
                                                UErrorCode     *status);
-    
+
 /**
 * Returns the BreakIterator that is used to restrict the points at which 
 * matches are detected. This will be the same object that was passed to the 
@@ -367,11 +410,13 @@ U_CAPI void U_EXPORT2 usearch_setBreakIterator(UStringSearch  *strsrch,
 * @param strsrch search iterator data struct
 * @return break iterator used
 * @see #usearch_setBreakIterator
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI const UBreakIterator * U_EXPORT2 usearch_getBreakIterator(
+U_STABLE const UBreakIterator * U_EXPORT2 usearch_getBreakIterator(
                                               const UStringSearch *strsrch);
     
+#endif
+    
 /**
 * Set the string text to be searched. Text iteration will hence begin at the 
 * start of the text string. This method is useful if you want to re-use an 
@@ -383,9 +428,9 @@ U_CAPI const UBreakIterator * U_EXPORT2 usearch_getBreakIterator(
 *               then an U_ILLEGAL_ARGUMENT_ERROR is returned with no change
 *               done to strsrch.
 * @see #usearch_getText
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setText(      UStringSearch *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setText(      UStringSearch *strsrch, 
                                       const UChar         *text,
                                             int32_t        textlength,
                                             UErrorCode    *status);
@@ -396,9 +441,9 @@ U_CAPI void U_EXPORT2 usearch_setText(      UStringSearch *strsrch,
 * @param length returned string text length
 * @return string text 
 * @see #usearch_setText
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI const UChar * U_EXPORT2 usearch_getText(const UStringSearch *strsrch, 
+U_STABLE const UChar * U_EXPORT2 usearch_getText(const UStringSearch *strsrch, 
                                                int32_t       *length);
 
 /**
@@ -409,9 +454,9 @@ U_CAPI const UChar * U_EXPORT2 usearch_getText(const UStringSearch *strsrch,
 * <tt>usearch_close</tt> will delete the collator if this search owns it.
 * @param strsrch search iterator data struct
 * @return collator
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI UCollator * U_EXPORT2 usearch_getCollator(
+U_STABLE UCollator * U_EXPORT2 usearch_getCollator(
                                                const UStringSearch *strsrch);
 
 /**
@@ -422,9 +467,9 @@ U_CAPI UCollator * U_EXPORT2 usearch_getCollator(
 * @param strsrch search iterator data struct
 * @param collator to be used
 * @param status for errors if it occurs
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setCollator(      UStringSearch *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setCollator(      UStringSearch *strsrch, 
                                           const UCollator     *collator,
                                                 UErrorCode    *status);
 
@@ -438,9 +483,9 @@ U_CAPI void U_EXPORT2 usearch_setCollator(      UStringSearch *strsrch,
 * @param status for errors if it occurs. If text is NULL, or textlength is 0 
 *               then an U_ILLEGAL_ARGUMENT_ERROR is returned with no change
 *               done to strsrch.
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_setPattern(      UStringSearch *strsrch, 
+U_STABLE void U_EXPORT2 usearch_setPattern(      UStringSearch *strsrch, 
                                          const UChar         *pattern,
                                                int32_t        patternlength,
                                                UErrorCode    *status);
@@ -451,9 +496,9 @@ U_CAPI void U_EXPORT2 usearch_setPattern(      UStringSearch *strsrch,
 * @param length return length of the pattern, -1 indicates that the pattern 
 *               is null-terminated
 * @return pattern string
-* @draft ICU 2.0
+* @stable ICU 2.4
 */
-U_CAPI const UChar * U_EXPORT2 usearch_getPattern(
+U_STABLE const UChar * U_EXPORT2 usearch_getPattern(
                                                const UStringSearch *strsrch, 
                                                      int32_t       *length);
 
@@ -463,74 +508,84 @@ U_CAPI const UChar * U_EXPORT2 usearch_getPattern(
 * Returns the first index at which the string text matches the search 
 * pattern.  
 * The iterator is adjusted so that its current index (as returned by 
-* {@link #usearch_getOffset}) is the match position if one was found.
+* <tt>usearch_getOffset</tt>) is the match position if one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE.
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>.
 * @param strsrch search iterator data struct
 * @param status for errors if it occurs
 * @return The character index of the first match, or 
 * <tt>USEARCH_DONE</tt> if there are no matches.
-* @draft ICU 2.0
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_first(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_first(UStringSearch *strsrch, 
                                            UErrorCode    *status);
 
 /**
 * Returns the first index greater than <tt>position</tt> at which the string 
 * text 
 * matches the search pattern. The iterator is adjusted so that its current 
-* index (as returned by {@link #usearch_getOffset}) is the match position if 
+* index (as returned by <tt>usearch_getOffset</tt>) is the match position if 
 * one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>
 * <p>
 * Search positions that may render incorrect results are highlighted in the
-* header comments.
+* header comments. If position is less than or greater than the text range 
+* for searching, an U_INDEX_OUTOFBOUNDS_ERROR will be returned
 * @param strsrch search iterator data struct
 * @param position to start the search at
 * @param status for errors if it occurs
 * @return The character index of the first match following <tt>pos</tt>,
 *         or <tt>USEARCH_DONE</tt> if there are no matches.
-* @draft ICU 2.0
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_following(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_following(UStringSearch *strsrch, 
                                                int32_t    position, 
                                                UErrorCode    *status);
     
 /**
 * Returns the last index in the target text at which it matches the search 
 * pattern. The iterator is adjusted so that its current 
-* index (as returned by {@link #usearch_getOffset}) is the match position if 
+* index (as returned by <tt>usearch_getOffset</tt>) is the match position if 
 * one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE.
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>.
 * @param strsrch search iterator data struct
 * @param status for errors if it occurs
 * @return The index of the first match, or <tt>USEARCH_DONE</tt> if there 
 *         are no matches.
-* @draft ICU 2.0
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_last(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_last(UStringSearch *strsrch, 
                                           UErrorCode    *status);
 
 /**
 * Returns the first index less than <tt>position</tt> at which the string text 
 * matches the search pattern. The iterator is adjusted so that its current 
-* index (as returned by {@link #usearch_getOffset}) is the match position if 
+* index (as returned by <tt>usearch_getOffset</tt>) is the match position if 
 * one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>
 * <p>
 * Search positions that may render incorrect results are highlighted in the
-* header comments.
+* header comments. If position is less than or greater than the text range 
+* for searching, an U_INDEX_OUTOFBOUNDS_ERROR will be returned
 * @param strsrch search iterator data struct
 * @param position index position the search is to begin at
 * @param status for errors if it occurs
 * @return The character index of the first match preceding <tt>pos</tt>,
 *         or <tt>USEARCH_DONE</tt> if there are no matches.
-* @draft ICU 2.0
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_preceding(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_preceding(UStringSearch *strsrch, 
                                                int32_t    position, 
                                                UErrorCode    *status);
     
@@ -538,35 +593,40 @@ U_CAPI int32_t U_EXPORT2 usearch_preceding(UStringSearch *strsrch,
 * Returns the index of the next point at which the string text matches the
 * search pattern, starting from the current position.
 * The iterator is adjusted so that its current 
-* index (as returned by {@link #usearch_getOffset}) is the match position if 
+* index (as returned by <tt>usearch_getOffset</tt>) is the match position if 
 * one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>
 * @param strsrch search iterator data struct
 * @param status for errors if it occurs
 * @return The index of the next match after the current position, or 
 *         <tt>USEARCH_DONE</tt> if there are no more matches.
 * @see #usearch_first
-* @draft ICU 2.0
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_next(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_next(UStringSearch *strsrch, 
                                           UErrorCode    *status);
 
 /**
 * Returns the index of the previous point at which the string text matches
 * the search pattern, starting at the current position.
 * The iterator is adjusted so that its current 
-* index (as returned by {@link #usearch_getOffset}) is the match position if 
+* index (as returned by <tt>usearch_getOffset</tt>) is the match position if 
 * one was found.
 * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
-* the iterator will be adjusted to the index USEARCH_DONE
+* the iterator will be adjusted to the index <tt>USEARCH_DONE</tt>
 * @param strsrch search iterator data struct
 * @param status for errors if it occurs
 * @return The index of the previous match before the current position,
 *         or <tt>USEARCH_DONE</tt> if there are no more matches.
-* @draft ICU 2.0
+* @see #usearch_last
+* @see #usearch_getOffset
+* @see #USEARCH_DONE
+* @stable ICU 2.4
 */
-U_CAPI int32_t U_EXPORT2 usearch_previous(UStringSearch *strsrch, 
+U_STABLE int32_t U_EXPORT2 usearch_previous(UStringSearch *strsrch, 
                                               UErrorCode    *status);
     
 /** 
@@ -576,10 +636,11 @@ U_CAPI int32_t U_EXPORT2 usearch_previous(UStringSearch *strsrch,
 * iteration is initiated before a forwards iteration, the search will begin
 * at the end of the text string.
 * @param strsrch search iterator data struct
-* @draft ICU 2.0
+* @see #usearch_first
+* @stable ICU 2.4
 */
-U_CAPI void U_EXPORT2 usearch_reset(UStringSearch *strsrch);
+U_STABLE void U_EXPORT2 usearch_reset(UStringSearch *strsrch);
+
+#endif /* #if !UCONFIG_NO_COLLATION */
 
 #endif
-
-
