@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001 IBM and others. All rights reserved.
+*   Copyright (C) 2001-2007 IBM and others. All rights reserved.
 **********************************************************************
 *   Date        Name        Description
 *  08/13/2001   synwee      Creation.
@@ -10,6 +10,9 @@
 #define USRCHIMP_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/ucoleitr.h"
 #include "unicode/ubrk.h"
@@ -23,6 +26,7 @@ struct USearch {
           int32_t             textLength; // exact length
           UBool               isOverlap;
           UBool               isCanonicalMatch;
+          UBreakIterator	 *internalBreakIter;  //internal character breakiterator
           UBreakIterator     *breakIter;
     // value USEARCH_DONE is the default value
     // if we are not at the start of the text or the end of the text, 
@@ -39,8 +43,8 @@ struct UPattern {
           int32_t             textLength; // exact length
           // length required for backwards ce comparison
           int32_t             CELength; 
-          uint32_t           *CE;
-          uint32_t            CEBuffer[INITIAL_ARRAY_SIZE_];
+          int32_t            *CE;
+          int32_t             CEBuffer[INITIAL_ARRAY_SIZE_];
           UBool               hasPrefixAccents;
           UBool               hasSuffixAccents;
           int16_t             defaultShiftSize;
@@ -59,7 +63,6 @@ struct UStringSearch {
     // iteration.
            UCollationElements *utilIter;
            UBool               ownCollator;
-           UBool               toNormalize;
            UCollationStrength  strength;
            uint32_t            ceMask;
            uint32_t            variableTop;
@@ -109,6 +112,7 @@ UBool usearch_handleNextCanonical(UStringSearch *strsrch, UErrorCode *status);
 * Comments follows from handleNextExact
 * @param strsrch string search data
 * @param status error status if any
+* @return True if a exact math is found, FALSE otherwise.
 */
 U_CFUNC
 UBool usearch_handlePreviousExact(UStringSearch *strsrch, UErrorCode *status);
@@ -125,6 +129,6 @@ U_CFUNC
 UBool usearch_handlePreviousCanonical(UStringSearch *strsrch, 
                                       UErrorCode    *status);
 
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif
-
-
