@@ -1,24 +1,25 @@
 
 /*
- * %W% %W%
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2007 - All Rights Reserved
  *
  */
 
 #ifndef __LAYOUTENGINE_H
 #define __LAYOUTENGINE_H
 
-#ifndef __LETYPES_H
 #include "LETypes.h"
-#endif
 
-#include <string.h>
+/**
+ * \file 
+ * \brief C++ API: Virtual base class for complex text layout.
+ */
 
 U_NAMESPACE_BEGIN
 
 class LEFontInstance;
 class LEGlyphFilter;
+class LEGlyphStorage;
 
 /**
  * This is a virtual base class used to do complex text layout. The text must all
@@ -60,38 +61,24 @@ class LEGlyphFilter;
  *
  * @see LEFontInstance
  * @see ScriptAndLanguageTags.h
+ *
+ * @stable ICU 2.8
  */
-class U_LAYOUT_API LayoutEngine
-{
+class U_LAYOUT_API LayoutEngine : public UObject {
 protected:
     /**
-     * The number of glyphs in the output
+     * The object which holds the glyph storage
+     *
+     * @internal
      */
-    le_int32 fGlyphCount;
-
-    /**
-     * The output glyph array
-     */
-    LEGlyphID *fGlyphs;
-
-    /**
-     * The character index array. One entry for each output glyph, giving the index
-     * in the input character array of the character which corresponds to this glyph.
-     */
-    le_int32 *fCharIndices;
-
-    /**
-     * The glyph position array. There are two entries for each glyph giving the
-     * X and Y positions of the glyph. Thus, for glyph i, the X position is at index
-     * 2i, and the Y position is at index 2i + 1. There are also two entries for the
-     * X and Y position of the advance of the last glyph.
-     */
-    float *fPositions;
+    LEGlyphStorage *fGlyphStorage;
 
     /**
      * The font instance for the text font.
      *
      * @see LEFontInstance
+     *
+     * @internal
      */
     const LEFontInstance *fFontInstance;
 
@@ -99,6 +86,8 @@ protected:
      * The script code for the text
      *
      * @see ScriptAndLanguageTags.h for script codes.
+     *
+     * @internal
      */
     le_int32 fScriptCode;
 
@@ -106,29 +95,77 @@ protected:
      * The langauge code for the text
      *
      * @see ScriptAndLanguageTags.h for language codes.
+     *
+     * @internal
      */
     le_int32 fLanguageCode;
 
+    /**
+     * The typographic control flags
+     *
+     * @internal
+     */
+    le_int32 fTypoFlags;
+
+    /**
+     * <code>TRUE</code> if <code>mapCharsToGlyphs</code> should replace ZWJ / ZWNJ with a glyph
+     * with no contours.
+     *
+     * @internal
+     */
+    le_bool fFilterZeroWidth;
+
     /**
      * This constructs an instance for a given font, script and language. Subclass constructors
      * must call this constructor.
      *
      * @param fontInstance - the font for the text
      * @param scriptCode - the script for the text
-     * @param langaugeCode - the language for the text
+     * @param languageCode - the language for the text
+     * @param typoFlags - the typographic control flags for the text.  Set bit 1 if kerning
+     * is desired, set bit 2 if ligature formation is desired.  Others are reserved.
      *
      * @see LEFontInstance
      * @see ScriptAndLanguageTags.h
+     *
+     * @internal
      */
-    LayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode);
+    LayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, le_int32 typoFlags);
 
     /**
      * This overrides the default no argument constructor to make it
      * difficult for clients to call it. Clients are expected to call
      * layoutEngineFactory.
+     *
+     * @internal
      */
     LayoutEngine();
 
+    /**
+     * This method does any required pre-processing to the input characters. It
+     * may generate output characters that differ from the input charcters due to
+     * insertions, deletions, or reorderings. In such cases, it will also generate an
+     * output character index array reflecting these changes.
+     *
+     * Subclasses must override this method.
+     *
+     * Input parameters:
+     * @param chars - the input character context
+     * @param offset - the index of the first character to process
+     * @param count - the number of characters to process
+     * @param max - the number of characters in the input context
+     * @param rightToLeft - TRUE if the characters are in a right to left directional run
+     * @param outChars - the output character array, if different from the input
+     * @param glyphStorage - the object that holds the per-glyph storage. The character index array may be set.
+     * @param success - set to an error code if the operation fails
+     *
+     * @return the output character count (input character count if no change)
+     *
+     * @internal
+     */
+    virtual le_int32 characterProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
+            LEUnicode *&outChars, LEGlyphStorage &glyphStorage, LEErrorCode &success);
+
     /**
      * This method does the glyph processing. It converts an array of characters
      * into an array of glyph indices and character indices. The characters to be
@@ -144,16 +181,18 @@ protected:
      * @param offset - the offset of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the context.
-     * @param rightToLeft - true if the text is in a right to left directional run
+     * @param rightToLeft - TRUE if the text is in a right to left directional run
+     * @param glyphStorage - the object which holds the per-glyph storage. The glyph and char indices arrays
+     *                       will be set.
      *
      * Output parameters:
-     * @param glyphs - the glyph index array
-     * @param charIndices - the character index array
      * @param success - set to an error code if the operation fails
      *
      * @return the number of glyphs in the glyph index array
+     *
+     * @internal
      */
-    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
+    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method does basic glyph positioning. The default implementation positions
@@ -161,15 +200,14 @@ protected:
      * is not expected that many subclasses will override this method.
      *
      * Input parameters:
-     * @param glyphs - the input glyph array
-     * @param glyphCount - the number of glyphs in the glyph array
+     * @param glyphStorage - the object which holds the per-glyph storage. The glyph position array will be set.
      * @param x - the starting X position
      * @param y - the starting Y position
+     * @param success - set to an error code if the operation fails
      *
-     * Output parameters:
-     * @param positions - the output X and Y positions (two entries per glyph)
+     * @internal
      */
-    virtual void positionGlyphs(const LEGlyphID glyphs[], le_int32 glyphCount, float x, float y, float *&positions, LEErrorCode &success);
+    virtual void positionGlyphs(LEGlyphStorage &glyphStorage, float x, float y, LEErrorCode &success);
 
     /**
      * This method does positioning adjustments like accent positioning and
@@ -184,29 +222,14 @@ protected:
      * @param chars - the input character context
      * @param offset - the offset of the first character to process
      * @param count - the number of characters to process
-     * @param reverse - true if the glyphs in the glyph array have been reordered
-     * @param glyphs - the input glyph array
-     * @param glyphCount - the number of glyphs
-     * @param positions - the position array, will be updated as needed
+     * @param reverse - <code>TRUE</code> if the glyphs in the glyph array have been reordered
+     * @param glyphStorage - the object which holds the per-glyph storage. The glyph positions will be
+     *                       adjusted as needed.
      * @param success - output parameter set to an error code if the operation fails
      *
-     * Note: the positions are passed as a plain array because this method should
-     * not need to reallocate them.
+     * @internal
      */
-    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphID glyphs[], le_int32 glyphCount, float positions[], LEErrorCode &success)
-    {
-        if (LE_FAILURE(success)) {
-            return;
-        }
-
-        if (chars == NULL || glyphs == NULL || positions == NULL || offset < 0 || count < 0) {
-            success = LE_ILLEGAL_ARGUMENT_ERROR;
-            return;
-        }
-
-        // default is no adjustments
-        return;
-    };
+    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This method gets a table from the font associated with
@@ -217,6 +240,8 @@ protected:
      * @param tableTag - the four byte table tag.
      *
      * @return the address of the table.
+     *
+     * @internal
      */
     virtual const void *getFontTable(LETag tableTag) const;
 
@@ -235,32 +260,53 @@ protected:
      * @param chars - the input character context
      * @param offset - the offset of the first character to be mapped
      * @param count - the number of characters to be mapped
-     * @param reverse - if true, the output will be in reverse order
-     * @param mirror - if true, do character mirroring
-     *
-     * Output parameters:
-     * @param glyphs - the glyph array
-     * @param charIndices - the character index array
+     * @param reverse - if <code>TRUE</code>, the output will be in reverse order
+     * @param mirror - if <code>TRUE</code>, do character mirroring
+     * @param glyphStorage - the object which holds the per-glyph storage. The glyph and char
+     *                       indices arrays will be filled in.
      * @param success - set to an error code if the operation fails
      *
      * @see LEFontInstance
+     *
+     * @internal
      */
-    virtual void mapCharsToGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, le_bool mirror, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
+    virtual void mapCharsToGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, le_bool mirror, LEGlyphStorage &glyphStorage, LEErrorCode &success);
 
     /**
      * This is a convenience method that forces the advance width of mark
      * glyphs to be zero, which is required for proper selection and highlighting.
      * 
-     * @param glyphs - the glyph array
-     * @param glyphCount - the number of glyphs
-     * @param reverse - true if the glyph array has been reordered
+     * @param glyphStorage - the object containing the per-glyph storage. The positions array will be modified.
      * @param markFilter - used to identify mark glyphs
-     * @param positions - the glyph position array - updated as required
      * @param success - output parameter set to an error code if the operation fails
      *
      * @see LEGlyphFilter
+     *
+     * @internal
      */
-    static void adjustMarkGlyphs(const LEGlyphID glyphs[], le_int32 glyphCount, le_bool reverse, LEGlyphFilter *markFilter, float positions[], LEErrorCode &success);
+    static void adjustMarkGlyphs(LEGlyphStorage &glyphStorage, LEGlyphFilter *markFilter, LEErrorCode &success);
+
+
+    /**
+     * This is a convenience method that forces the advance width of mark
+     * glyphs to be zero, which is required for proper selection and highlighting.
+     * This method uses the input characters to identify marks. This is required in
+     * cases where the font does not contain enough information to identify them based
+     * on the glyph IDs.
+     * 
+     * @param chars - the array of input characters
+     * @param charCount - the number of input characers
+     * @param glyphStorage - the object containing the per-glyph storage. The positions array will be modified.
+     * @param reverse - <code>TRUE</code> if the glyph array has been reordered
+     * @param markFilter - used to identify mark glyphs
+     * @param success - output parameter set to an error code if the operation fails
+     *
+     * @see LEGlyphFilter
+     *
+     * @internal
+     */
+    static void adjustMarkGlyphs(const LEUnicode chars[], le_int32 charCount, le_bool reverse, LEGlyphStorage &glyphStorage, LEGlyphFilter *markFilter, LEErrorCode &success);
+
 
 public:
     /**
@@ -268,27 +314,35 @@ public:
      * glyph, character index and position arrays by calling the reset
      * method. It is declared virtual so that it will be invoked by the
      * subclass destructors.
+     *
+     * @stable ICU 2.8
      */
     virtual ~LayoutEngine();
 
     /**
      * This method will invoke the layout steps in their correct order by calling
-     * the computeGlyphs, positionGlyphs and adjustGlyphPosition methods.. It will
+     * the computeGlyphs, positionGlyphs and adjustGlyphPosition methods. It will
      * compute the glyph, character index and position arrays.
      *
      * @param chars - the input character context
      * @param offset - the offset of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the characers are in a right to left directional run
+     * @param rightToLeft - TRUE if the characers are in a right to left directional run
      * @param x - the initial X position
      * @param y - the initial Y position
      * @param success - output parameter set to an error code if the operation fails
      *
      * @return the number of glyphs in the glyph array
      *
-     * Note; the glyph, character index and position array can be accessed
-     * using the getter method below.
+     * Note: The glyph, character index and position array can be accessed
+     * using the getter methods below.
+     *
+     * Note: If you call this method more than once, you must call the reset()
+     * method first to free the glyph, character index and position arrays
+     * allocated by the previous call.
+     *
+     * @stable ICU 2.8
      */
     virtual le_int32 layoutChars(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, float x, float y, LEErrorCode &success);
 
@@ -298,11 +352,10 @@ public:
      * of characters used to create the LayoutEngine.
      *
      * @return the number of glyphs in the glyph array
+     *
+     * @stable ICU 2.8
      */
-    le_int32 getGlyphCount() const
-    {
-        return fGlyphCount;
-    };
+    le_int32 getGlyphCount() const;
 
     /**
      * This method copies the glyph array into a caller supplied array.
@@ -311,24 +364,10 @@ public:
      *
      * @param glyphs - the destiniation glyph array
      * @param success - set to an error code if the operation fails
+     *
+     * @stable ICU 2.8
      */
-    void getGlyphs(LEGlyphID glyphs[], LEErrorCode &success) const
-    {
-        if (LE_FAILURE(success)) {
-            return;
-        }
-
-        if (glyphs == NULL) {
-            success = LE_ILLEGAL_ARGUMENT_ERROR;
-            return;
-        }
-
-        if (fGlyphs == NULL) {
-            success = LE_NO_LAYOUT_ERROR;
-        }
-
-        LE_ARRAY_COPY(glyphs, fGlyphs, fGlyphCount);
-    };
+    void getGlyphs(LEGlyphID glyphs[], LEErrorCode &success) const;
 
     /**
      * This method copies the glyph array into a caller supplied array,
@@ -339,6 +378,8 @@ public:
      * @param glyphs - the destination (32 bit) glyph array
      * @param extraBits - this value will be ORed with each glyph index
      * @param success - set to an error code if the operation fails
+     *
+     * @stable ICU 2.8
      */
     virtual void getGlyphs(le_uint32 glyphs[], le_uint32 extraBits, LEErrorCode &success) const;
 
@@ -349,25 +390,10 @@ public:
      *
      * @param charIndices - the destiniation character index array
      * @param success - set to an error code if the operation fails
+     *
+     * @stable ICU 2.8
      */
-    void getCharIndices(le_int32 charIndices[], LEErrorCode &success) const
-    {
-        if LE_FAILURE(success) {
-            return;
-        }
-
-        if (charIndices == NULL) {
-            success = LE_ILLEGAL_ARGUMENT_ERROR;
-            return;
-        }
-
-        if (fCharIndices == NULL) {
-            success = LE_NO_LAYOUT_ERROR;
-            return;
-        }
-
-        LE_ARRAY_COPY(charIndices, fCharIndices, fGlyphCount);
-    };
+    void getCharIndices(le_int32 charIndices[], LEErrorCode &success) const;
 
     /**
      * This method copies the character index array into a caller supplied array.
@@ -377,6 +403,8 @@ public:
      * @param charIndices - the destiniation character index array
      * @param indexBase - an offset which will be added to each index
      * @param success - set to an error code if the operation fails
+     *
+     * @stable ICU 2.8
      */
     void getCharIndices(le_int32 charIndices[], le_int32 indexBase, LEErrorCode &success) const;
 
@@ -386,27 +414,12 @@ public:
      * X and Y position for each glyph, plus an extra X and Y for the
      * advance of the last glyph.
      *
-     * @param glyphs - the destiniation position array
+     * @param positions - the destiniation position array
      * @param success - set to an error code if the operation fails
+     *
+     * @stable ICU 2.8
      */
-    void getGlyphPositions(float positions[], LEErrorCode &success) const
-    {
-        if LE_FAILURE(success) {
-            return;
-        }
-
-        if (positions == NULL) {
-            success = LE_ILLEGAL_ARGUMENT_ERROR;
-            return;
-        }
-
-        if (fPositions == NULL) {
-            success = LE_NO_LAYOUT_ERROR;
-            return;
-        }
-
-        LE_ARRAY_COPY(positions, fPositions, fGlyphCount * 2 + 2);
-    };
+    void getGlyphPositions(float positions[], LEErrorCode &success) const;
 
     /**
      * This method returns the X and Y position of the glyph at
@@ -420,31 +433,16 @@ public:
      * @param y - the glyph's Y position
      * @param success - set to an error code if the operation fails
      *
+     * @stable ICU 2.8
      */
-    void getGlyphPosition(le_int32 glyphIndex, float &x, float &y, LEErrorCode &success) const
-    {
-        if (LE_FAILURE(success)) {
-            return;
-        }
-
-        if (glyphIndex > fGlyphCount) {
-            success = LE_INDEX_OUT_OF_BOUNDS_ERROR;
-            return;
-        }
-
-        if (fPositions == NULL) {
-            success = LE_NO_LAYOUT_ERROR;
-            return;
-        }
-
-        x = fPositions[glyphIndex * 2];
-        y = fPositions[glyphIndex * 2 + 1];
-    };
+    void getGlyphPosition(le_int32 glyphIndex, float &x, float &y, LEErrorCode &success) const;
 
     /**
      * This method frees the glyph, character index and position arrays
      * so that the LayoutEngine can be reused to layout a different
      * characer array. (This method is also called by the destructor)
+     *
+     * @stable ICU 2.8
      */
     virtual void reset();
 
@@ -455,15 +453,37 @@ public:
      *
      * @param fontInstance - the font of the text
      * @param scriptCode - the script of the text
-     * @param langaugeCode - the language of the text
+     * @param languageCode - the language of the text
      * @param success - output parameter set to an error code if the operation fails
      *
      * @return a LayoutEngine which can layout text in the given font.
      *
      * @see LEFontInstance
+     *
+     * @stable ICU 2.8
      */
     static LayoutEngine *layoutEngineFactory(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, LEErrorCode &success);
 
+    /**
+     * Override of existing call that provides flags to control typography.
+     * @stable ICU 3.4
+     */
+    static LayoutEngine *layoutEngineFactory(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, le_int32 typo_flags, LEErrorCode &success);
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.8
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.8
+     */
+    static UClassID getStaticClassID();
+
 };
 
 U_NAMESPACE_END
