@@ -332,16 +332,25 @@ void OpenTypeLayoutEngine::adjustGlyphPositions(const LEUnicode chars[], le_int3
         delete adjustments;
     }
 
-    LEGlyphID zwnj = fFontInstance->mapCharToGlyph(0x200C);
-    LEGlyphID nbsp = fFontInstance->mapCharToGlyph(0x00A0);
+    LEGlyphID zwnj  = fFontInstance->mapCharToGlyph(0x200C);
+#if 0
+    // The nbsp translation was only here to make one
+    // broken font work. Not a good idea in general... 
+    LEGlyphID nbsp  = fFontInstance->mapCharToGlyph(0x00A0);
+    LEGlyphID space = fFontInstance->mapCharToGlyph(0x0020);
+#endif
 
-    for (le_int32 g = 0; g < glyphCount; g += 1) {
-        LEGlyphID glyph = glyphStorage[g];
+    if (zwnj != 0x0000) {
+        for (le_int32 g = 0; g < glyphCount; g += 1) {
+            LEGlyphID glyph = glyphStorage[g];
 
-        if (glyph == zwnj) {
-            glyphStorage[g] = LE_SET_GLYPH(glyph, 0xFFFF);
-        } else if (glyph == nbsp) {
-            glyphStorage[g] = LE_SET_GLYPH(glyph, 0x0003);
+            if (glyph == zwnj) {
+                glyphStorage[g] = LE_SET_GLYPH(glyph, 0xFFFF);
+    #if 0
+            } else if (glyph == nbsp) {
+                glyphStorage[g] = LE_SET_GLYPH(glyph, space);
+    #endif
+            }
         }
     }
 
