@@ -1,13 +1,17 @@
 /*
- * @(#)TrimmedArrayProcessor.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
  *
  */
 
 #ifndef __TRIMMEDARRAYPROCESSOR_H
 #define __TRIMMEDARRAYPROCESSOR_H
 
+/**
+ * \file
+ * \internal
+ */
+
 #include "LETypes.h"
 #include "MorphTables.h"
 #include "SubtableProcessor.h"
@@ -16,22 +20,39 @@
 
 U_NAMESPACE_BEGIN
 
+class LEGlyphStorage;
+
 class TrimmedArrayProcessor : public NonContextualGlyphSubstitutionProcessor
 {
 public:
-    virtual void process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyphCount);
+    virtual void process(LEGlyphStorage &glyphStorage);
 
     TrimmedArrayProcessor(const MorphSubtableHeader *morphSubtableHeader);
 
     virtual ~TrimmedArrayProcessor();
 
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.8
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.8
+     */
+    static UClassID getStaticClassID();
+
 private:
     TrimmedArrayProcessor();
 
 protected:
-    le_int16 firstGlyph;
-    le_int16 lastGlyph;
+    TTGlyphID firstGlyph;
+    TTGlyphID lastGlyph;
     const TrimmedArrayLookupTable *trimmedArrayLookupTable;
+
 };
 
 U_NAMESPACE_END
