@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2007, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 //===============================================================================
@@ -225,11 +225,16 @@ CollationAPITest::TestProperty(/* char* par */)
     if (U_FAILURE(success))
     {
         errln("Creating French collator failed.");
-        delete col; delete junk;
+        delete col;
+        delete junk;
         return;
     }
 
-    doAssert((*frCol != *junk), "The junk is the same as the French collator.");
+    // If the default locale isn't French, the French and non-French collators
+    // should be different
+    if (frCol->getLocale(ULOC_ACTUAL_LOCALE, success) != Locale::getFrench()) {
+        doAssert((*frCol != *junk), "The junk is the same as the French collator.");
+    }
     Collator *aFrCol = frCol->clone();
     doAssert((*frCol == *aFrCol), "The cloning of a French collator failed.");
     logln("Collator property test ended.");
@@ -1017,15 +1022,61 @@ CollationAPITest::TestCompare(/* char* par */)
 void
 CollationAPITest::TestGetAll(/* char* par */)
 {
-    int32_t count;
-    const Locale* list = Collator::getAvailableLocales(count);
-    for (int32_t i = 0; i < count; ++i) {
-        UnicodeString locName, dispName;
-        log("Locale name: ");
-        log(list[i].getName());
-        log(" , the display name is : ");
-        logln(list[i].getDisplayName(dispName));
+    int32_t count1, count2;
+    UErrorCode status = U_ZERO_ERROR;
+
+    logln("Trying Collator::getAvailableLocales(int&)");
+
+    const Locale* list = Collator::getAvailableLocales(count1);
+    for (int32_t i = 0; i < count1; ++i) {
+        UnicodeString dispName;
+        logln(UnicodeString("Locale name: ")
+            + UnicodeString(list[i].getName())
+            + UnicodeString(" , the display name is : ")
+            + UnicodeString(list[i].getDisplayName(dispName)));
     }
+
+    if (count1 == 0 || list == NULL) {
+        errln("getAvailableLocales(int&) returned an empty list");
+    }
+
+    logln("Trying Collator::getAvailableLocales()");
+    StringEnumeration* localeEnum = Collator::getAvailableLocales();
+    const UnicodeString* locStr;
+    const char *locCStr;
+    count2 = 0;
+
+    if (localeEnum == NULL) {
+        errln("getAvailableLocales() returned NULL");
+        return;
+    }
+
+    while ((locStr = localeEnum->snext(status)) != NULL)
+    {
+        logln(UnicodeString("Locale name is: ") + *locStr);
+        count2++;
+    }
+    if (count1 != count2) {
+        errln("getAvailableLocales(int&) returned %d and getAvailableLocales() returned %d", count1, count2);
+    }
+
+    logln("Trying Collator::getAvailableLocales() clone");
+    count1 = 0;
+    StringEnumeration* localeEnum2 = localeEnum->clone();
+    localeEnum2->reset(status);
+    while ((locCStr = localeEnum2->next(NULL, status)) != NULL)
+    {
+        logln(UnicodeString("Locale name is: ") + UnicodeString(locCStr));
+        count1++;
+    }
+    if (count1 != count2) {
+        errln("getAvailableLocales(3rd time) returned %d and getAvailableLocales(2nd time) returned %d", count1, count2);
+    }
+    if (localeEnum->count(status) != count1) {
+        errln("localeEnum->count() returned %d and getAvailableLocales() returned %d", localeEnum->count(status), count1);
+    }
+    delete localeEnum;
+    delete localeEnum2;
 }
 
 void CollationAPITest::TestSortKey()
@@ -1627,7 +1678,7 @@ void CollationAPITest::TestBounds(void) {
     uint8_t sortkey[512], lower[512], upper[512];
     UChar buffer[512];
 
-    const char *test[] = {
+    static const char * const test[] = {
         "John Smith",
         "JOHN SMITH",
         "john SMITH",
@@ -1638,7 +1689,7 @@ void CollationAPITest::TestBounds(void) {
         "John Smithsonian"
     };
 
-    static struct teststruct tests[] = {
+    struct teststruct tests[] = {
         {"\\u010CAKI MIHALJ", {0}},
         {"\\u010CAKI MIHALJ", {0}},
         {"\\u010CAKI PIRO\\u0160KA", {0}},
@@ -2187,6 +2238,12 @@ void CollationAPITest::TestClone() {
     logln("\ninit c0");
     UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator* c0 = (RuleBasedCollator*)Collator::createInstance(status);
+
+    if (U_FAILURE(status)) {
+        errln("Collator::CreateInstance(status) failed with %s", u_errorName(status));
+        return;
+    }
+
     c0->setStrength(Collator::TERTIARY);
     dump("c0", c0, status);
 
