@@ -1,6 +1,6 @@
 /************************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation
+ * Copyright (c) 1997-2007, International Business Machines Corporation
  * and others. All Rights Reserved.
  ************************************************************************/
 
@@ -9,9 +9,11 @@
 #if !UCONFIG_NO_FORMATTING
 
 #include "caltest.h"
+#include "unicode/dtfmtsym.h"
 #include "unicode/gregocal.h"
 #include "unicode/smpdtfmt.h"
 #include "unicode/simpletz.h"
+#include "unicode/dbgutil.h"
 
 // *****************************************************************************
 // class CalendarTest
@@ -194,6 +196,13 @@ void CalendarTest::runIndexedTest( int32_t index, UBool exec, const char* &name,
             TestJD();
           }
           break;
+        case 21:
+          name = "TestDebug";
+          if(exec) {
+            logln("TestDebug---"); logln("");
+            TestDebug();
+          }
+          break;
            
         default: name = ""; break;
     }
@@ -1288,9 +1297,16 @@ CalendarTest::TestDOW_LOCALandYEAR_WOY()
     if (U_FAILURE(status)) { errln("Couldn't create GregorianCalendar"); return; }
     SimpleDateFormat *sdf=new SimpleDateFormat(UnicodeString("YYYY'-W'ww-ee"), Locale::getGermany(), status);
     if (U_FAILURE(status)) { errln("Couldn't create SimpleDateFormat"); return; }
+
+    // ICU no longer use localized date-time pattern characters by default.
+    // So we set pattern chars using 'J' instead of 'Y'.
+    DateFormatSymbols *dfs = new DateFormatSymbols(Locale::getGermany(), status);
+    dfs->setLocalPatternChars(UnicodeString("GyMdkHmsSEDFwWahKzJeugAZvcLQq"));
+    sdf->adoptDateFormatSymbols(dfs);
     sdf->applyLocalizedPattern(UnicodeString("JJJJ'-W'ww-ee"), status);
     if (U_FAILURE(status)) { errln("Couldn't apply localized pattern"); return; }
-    cal->clear();
+
+	cal->clear();
     cal->set(1997, UCAL_DECEMBER, 25);
     doYEAR_WOYLoop(cal, sdf, times, status);
     //loop_addroll(cal, /*sdf,*/ times, UCAL_YEAR_WOY, UCAL_YEAR,  status);
@@ -1872,6 +1888,45 @@ void CalendarTest::TestJD()
 
 }
 
+// make sure the ctestfw utilities are in sync with the Calendar
+void CalendarTest::TestDebug()
+{
+	for(int32_t  t=0;t<=UDBG_ENUM_COUNT;t++) {
+		int32_t count = udbg_enumCount((UDebugEnumType)t);
+		if(count == -1) {
+			logln("enumCount(%d) returned -1", count);
+			continue;
+		}
+	    for(int32_t i=0;i<=count;i++) {
+	  	  if(i<count) {
+	  		  if( i!=udbg_enumArrayValue((UDebugEnumType)t, i)) {
+	  			  errln("FAIL: udbg_enumArrayValue(%d,%d) returned %d, expected %d", t, i, udbg_enumArrayValue((UDebugEnumType)t,i), i);
+	  		  }
+	  	  } else {
+	  		  logln("Testing count+1:");
+	  	  }
+                  const char *name = udbg_enumName((UDebugEnumType)t,i);
+                  if(name==NULL) {
+                          if(i==count) {
+                                logln(" null name - expected.\n");
+                          } else {
+                                errln("FAIL: udbg_enumName(%d,%d) returned NULL", t, i);
+                          }
+                          name = "(null)";
+                  }
+		  logln("udbg_enumArrayValue(%d,%d) = %s, returned %d", t, i, 
+				  	name, udbg_enumArrayValue((UDebugEnumType)t,i));
+	  	  logln("udbg_enumString = " + udbg_enumString((UDebugEnumType)t,i));
+	    }
+	    if(udbg_enumExpectedCount((UDebugEnumType)t) != count) {
+	  	  errln("FAIL: udbg_enumExpectedCount(%d): %d, != UCAL_FIELD_COUNT=%d ", t, udbg_enumExpectedCount((UDebugEnumType)t), count);
+	    } else {
+	  	  logln("udbg_ucal_fieldCount: %d, UCAL_FIELD_COUNT=udbg_enumCount %d ", udbg_enumExpectedCount((UDebugEnumType)t), count);
+	    }
+	}
+}
+
+
 #undef CHECK
 
 // List of interesting locales
@@ -1933,6 +1988,7 @@ UDate CalendarTest::minDateOfCalendar(const Calendar& cal, UBool &isGregorian, U
 
 
 
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 //eof
