@@ -1,9 +1,12 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
 
 #include "unicode/coll.h"
 #include "unicode/tblcoll.h"
@@ -15,7 +18,7 @@ CollationEnglishTest::CollationEnglishTest()
 : myCollation(0)
 {
     UErrorCode status = U_ZERO_ERROR;
-    myCollation = Collator::createInstance(Locale::ENGLISH, status);
+    myCollation = Collator::createInstance(Locale::getEnglish(), status);
 }
 
 CollationEnglishTest::~CollationEnglishTest()
@@ -23,7 +26,7 @@ CollationEnglishTest::~CollationEnglishTest()
     delete myCollation;
 }
 
-const UChar CollationEnglishTest::testSourceCases[][CollationEnglishTest::MAX_TOKEN_LEN] = {
+static const UChar testSourceCases[][CollationEnglishTest::MAX_TOKEN_LEN] = {
         {(UChar)0x0061 /* 'a' */, (UChar)0x0062 /* 'b' */, 0},
         {(UChar)0x0062 /* 'b' */, (UChar)0x006C /* 'l' */, (UChar)0x0061 /* 'a' */, (UChar)0x0063 /* 'c' */, (UChar)0x006B /* 'k' */, (UChar)0x002D /* '-' */, (UChar)0x0062 /* 'b' */, (UChar)0x0069 /* 'i' */, (UChar)0x0072 /* 'r' */, (UChar)0x0064 /* 'd' */, 0},
         {(UChar)0x0062 /* 'b' */, (UChar)0x006C /* 'l' */, (UChar)0x0061 /* 'a' */, (UChar)0x0063 /* 'c' */, (UChar)0x006B /* 'k' */, (UChar)0x0020 /* ' ' */, (UChar)0x0062 /* 'b' */, (UChar)0x0069 /* 'i' */, (UChar)0x0072 /* 'r' */, (UChar)0x0064 /* 'd' */, 0},
@@ -75,7 +78,7 @@ const UChar CollationEnglishTest::testSourceCases[][CollationEnglishTest::MAX_TO
         {(UChar)0x0070 /* 'p' */, 0x00E9, (UChar)0x0063 /* 'c' */, (UChar)0x0068 /* 'h' */, 0x00E9, 0}                                            // 49
 };
 
-const UChar CollationEnglishTest::testTargetCases[][CollationEnglishTest::MAX_TOKEN_LEN] = {
+static const UChar testTargetCases[][CollationEnglishTest::MAX_TOKEN_LEN] = {
         {(UChar)0x0061 /* 'a' */, (UChar)0x0062 /* 'b' */, (UChar)0x0063 /* 'c' */, 0},
         {(UChar)0x0062 /* 'b' */, (UChar)0x006C /* 'l' */, (UChar)0x0061 /* 'a' */, (UChar)0x0063 /* 'c' */, (UChar)0x006B /* 'k' */, (UChar)0x0062 /* 'b' */, (UChar)0x0069 /* 'i' */, (UChar)0x0072 /* 'r' */, (UChar)0x0064 /* 'd' */, 0},
         {(UChar)0x0062 /* 'b' */, (UChar)0x006C /* 'l' */, (UChar)0x0061 /* 'a' */, (UChar)0x0063 /* 'c' */, (UChar)0x006B /* 'k' */, (UChar)0x002D /* '-' */, (UChar)0x0062 /* 'b' */, (UChar)0x0069 /* 'i' */, (UChar)0x0072 /* 'r' */, (UChar)0x0064 /* 'd' */, 0},
@@ -127,7 +130,7 @@ const UChar CollationEnglishTest::testTargetCases[][CollationEnglishTest::MAX_TO
         {(UChar)0x0070 /* 'p' */, 0x00EA, (UChar)0x0063 /* 'c' */, (UChar)0x0068 /* 'h' */, (UChar)0x0065 /* 'e' */, 0}                                           // 49
 };
 
-const Collator::EComparisonResult CollationEnglishTest::results[] = {
+static const Collator::EComparisonResult results[] = {
         Collator::LESS, 
         Collator::LESS, /*Collator::GREATER,*/
         Collator::LESS,
@@ -183,7 +186,7 @@ const Collator::EComparisonResult CollationEnglishTest::results[] = {
         Collator::LESS                                                                  // 49
 };
 
-const UChar CollationEnglishTest::testBugs[][CollationEnglishTest::MAX_TOKEN_LEN] = {
+static const UChar testBugs[][CollationEnglishTest::MAX_TOKEN_LEN] = {
     {0x61, 0},
     {0x41, 0},
     {0x65, 0},
@@ -198,7 +201,7 @@ const UChar CollationEnglishTest::testBugs[][CollationEnglishTest::MAX_TOKEN_LEN
 
 // 0x0300 is grave, 0x0301 is acute
 // the order of elements in this array must be different than the order in CollationFrenchTest
-const UChar CollationEnglishTest::testAcute[][CollationEnglishTest::MAX_TOKEN_LEN] = {
+static const UChar testAcute[][CollationEnglishTest::MAX_TOKEN_LEN] = {
     {0x65, 0x65, 0},
     {0x65, 0x65, 0x0301, 0},
     {0x65, 0x65, 0x0301, 0x0300, 0},
@@ -228,39 +231,22 @@ const UChar CollationEnglishTest::testAcute[][CollationEnglishTest::MAX_TOKEN_LE
 
 static const UChar testMore[][CollationEnglishTest::MAX_TOKEN_LEN] = {
     {(UChar)0x0061 /* 'a' */, (UChar)0x0065 /* 'e' */, 0},
-    {(UChar)0x0061 /* 'a' */, (UChar)0x0066 /* 'f' */, 0},
     { 0x00E6, 0},
     { 0x00C6, 0},
+    {(UChar)0x0061 /* 'a' */, (UChar)0x0066 /* 'f' */, 0},
     {(UChar)0x006F /* 'o' */, (UChar)0x0065 /* 'e' */, 0},
     { 0x0153, 0},
     { 0x0152, 0},
     {(UChar)0x006F /* 'o' */, (UChar)0x0066 /* 'f' */, 0},
 };
 
-void CollationEnglishTest::doTest( UnicodeString source, UnicodeString target, Collator::EComparisonResult result)
-{
-    Collator::EComparisonResult compareResult = myCollation->compare(source, target);
-    CollationKey sortKey1, sortKey2;
-    UErrorCode key1status = U_ZERO_ERROR, key2status = U_ZERO_ERROR; //nos
-    myCollation->getCollationKey(source, /*nos*/ sortKey1, key1status );
-    myCollation->getCollationKey(target, /*nos*/ sortKey2, key2status );
-    if (U_FAILURE(key1status) || U_FAILURE(key2status))
-    {
-        errln("SortKey generation Failed.\n");
-        return;
-    }
-
-    Collator::EComparisonResult keyResult = sortKey1.compareTo(sortKey2);
-    reportCResult( source, target, sortKey1, sortKey2, compareResult, keyResult, compareResult, result );
-}
-
 void CollationEnglishTest::TestTertiary(/* char* par */)
 {
     int32_t i = 0;
     myCollation->setStrength(Collator::TERTIARY);
     for (i = 0; i < 38 ; i++)
     {
-        doTest(testSourceCases[i], testTargetCases[i], results[i]);
+        doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
 
     int32_t j = 0;
@@ -268,7 +254,7 @@ void CollationEnglishTest::TestTertiary(/* char* par */)
     {
         for (j = i+1; j < 10; j++)
         {
-            doTest(testBugs[i], testBugs[j], Collator::LESS);
+            doTest(myCollation, testBugs[i], testBugs[j], Collator::LESS);
         }
     }
 
@@ -285,7 +271,7 @@ void CollationEnglishTest::TestTertiary(/* char* par */)
                 expected = Collator::EQUAL;
             else // (i >  j)
                 expected = Collator::GREATER;
-            doTest(testMore[i], testMore[j], expected );
+            doTest(myCollation, testMore[i], testMore[j], expected );
         }
     }
 
@@ -297,7 +283,7 @@ void CollationEnglishTest::TestPrimary(/* char* par */)
     myCollation->setStrength(Collator::PRIMARY);
     for (i = 38; i < 43 ; i++)
     {
-        doTest(testSourceCases[i], testTargetCases[i], results[i]);
+        doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
 }
 
@@ -307,7 +293,7 @@ void CollationEnglishTest::TestSecondary(/* char* par */)
     myCollation->setStrength(Collator::SECONDARY);
     for (i = 43; i < 49 ; i++)
     {
-        doTest(testSourceCases[i], testTargetCases[i], results[i]);
+        doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
 
     //test acute and grave ordering (compare to french collation)
@@ -324,7 +310,7 @@ void CollationEnglishTest::TestSecondary(/* char* par */)
                 expected = Collator::EQUAL;
             else // (i >  j)
                 expected = Collator::GREATER;
-            doTest(testAcute[i], testAcute[j], expected );
+            doTest(myCollation, testAcute[i], testAcute[j], expected );
         }
     }
 }
@@ -332,12 +318,17 @@ void CollationEnglishTest::TestSecondary(/* char* par */)
 void CollationEnglishTest::runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par*/ )
 {
     if (exec) logln("TestSuite CollationEnglishTest: ");
-    switch (index) {
-        case 0: name = "TestPrimary";   if (exec)   TestPrimary(/* par */); break;
-        case 1: name = "TestSecondary"; if (exec)   TestSecondary(/* par */); break;
-        case 2: name = "TestTertiary";  if (exec)   TestTertiary(/* par */); break;
-        default: name = ""; break;
+    if(myCollation) {
+      switch (index) {
+          case 0: name = "TestPrimary";   if (exec)   TestPrimary(/* par */); break;
+          case 1: name = "TestSecondary"; if (exec)   TestSecondary(/* par */); break;
+          case 2: name = "TestTertiary";  if (exec)   TestTertiary(/* par */); break;
+          default: name = ""; break;
+      }
+    } else {
+      errln("Collator couldn't be instantiated!");
+      name = "";
     }
 }
 
-
+#endif /* #if !UCONFIG_NO_COLLATION */
