@@ -1,9 +1,13 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2005, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "pptest.h"
 
 #include "unicode/numfmt.h"
@@ -87,6 +91,11 @@ void ParsePositionTest::TestParsePosition()
         errln("*** PP operator= operator== or operator != result");
     }
 
+    ParsePosition *ppp = pp5.clone();
+    if(ppp == &pp5 || *ppp != pp5) {
+        errln("ParsePosition.clone() failed");
+    }
+    delete ppp;
 }
 
 void ParsePositionTest::TestFieldPosition() 
@@ -99,6 +108,13 @@ void ParsePositionTest::TestFieldPosition()
         errln("*** FP constructor(int) or getField");
     }
 
+    FieldPosition fpc(fp);
+    if (fpc.getField() == 7) {
+        logln("FP Constructor(FP&) passed");
+    } else {
+        errln("*** FP Constructor(FP&)");
+    }
+
     FieldPosition fph( 3 );
     if ( fph.getField() != 3) 
         errln("*** FP getField or heap constr.");
@@ -131,6 +147,12 @@ void ParsePositionTest::TestFieldPosition()
     }
 
     logln("");
+
+    FieldPosition *pfp = fp.clone();
+    if(pfp == &fp || *pfp != fp) {
+        errln("FieldPosition.clone() failed");
+    }
+    delete pfp;
 }
 
 void ParsePositionTest::TestFieldPosition_example() 
@@ -151,7 +173,10 @@ void ParsePositionTest::TestFieldPosition_example()
 
     UErrorCode status = U_ZERO_ERROR;
     NumberFormat *nf = NumberFormat::createInstance(status);
-    failure(status, "NumberFormat::createInstance");
+    if (failure(status, "NumberFormat::createInstance")){
+        delete nf;
+        return;
+    };
 
     if(nf->getDynamicClassID() != DecimalFormat::getStaticClassID()) {
         errln("NumberFormat::createInstance returned unexpected class type");
@@ -198,3 +223,5 @@ void ParsePositionTest::Test4109023()
     if (fp != fp2)
         errln("Error : FieldPosition.equals() failed");
 }
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
