@@ -1,11 +1,14 @@
 
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "sdtfmtts.h"
 
 #include "unicode/smpdtfmt.h"
@@ -23,11 +26,13 @@ void IntlTestSimpleDateFormatAPI::runIndexedTest( int32_t index, UBool exec, con
                 if (exec) {
                     logln("SimpleDateFormat API test---"); logln("");
                     UErrorCode status = U_ZERO_ERROR;
-                    Locale::setDefault(Locale::ENGLISH, status);
+                    Locale saveLocale;
+                    Locale::setDefault(Locale::getEnglish(), status);
                     if(U_FAILURE(status)) {
                         errln("ERROR: Could not set default locale, test may not give correct results");
                     }
                     testAPI(/*par*/);
+                    Locale::setDefault(saveLocale, status);
                 }
                 break;
 
@@ -48,7 +53,8 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
 
     SimpleDateFormat def(status);
     if(U_FAILURE(status)) {
-        errln("ERROR: Could not create SimpleDateFormat (default)");
+        dataerrln("ERROR: Could not create SimpleDateFormat (default) - exitting");
+        return;
     }
 
     status = U_ZERO_ERROR;
@@ -59,13 +65,13 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
     }
 
     status = U_ZERO_ERROR;
-    SimpleDateFormat pat_fr(pattern, Locale::FRENCH, status);
+    SimpleDateFormat pat_fr(pattern, Locale::getFrench(), status);
     if(U_FAILURE(status)) {
         errln("ERROR: Could not create SimpleDateFormat (pattern French)");
     }
 
     status = U_ZERO_ERROR;
-    DateFormatSymbols *symbols = new DateFormatSymbols(Locale::FRENCH, status);
+    DateFormatSymbols *symbols = new DateFormatSymbols(Locale::getFrench(), status);
     if(U_FAILURE(status)) {
         errln("ERROR: Could not create DateFormatSymbols (French)");
     }
@@ -73,7 +79,8 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
     status = U_ZERO_ERROR;
     SimpleDateFormat cust1(pattern, symbols, status);
     if(U_FAILURE(status)) {
-        errln("ERROR: Could not create SimpleDateFormat (pattern, symbols*)");
+        dataerrln("ERROR: Could not create SimpleDateFormat (pattern, symbols*) - exitting");
+        return;
     }
 
     status = U_ZERO_ERROR;
@@ -145,6 +152,13 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
     logln("Testing getters and setters");
 
     const DateFormatSymbols *syms = pat.getDateFormatSymbols();
+    if(!syms) {
+      errln("Couldn't obtain DateFormatSymbols. Quitting test!");
+      return;
+    }
+    if(syms->getDynamicClassID() != DateFormatSymbols::getStaticClassID()) {
+        errln("ERROR: format->getDateFormatSymbols()->getDynamicClassID() != DateFormatSymbols::getStaticClassID()");
+    }
     DateFormatSymbols *newSyms = new DateFormatSymbols(*syms);
     def.adoptDateFormatSymbols(newSyms);    
     pat_fr.setDateFormatSymbols(*newSyms);
@@ -164,6 +178,13 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
         errln("ERROR: setTwoDigitStartDate() failed");
     }
 
+// ======= Test DateFormatSymbols constructor
+    newSyms  =new DateFormatSymbols("gregorian", status);
+    if(U_FAILURE(status)) {
+        errln("ERROR: new DateFormatSymbols() failed");
+    }
+    def.adoptDateFormatSymbols(newSyms);
+
 // ======= Test applyPattern()
 
     logln("Testing applyPattern()");
@@ -213,3 +234,5 @@ void IntlTestSimpleDateFormatAPI::testAPI(/*char *par*/)
 
     delete test;
 }
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
