@@ -154,6 +154,7 @@ void StringSearchTest::runIndexedTest(int32_t index, UBool exec,
         CASE(33, TestUClassID)
         CASE(34, TestSubclass)
         CASE(35, TestCoverage)
+        CASE(36, TestDiactricMatch)
         default: name = ""; break;
     }
 }
@@ -1544,6 +1545,43 @@ void StringSearchTest::TestIgnorable()
     delete strsrch;
     delete collator;
 }
+
+void StringSearchTest::TestDiactricMatch()
+{
+	UChar temp[128];
+    UErrorCode status = U_ZERO_ERROR;
+    int        count  = 0;
+    RuleBasedCollator* coll = NULL;
+    StringSearch *strsrch = NULL;
+
+    UnicodeString pattern("pattern");
+    UnicodeString text("text");
+    
+    const SearchData *search; 
+    
+    search = &(DIACTRICMATCH[count]);
+    while (search->text != NULL) {
+   		coll = getCollator(search->collator);
+    	coll->setStrength(getECollationStrength(search->strength));
+    	strsrch = new StringSearch(pattern, text, coll, getBreakIterator(search->breaker), status);
+    	if (U_FAILURE(status)) {
+	        errln("Error opening string search %s", u_errorName(status));
+	        return;
+	    }  
+        u_unescape(search->text, temp, 128);
+        text.setTo(temp, u_strlen(temp));
+        u_unescape(search->pattern, temp, 128);
+        pattern.setTo(temp, u_strlen(temp));
+        strsrch->setText(text, status);
+        strsrch->setPattern(pattern, status);
+        if (!assertEqualWithStringSearch(strsrch, search)) {
+            errln("Error at test number %d", count);
+        }
+        search = &(DIACTRICMATCH[++count]);
+        delete strsrch;
+    }
+    
+}
  
 void StringSearchTest::TestCanonical()
 {
