@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -13,6 +13,10 @@
 #ifndef _TRCOLL
 #define _TRCOLL
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "tscoll.h"
 
 class CollationTurkishTest: public IntlTestCollator {
@@ -25,9 +29,6 @@ public:
     virtual ~CollationTurkishTest();
     void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
 
-    // main test routine, tests rules specific to turkish locale
-    void doTest( UnicodeString source, UnicodeString target, Collator::EComparisonResult result);
-
     // perform tests for turkish locale with strength PRIMARY
     void TestPrimary(/* char* par */);
 
@@ -42,4 +43,7 @@ private:
 
     Collator *myCollation;
 };
+
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif
