@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2002-2005, International Business Machines
+*   Copyright (C) 2002-2007, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   file name:  iotest.cpp
@@ -286,7 +286,11 @@ static void U_CALLCONV DataDrivenPrintf(void)
                     log_err("FAILURE string test case %d \"%s\" - Got: \"%s\" Expected: \"%s\"\n",
                             i, cFormat, cBuffer, cExpected);
                 }
-                if (uBuffer[uBufferLenReturned-1] == 0
+                if (uBufferLenReturned <= 0) {
+                    log_err("FAILURE test case %d - \"%s\" is an empty string.\n",
+                            i, cBuffer);
+                }
+                else if (uBuffer[uBufferLenReturned-1] == 0
                     || uBuffer[uBufferLenReturned] != 0
                     || uBuffer[uBufferLenReturned+1] != 0x2A
                     || uBuffer[uBufferLenReturned+2] != 0x2A)
@@ -361,6 +365,7 @@ static void U_CALLCONV DataDrivenScanf(void)
     int32_t i32, expected32;
     int64_t i64, expected64;
     double dbl, expectedDbl;
+    volatile float flt, expectedFlt; // Use volatile in order to get around an Intel compiler issue.
     int32_t uBufferLenReturned;
 
     //const char *fileLocale = "en_US_POSIX";
@@ -410,6 +415,15 @@ static void U_CALLCONV DataDrivenScanf(void)
                                 i, dbl, expectedDbl);
                     }
                     break;
+                case 0x66:  // 'f' float
+                    expectedFlt = (float)atof(u_austrcpy(cBuffer, expectedResult));
+                    uBufferLenReturned = u_sscanf_u(argument, format, &flt);
+                    //uFileBufferLenReturned = u_fscanf_u(testFile, format, flt);
+                    if (flt != expectedFlt) {
+                        log_err("error in scanf test case[%d] Got: %f Exp: %f\n",
+                                i, flt, expectedFlt);
+                    }
+                    break;
                 case 0x31:  // '1' int8_t
                     expected8 = (int8_t)uto64(expectedResult);
                     uBufferLenReturned = u_sscanf_u(argument, format, &i8);
@@ -626,7 +640,11 @@ static void U_CALLCONV DataDrivenPrintfPrecision(void)
                     log_err("FAILURE test case %d \"%s\" - Got: \"%s\" Expected: \"%s\"\n",
                             i, cFormat, cBuffer, cExpected);
                 }
-                if (uBuffer[uBufferLenReturned-1] == 0
+                if (uBufferLenReturned <= 0) {
+                    log_err("FAILURE test case %d - \"%s\" is an empty string.\n",
+                            i, cBuffer);
+                }
+                else if (uBuffer[uBufferLenReturned-1] == 0
                     || uBuffer[uBufferLenReturned] != 0
                     || uBuffer[uBufferLenReturned+1] != 0x2A
                     || uBuffer[uBufferLenReturned+2] != 0x2A)
