@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2001, International Business Machines
+*   Copyright (C) 1999-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -16,12 +16,12 @@
 *   This program reads the Unicode character database text file,
 *   parses it, and extracts the character code,
 *   the "modern" character name, and optionally the
-*   Unicode 1.0 character name.
+*   Unicode 1.0 character name, and (starting with ICU 2.2) the ISO 10646 comment.
 *   It then tokenizes and compresses the names and builds
 *   compact binary tables for random-access lookup
 *   in a u_charName() API function.
 *
-* unames.dat file format (after UDataInfo header etc. - see udata.c)
+* unames.icu file format (after UDataInfo header etc. - see udata.c)
 * (all data is static const)
 *
 * UDataInfo fields:
@@ -53,6 +53,9 @@
 *           tokenString=tokenStrings+token; (tokenStrings=start of names data + tokenStringOffset;)
 *           append zero-terminated tokenString;
 *
+*    Different strings for a code point - normal name, 1.0 name, and ISO comment -
+*    are separated by ';'.
+*
 * uint16_t groupCount;
 * struct {
 *   uint16_t groupMSB; -- for a group of 32 character names stored, this is code point>>5
@@ -116,12 +119,13 @@
 */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include "unicode/utypes.h"
 #include "unicode/putil.h"
+#include "unicode/uclean.h"
+#include "unicode/udata.h"
 #include "cmemory.h"
 #include "cstring.h"
-#include "unicode/udata.h"
+#include "uarrsort.h"
 #include "unewdata.h"
 #include "uoptions.h"
 #include "uparse.h"
@@ -138,13 +142,54 @@
 #define MAX_GROUP_COUNT 5000
 
 #define DATA_NAME "unames"
-#define DATA_TYPE "dat"
+#define DATA_TYPE "icu"
 #define VERSION_STRING "unam"
 #define NAME_SEPARATOR_CHAR ';'
 
+/* Unicode versions --------------------------------------------------------- */
+
+enum {
+    UNI_1_0,
+    UNI_1_1,
+    UNI_2_0,
+    UNI_3_0,
+    UNI_3_1,
+    UNI_3_2,
+    UNI_4_0,
+    UNI_4_0_1,
+    UNI_4_1,
+    UNI_VER_COUNT
+};
+
 static const UVersionInfo
-unicode_3_0={ 3, 0, 0, 0 },
-unicode_3_1={ 3, 1, 0, 0 };
+unicodeVersions[]={
+    { 1, 0, 0, 0 },
+    { 1, 1, 0, 0 },
+    { 2, 0, 0, 0 },
+    { 3, 0, 0, 0 },
+    { 3, 1, 0, 0 },
+    { 3, 2, 0, 0 },
+    { 4, 0, 0, 0 },
+    { 4, 0, 1, 0 },
+    { 4, 1, 0, 0 }
+};
+
+static int32_t ucdVersion=UNI_4_1;
+
+static int32_t
+findUnicodeVersion(const UVersionInfo version) {
+    int32_t i;
+
+    for(i=0; /* while(version>unicodeVersions[i]) {} */
+        i<UNI_VER_COUNT && uprv_memcmp(version, unicodeVersions[i], 4)>0;
+        ++i) {}
+    if(0<i && i<UNI_VER_COUNT && uprv_memcmp(version, unicodeVersions[i], 4)<0) {
+        --i; /* fix 4.0.2 to land before 4.1, for valid x>=ucdVersion comparisons */
+    }
+    return i; /* version>=unicodeVersions[i] && version<unicodeVersions[i+1]; possible: i==UNI_VER_COUNT */
+}
+
+/* generator data ----------------------------------------------------------- */
 
 /* UDataInfo cf. udata.h */
 static UDataInfo dataInfo={
@@ -220,8 +265,8 @@ compressLines(void);
 static int16_t
 compressLine(uint8_t *s, int16_t length, int16_t *pGroupTop);
 
-static int
-compareWords(const void *word1, const void *word2);
+static int32_t
+compareWords(const void *context, const void *word1, const void *word2);
 
 static void
 generateData(const char *dataDir);
@@ -242,7 +287,7 @@ static void
 countWord(Word *word);
 
 static void
-addLine(uint32_t code, char *name1, int16_t name1Length, char *name2, int16_t name2Length);
+addLine(uint32_t code, char *names[], int16_t lengths[], int16_t count);
 
 static void
 addGroup(uint32_t groupMSB, uint8_t *strings, int16_t length);
@@ -257,7 +302,7 @@ static void
 appendLineLengthNibble(uint8_t nibble);
 
 static uint8_t *
-allocLine(uint32_t length);
+allocLine(int32_t length);
 
 static uint8_t *
 allocWord(uint32_t length);
@@ -279,12 +324,25 @@ extern int
 main(int argc, char* argv[]) {
     UVersionInfo version;
     UBool store10Names=FALSE;
+    UErrorCode errorCode = U_ZERO_ERROR;
 
     U_MAIN_INIT_ARGS(argc, argv);
 
+    /* Initialize ICU */
+    u_init(&errorCode);
+    if (U_FAILURE(errorCode) && errorCode != U_FILE_ACCESS_ERROR) {
+        /* Note: u_init() will try to open ICU property data.
+         *       failures here are expected when building ICU from scratch.
+         *       ignore them.
+         */
+        fprintf(stderr, "%s: can not initialize ICU.  errorCode = %s\n",
+            argv[0], u_errorName(errorCode));
+        exit(1);
+    }
+
     /* preset then read command line options */
     options[5].value=u_getDataDirectory();
-    options[6].value="3.1.1";
+    options[6].value="4.1";
     argc=u_parseArgs(argc, argv, sizeof(options)/sizeof(options[0]), options);
 
     /* error handling, printing usage message */
@@ -331,12 +389,14 @@ main(int argc, char* argv[]) {
     /* set the Unicode version */
     u_versionFromString(version, options[6].value);
     uprv_memcpy(dataInfo.dataVersion, version, 4);
+    ucdVersion=findUnicodeVersion(version);
 
     init();
     parseDB(argc>=2 ? argv[1] : "-", store10Names);
     compress();
     generateData(options[5].value);
 
+    u_cleanup();
     return 0;
 }
 
@@ -351,14 +411,30 @@ init() {
 
 /* parsing ------------------------------------------------------------------ */
 
-static void
+/* get a name, strip leading and trailing whitespace */
+static int16_t
+getName(char **pStart, char *limit) {
+    /* strip leading whitespace */
+    char *start=(char *)u_skipWhitespace(*pStart);
+
+    /* strip trailing whitespace */
+    while(start<limit && (*(limit-1)==' ' || *(limit-1)=='\t')) {
+        --limit;
+    }
+
+    /* return results */
+    *pStart=start;
+    return (int16_t)(limit-start);
+}
+
+static void U_CALLCONV
 lineFn(void *context,
        char *fields[][2], int32_t fieldCount,
        UErrorCode *pErrorCode) {
+    char *names[3];
+    int16_t lengths[3];
     static uint32_t prevCode=0;
     uint32_t code=0;
-    char *name1Start, *name2Start;
-    int16_t name1Length, name2Length;
 
     if(U_FAILURE(*pErrorCode)) {
         return;
@@ -367,25 +443,29 @@ lineFn(void *context,
     code=uprv_strtoul(fields[0][0], NULL, 16);
 
     /* get the character name */
-    name1Start=fields[1][0];
-    if(fields[1][0][0]!='<') {
-        name1Length=(int16_t)(fields[1][1]-name1Start);
-    } else {
+    names[0]=fields[1][0];
+    lengths[0]=getName(names+0, fields[1][1]);
+    if(names[0][0]=='<') {
         /* do not store pseudo-names in <> brackets */
-        name1Length=0;
+        lengths[0]=0;
     }
 
     /* store 1.0 names */
     /* get the second character name, the one from Unicode 1.0 */
     /* do not store pseudo-names in <> brackets */
-    name2Start=fields[10][0];
-    if(*(UBool *)context && fields[10][0][0]!='<') {
-        name2Length=(int16_t)(fields[10][1]-name2Start);
+    names[1]=fields[10][0];
+    lengths[1]=getName(names+1, fields[10][1]);
+    if(*(UBool *)context && names[1][0]!='<') {
+        /* keep the name */
     } else {
-        name2Length=0;
+        lengths[1]=0;
     }
 
-    if(name1Length+name2Length==0) {
+    /* get the ISO 10646 comment */
+    names[2]=fields[11][0];
+    lengths[2]=getName(names+2, fields[11][1]);
+
+    if(lengths[0]+lengths[1]+lengths[2]==0) {
         return;
     }
 
@@ -406,22 +486,27 @@ lineFn(void *context,
     }
     prevCode=code;
 
-    /* printf("%lx:%.*s(%.*s)\n", code, name1Length, line+name1Start, name2Length, line+name2Start); */
+    parseName(names[0], lengths[0]);
+    parseName(names[1], lengths[1]);
+    parseName(names[2], lengths[2]);
 
-    parseName(name1Start, name1Length);
-    parseName(name2Start, name2Length);
-
-    addLine(code, name1Start, name1Length, name2Start, name2Length);
+    /*
+     * set the count argument to
+     * 1: only store regular names
+     * 2: store regular and 1.0 names
+     * 3: store names and ISO 10646 comment
+     */
+    addLine(code, names, lengths, 3);
 }
 
 static void
 parseDB(const char *filename, UBool store10Names) {
-    char *fields[11][2];
+    char *fields[15][2];
     UErrorCode errorCode=U_ZERO_ERROR;
 
-    /* parsing the 11 fields 0..10 is enough for gennames */
-    u_parseDelimitedFile(filename, ';', fields, 11, lineFn, &store10Names, &errorCode);
+    u_parseDelimitedFile(filename, ';', fields, 15, lineFn, &store10Names, &errorCode);
     if(U_FAILURE(errorCode)) {
+        fprintf(stderr, "gennames parse error: %s\n", u_errorName(errorCode));
         exit(errorCode);
     }
 
@@ -482,17 +567,23 @@ parseName(char *name, int16_t length) {
     }
 }
 
+static UBool U_INLINE
+isWordChar(char c) {
+    return ('A'<=c && c<='I') || /* EBCDIC-safe check for letters */
+           ('J'<=c && c<='R') ||
+           ('S'<=c && c<='Z') ||
+
+           ('a'<=c && c<='i') || /* lowercase letters for ISO comments */
+           ('j'<=c && c<='r') ||
+           ('s'<=c && c<='z') ||
+
+           ('0'<=c && c<='9');
+}
+
 static int16_t
 skipNoise(char *line, int16_t start, int16_t limit) {
-    char c;
-
     /* skip anything that is not part of a word in this sense */
-    while(start<limit &&
-          !(('A'<=(c=line[start]) && c<='I') || /* EBCDIC-safe check for letters */
-            ('J'<=c && c<='R') ||
-            ('S'<=c && c<='Z') ||
-            ('0'<=c && c<='9'))
-    ) {
+    while(start<limit && !isWordChar(line[start])) {
         ++start;
     }
 
@@ -504,17 +595,12 @@ getWord(char *line, int16_t start, int16_t limit) {
     char c=0; /* initialize to avoid a compiler warning although the code was safe */
 
     /* a unicode character name word consists of A-Z0-9 */
-    while(start<limit &&
-          (('A'<=(c=line[start]) && c<='I') || /* EBCDIC-safe check for letters */
-           ('J'<=c && c<='R') ||
-           ('S'<=c && c<='Z') ||
-           ('0'<=c && c<='9'))
-    ) {
+    while(start<limit && isWordChar(line[start])) {
         ++start;
     }
 
     /* include a following space or dash */
-    if(start<limit && (c==' ' || c=='-')) {
+    if(start<limit && ((c=line[start])==' ' || c=='-')) {
         ++start;
     }
 
@@ -527,9 +613,12 @@ static void
 compress() {
     uint32_t i, letterCount;
     int16_t wordNumber;
+    UErrorCode errorCode;
 
     /* sort the words in reverse order by weight */
-    qsort(words, wordCount, sizeof(Word), compareWords);
+    errorCode=U_ZERO_ERROR;
+    uprv_sortArray(words, wordCount, sizeof(Word),
+                    compareWords, NULL, FALSE, &errorCode);
 
     /* remove the words that do not save anything */
     while(wordCount>0 && words[wordCount-1].weight<1) {
@@ -544,7 +633,7 @@ compress() {
         }
     }
     if(!beQuiet) {
-        printf("number of letters used in the names: %d\n", letterCount);
+        printf("number of letters used in the names: %d\n", (int)letterCount);
     }
 
     /* do we need double-byte tokens? */
@@ -556,7 +645,7 @@ compress() {
                 tokens[i]=wordNumber;
                 if(beVerbose) {
                     printf("tokens[0x%03x]: word%8ld \"%.*s\"\n",
-                            i, (long)words[wordNumber].weight,
+                            (int)i, (long)words[wordNumber].weight,
                             words[wordNumber].length, words[wordNumber].s);
                 }
                 ++wordNumber;
@@ -575,7 +664,9 @@ compress() {
         }
 
         /* sort these words in reverse order by weight */
-        qsort(words+tokenCount, wordCount-tokenCount, sizeof(Word), compareWords);
+        errorCode=U_ZERO_ERROR;
+        uprv_sortArray(words+tokenCount, wordCount-tokenCount, sizeof(Word),
+                        compareWords, NULL, FALSE, &errorCode);
 
         /* remove the words that do not save anything */
         while(wordCount>0 && words[wordCount-1].weight<1) {
@@ -625,7 +716,7 @@ compress() {
                 tokens[i]=wordNumber;
                 if(beVerbose) {
                     printf("tokens[0x%03x]: word%8ld \"%.*s\"\n",
-                            i, (long)words[wordNumber].weight,
+                            (int)i, (long)words[wordNumber].weight,
                             words[wordNumber].length, words[wordNumber].s);
                 }
                 ++wordNumber;
@@ -640,7 +731,7 @@ compress() {
                 tokens[i]=wordNumber;
                 if(beVerbose) {
                     printf("tokens[0x%03x]: word%8ld \"%.*s\"\n",
-                            i, (long)words[wordNumber].weight,
+                            (int)i, (long)words[wordNumber].weight,
                             words[wordNumber].length, words[wordNumber].s);
                 }
                 ++wordNumber;
@@ -779,8 +870,8 @@ compressLine(uint8_t *s, int16_t length, int16_t *pGroupTop) {
     return length;
 }
 
-static int
-compareWords(const void *word1, const void *word2) {
+static int32_t
+compareWords(const void *context, const void *word1, const void *word2) {
     /* reverse sort by word weight */
     return ((Word *)word2)->weight-((Word *)word1)->weight;
 }
@@ -797,7 +888,7 @@ generateData(const char *dataDir) {
     long dataLength;
     int16_t token;
 
-    pData=udata_create(dataDir, DATA_TYPE, DATA_NAME, &dataInfo,
+    pData=udata_create(dataDir, DATA_TYPE,DATA_NAME, &dataInfo,
                        haveCopyright ? U_COPYRIGHT_STRING : NULL, &errorCode);
     if(U_FAILURE(errorCode)) {
         fprintf(stderr, "gennames: unable to create data memory, error %d\n", errorCode);
@@ -812,6 +903,42 @@ generateData(const char *dataDir) {
         }
     }
 
+    /*
+     * Required padding for data swapping:
+     * The token table undergoes a permutation during data swapping when the
+     * input and output charsets are different.
+     * The token table cannot grow during swapping, so we need to make sure that
+     * the table is long enough for successful in-place permutation.
+     *
+     * We simply round up tokenCount to the next multiple of 256 to account for
+     * all possible permutations.
+     *
+     * An optimization is possible if we only ever swap between ASCII and EBCDIC:
+     *
+     * If tokenCount>256, then a semicolon (NAME_SEPARATOR_CHAR) is used
+     * and will be swapped between ASCII and EBCDIC between
+     * positions 0x3b (ASCII semicolon) and 0x5e (EBCDIC semicolon).
+     * This should be the only -1 entry in tokens[256..511] on which the data
+     * swapper bases its trail byte permutation map (trailMap[]).
+     *
+     * It would be sufficient to increase tokenCount so that its lower 8 bits
+     * are at least 0x5e+1 to make room for swapping between the two semicolons.
+     * For values higher than 0x5e, the trail byte permutation map (trailMap[])
+     * should always be an identity map, where we do not need additional room.
+     */
+    i=tokenCount;
+    tokenCount=(tokenCount+0xff)&~0xff;
+    if(!beQuiet && i<tokenCount) {
+        printf("number of tokens[] padding entries for data swapping: %lu\n", (unsigned long)(tokenCount-i));
+    }
+    for(; i<tokenCount; ++i) {
+        if((i&0xff)==NAME_SEPARATOR_CHAR) {
+            tokens[i]=-1; /* do not use NAME_SEPARATOR_CHAR as a second token byte */
+        } else {
+            tokens[i]=0; /* unused token for padding */
+        }
+    }
+
     /*
      * Calculate the total size in bytes of the data including:
      * - the offset to the token strings, uint32_t (4)
@@ -871,7 +998,7 @@ generateData(const char *dataDir) {
         groupWords[0]=(uint16_t)lines[i].code;
 
         /* offset */
-        offset=lines[i].s-stringStore;
+        offset = (uint32_t)(lines[i].s - stringStore);
         groupWords[1]=(uint16_t)(offset>>16);
         groupWords[2]=(uint16_t)(offset);
         udata_writeBlock(pData, groupWords, 6);
@@ -960,11 +1087,16 @@ generateAlgorithmicData(UNewDataMemory *pData) {
 
     size=0;
 
+    if(ucdVersion>=UNI_4_1) {
+        /* Unicode 4.1 and up has a longer CJK Unihan range than before */
+        cjk.rangeEnd=0x9FBB;
+    }
+
     /* number of ranges of algorithmic names */
-    if(uprv_memcmp(dataInfo.dataVersion, unicode_3_1, sizeof(UVersionInfo))>=0) {
+    if(ucdVersion>=UNI_3_1) {
         /* Unicode 3.1 and up has 4 ranges including CJK Extension B */
         countAlgRanges=4;
-    } else if(uprv_memcmp(dataInfo.dataVersion, unicode_3_0, sizeof(UVersionInfo))>=0) {
+    } else if(ucdVersion>=UNI_3_0) {
         /* Unicode 3.0 has 3 ranges including CJK Extension A */
         countAlgRanges=3;
     } else {
@@ -1045,7 +1177,7 @@ findToken(uint8_t *s, int16_t length) {
 
     for(i=0; i<(int16_t)tokenCount; ++i) {
         token=tokens[i];
-        if(token!=-1 && length==words[token].length && 0==uprv_memcmp(s, words[token].s, length)) {
+        if(token>=0 && length==words[token].length && 0==uprv_memcmp(s, words[token].s, length)) {
             return i;
         }
     }
@@ -1103,28 +1235,46 @@ countWord(Word *word) {
 }
 
 static void
-addLine(uint32_t code, char *name1, int16_t name1Length, char *name2, int16_t name2Length) {
+addLine(uint32_t code, char *names[], int16_t lengths[], int16_t count) {
     uint8_t *stringStart;
     Line *line;
-    int16_t length;
+    int16_t i, length;
 
     if(lineCount==MAX_LINE_COUNT) {
         fprintf(stderr, "gennames: too many lines\n");
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
 
-    length=name1Length;
-    if(name2Length>0) {
-        length=(int16_t)(length+1+name2Length);
+    /* find the last non-empty name */
+    while(count>0 && lengths[count-1]==0) {
+        --count;
+    }
+    if(count==0) {
+        return; /* should not occur: caller should not have called */
     }
 
-    stringStart=allocLine(length);
-    if(name1Length>0) {
-        uprv_memcpy(stringStart, name1, name1Length);
+    /* there will be (count-1) separator characters */
+    i=count;
+    length=count-1;
+
+    /* add lengths of strings */
+    while(i>0) {
+        length+=lengths[--i];
     }
-    if(name2Length>0) {
-        stringStart[name1Length]=NAME_SEPARATOR_CHAR;
-        uprv_memcpy(stringStart+name1Length+1, name2, name2Length);
+
+    /* allocate line memory */
+    stringStart=allocLine(length);
+
+    /* copy all strings into the line memory */
+    length=0; /* number of chars copied so far */
+    for(i=0; i<count; ++i) {
+        if(i>0) {
+            stringStart[length++]=NAME_SEPARATOR_CHAR;
+        }
+        if(lengths[i]>0) {
+            uprv_memcpy(stringStart+length, names[i], lengths[i]);
+            length+=lengths[i];
+        }
     }
 
     line=lines+lineCount;
@@ -1174,7 +1324,7 @@ addToken(uint8_t *s, int16_t length) {
     uprv_memcpy(stringStart, s, length);
     stringStart[length]=0;
 
-    return stringStart-stringStore;
+    return (uint32_t)(stringStart - stringStore);
 }
 
 static void
@@ -1201,7 +1351,7 @@ appendLineLengthNibble(uint8_t nibble) {
 }
 
 static uint8_t *
-allocLine(uint32_t length) {
+allocLine(int32_t length) {
     uint32_t top=lineTop+length;
     uint8_t *p;
 
