@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2002, International Business Machines
+*   Copyright (C) 1999-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -22,46 +22,32 @@
 
 /* file definitions */
 #define DATA_NAME "uprops"
-#define DATA_TYPE "dat"
-
-/* special casing data */
-typedef struct {
-    uint32_t code;
-    UBool isComplex;
-    UChar lowerCase[32], upperCase[32], titleCase[32];
-} SpecialCasing;
-
-/* case folding data */
-typedef struct {
-    uint32_t code, simple;
-    char status;
-    UChar full[32];
-} CaseFolding;
+#define DATA_TYPE "icu"
 
 /* character properties */
 typedef struct {
-    uint32_t code, lowerCase, upperCase, titleCase, mirrorMapping;
+    uint32_t code;
     int32_t numericValue; /* see numericType */
     uint32_t denominator; /* 0: no value */
-    uint8_t generalCategory, bidi, isMirrored, numericType;
-    SpecialCasing *specialCasing;
-    CaseFolding *caseFolding;
+    uint8_t generalCategory, numericType, exponent;
 } Props;
 
 /* global flags */
 extern UBool beVerbose, haveCopyright;
 
-/* name tables */
-extern const char *const
-bidiNames[];
-
 extern const char *const
 genCategoryNames[];
 
+/* properties vectors in props2.c */
+extern uint32_t *pv;
+
 /* prototypes */
 U_CFUNC void
 writeUCDFilename(char *basename, const char *filename, const char *suffix);
 
+U_CFUNC UBool
+isToken(const char *token, const char *s);
+
 U_CFUNC int32_t
 getTokenIndex(const char *const tokens[], int32_t countTokens, const char *s);
 
@@ -71,33 +57,35 @@ setUnicodeVersion(const char *v);
 extern void
 initStore(void);
 
+extern void
+exitStore(void);
+
 extern uint32_t
 makeProps(Props *p);
 
 extern void
 addProps(uint32_t c, uint32_t props);
 
+extern uint32_t
+getProps(uint32_t c);
+
 extern void
 repeatProps(uint32_t first, uint32_t last, uint32_t props);
 
-U_CAPI uint32_t U_EXPORT2
-getFoldedPropsValue(UNewTrie *trie, UChar32 start, int32_t offset);
-
 extern void
-generateData(const char *dataDir);
+generateData(const char *dataDir, UBool csource);
 
 /* props2.c */
 U_CFUNC void
 initAdditionalProperties(void);
 
 U_CFUNC void
-setMainProperties(uint32_t start, uint32_t limit, uint32_t value);
+exitAdditionalProperties(void);
 
 U_CFUNC void
 generateAdditionalProperties(char *filename, const char *suffix, UErrorCode *pErrorCode);
 
 U_CFUNC int32_t
-writeAdditionalData(uint8_t *p, int32_t capacity, int32_t indexes[16]);
+writeAdditionalData(FILE *f, uint8_t *p, int32_t capacity, int32_t indexes[16]);
 
 #endif
-
